/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/Acknowledgement.java-arc   1.4   Apr 08 2005 17:46:38   adij  $ 
 * $Revision:   1.4  $ 
 * $Date:   Apr 08 2005 17:46:38  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved. 
 * NEMMCO CONFIDENTIAL.
 * Date Changed   Changed By        Comments
 * ------------   ----------        -----------
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.capgemini.nemmco.b2bb.enums.Status;
import com.capgemini.nemmco.b2bb.enums.XMLObjectType;
import com.capgemini.nemmco.b2bb.enums.YesNo;
import com.capgemini.nemmco.b2bb.exceptions.UnableToParseXMLException;
import com.capgemini.nemmco.b2bb.utils.B2BBDOMHelper;
import com.capgemini.nemmco.b2bb.utils.B2BDateTimeFormatter;
import com.capgemini.nemmco.b2bb.utils.EventObject;


/**
 * Defines a header element.
 *
 * @author $Author:   adij  $
 * @version $Revision:   1.4  $ $Date:   Apr 08 2005 17:46:38  $
 */
public abstract class Acknowledgement extends XMLObject {
  public static final String RECEIPT_DATE = "receiptDate";
  public static final String RECEIPT_ID = "receiptID";
  public static final String STATUS = "status";
  public static final String DUPLICATE = "duplicate";
  public static final String EVENTS = "events";
  
  private String duplicate = "";
  private String receiptDate = "";
  private boolean receiptDateSet;
  private String receiptDisplayDate = "";
  private String receiptID = "";
  private String status = "";
  private List<Object> events = new ArrayList<Object>();
  
  /**
   * @param xmlObjectType
   */
  public Acknowledgement(XMLObjectType xmlObjectType) {
    super(xmlObjectType);
  }
  
  /**
   * @return Returns the events.
   */
  public List<Object> getEvents() {
    return events;
  }
  
  /**
   * @param events The events to set.
   */
  public void setEvents(List<Object> events) {
    this.events = events;
  }
  
  /**
   * @param event The event to set.
   */
  public void setEvent(EventObject event) {
    List<Object> events = new ArrayList<Object>();
    events.add(event);
    
    this.events = events;
  }
  
  /**
   * @return Returns the duplicate.
   */
  public String getDuplicate() {
    return duplicate;
  }

  /**
   * @return Returns the receiptDate.
   */
  public String getReceiptDate() {
    return receiptDate;
  }
  
  /**
   * @return Returns the receiptDisplayDate.
   */
  public String getReceiptDisplayDate() {
    return receiptDisplayDate;
  }
  
  /**
   * @return Returns the receiptID.
   */
  public String getReceiptID() {
    return receiptID;
  }
  
  /**
   * @return Returns the status.
   */
  public String getStatus() {
    return status;
  }
  
  /**
   * @return Returns the status list 
   */
  public List<? extends Object> getStatuses() {
    return Status.STATUSES;
  }
  
  /**
   * @return Returns the yes no 
   */
  public List<? extends Object> getYesNo() {
    return YesNo.YESNO_CODES;
  }
  
  /**
   * @return Returns the receiptDateSet.
   */
  public boolean isReceiptDateSet() {
    return receiptDateSet;
  }
  
  /**
   * @param duplicate The duplicate to set.
   */
  public void setDuplicate(String duplicate) {
    this.duplicate = duplicate;
  }
  
  /**
   * @param receiptDate The receiptDate to set.
   */
  public void setReceiptDate(String receiptDate) {
    this.receiptDateSet = true;
    this.receiptDate = receiptDate;
    try {
      B2BDateTimeFormatter df = new B2BDateTimeFormatter(receiptDate, B2BDateTimeFormatter.XMLDATE);
      this.receiptDisplayDate = df.getB2BFormattedDateTime();
    } catch (Exception e) {
      this.receiptDisplayDate = "";
    }
  }
  
  /**
   * @param receiptDateSet The receiptDateSet to set.
   */
  public void setReceiptDateSet(boolean receiptDateSet) {
    this.receiptDateSet = receiptDateSet;
  }
  
  /**
   * @param receiptDisplayDate The receiptDisplayDate to set.
   */
  public void setReceiptDisplayDate(String receiptDisplayDate) {
    this.receiptDateSet = false;
    this.receiptDisplayDate = receiptDisplayDate;
    try {
      B2BDateTimeFormatter df = new B2BDateTimeFormatter(receiptDisplayDate, B2BDateTimeFormatter.B2BDATE);
      this.receiptDate = df.getXMLFormattedDateTime();
    } catch (Exception e) {
      this.receiptDate = "";
    }
  }
  
  /**
   * @see com.capgemini.nemmco.b2bb.XMLObject#createXML()
   */
  public Node createXML() throws Exception {
    Node acknowledgementNode = null;
    Node eventNode = null;
    EventObject event;
    
    // create ack node
    acknowledgementNode = super.createXML();
    
    // create event nodes if available
    if (events != null && !events.isEmpty()) {   
      Iterator<Object> iter = events.iterator();
      while (iter.hasNext()) {
        event = (EventObject) iter.next();
        
        if (event != null) {
          eventNode = event.createXML();
          
          // append transaction to transactions
          B2BBDOMHelper.appendNode(acknowledgementNode, eventNode);
        }
      }
    }
    
    return acknowledgementNode;
  }
  
  /**
   * @see com.capgemini.nemmco.b2bb.XMLObject#setXMLString(org.w3c.dom.Node)
   */
  public void setXMLString(Node xmlNode) throws UnableToParseXMLException {
    try {
      super.setXMLString(xmlNode);
      
      // now grab all the events if any
      Node eventNode;
      NodeList eventsNodeList;
      EventObject event;
      
      eventsNodeList = B2BBDOMHelper.getSelectedNodes(xmlNode, "Event");
      if (eventsNodeList != null) {
        for (int i = 0; i < eventsNodeList.getLength(); i++) {
          eventNode = eventsNodeList.item(i);
          event = null;
          if (eventNode != null) {
            event = new EventObject();
            event.setXMLString(eventNode);
            events.add(event);
          }
        }
      }
    } catch (Exception e) {
      throw new UnableToParseXMLException(e.getMessage());
    }
  }
  
  /**
   * @param receiptID The receiptID to set.
   */
  public void setReceiptID(String receiptID) {
    this.receiptID = receiptID;
  }
  
  /**
   * @param status The status to set.
   */
  public void setStatus(String status) {
    this.status = status;
  }
}

