/*
 * NoticeOfMeteringWorks.java
 *
 * The Copyright of this work is vested in the Australian Energy Market
 * Operator (AEMO) and the software is issued in confidence for the purpose
 * only for which it is supplied. It must not be reproduced in whole or in
 * part or used for tendering or manufacturing purposes except under an
 * agreement or with the consent in writing of AEMO and then only on the
 * condition that this notice is included in any such reproduction.
 * No information as to the contents or subject matter of this software or
 * any part thereof arising directly or indirectly there from shall be
 * given orally or in writing or communicated in any manner whatsoever to
 * any third party being an individual firm or company or any employee
 * thereof without the prior consent in writing of AEMO.
 *
 * Copyright Australian Energy Market Operator Limited. 2017
 *
 * Modification History
 *
 * Date Changed   Changed By       Description
 * ------------   ----------       -----------
 * 11-Jul-2017    SP                Created as part of the SMP new screens
 */
package com.capgemini.nemmco.b2bb;

import com.capgemini.nemmco.b2bb.enums.ControlEquipmentType;
import com.capgemini.nemmco.b2bb.enums.CustomerClassCode;
import com.capgemini.nemmco.b2bb.enums.CustomerType;
import com.capgemini.nemmco.b2bb.enums.EnergisationStatus;
import com.capgemini.nemmco.b2bb.enums.GenerationType;
import com.capgemini.nemmco.b2bb.enums.NetWorkDeviceLocation;
import com.capgemini.nemmco.b2bb.enums.RemovedEquipmentType;
import com.capgemini.nemmco.b2bb.enums.SORRole;
import com.capgemini.nemmco.b2bb.enums.SupplyPhase;
import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.enums.TransformerType;
import com.capgemini.nemmco.b2bb.enums.Voltage;
import com.capgemini.nemmco.b2bb.enums.WorkType;
import com.capgemini.nemmco.b2bb.enums.YesNo;
import com.capgemini.nemmco.b2bb.utils.B2BDateTimeFormatter;

import java.text.ParseException;

import java.util.ArrayList;
import java.util.List;

public class NoticeOfMeteringWorks extends Transaction {

   public static final String NOMWID ="noMWID";
   public static final String WORK_TYPE = "workType";
   public static final String FIELD_WORK_DATE ="fieldworkDate";
   public static final String FIELD_WORK_TIME = "fieldworkTime";
   public static final String CUST_CLASS_CODE ="custClassCode";
   public static final String PARTICPANT_ID = "partId";
   public static final String LATITUDE ="latitude";
   public static final String LONGITUDE = "longitude";
   public static final String ENERGISATION_STATUS ="energStatus";
   public static final String PRIMARY_VOLTAGE ="primVol";
   public static final String TOTAL_INSTALLED_METERS ="totInstalledMeters";
   public static final String METER_SERIAL_NUMBER1 ="meterSerialNumber1";
   public static final String SUPPLY_PHASE1 ="supPhase1";
   public static final String GENERAL_SUPPLY1 ="genSup1";
   public static final String CONTROL_LOAD1 ="conLod1";
   public static final String GENERATION_TYPE1 ="genType1";
   public static final String METER_SERIAL_NUMBER2 ="meterSerialNumber2";
   public static final String SUPPLY_PHASE2 ="supPhase2";
   public static final String GENERAL_SUPPLY2 ="genSup2";
   public static final String CONTROL_LOAD2 ="conLod2";
   public static final String GENERATION_TYPE2 ="genType2";
   public static final String METER_SERIAL_NUMBER3 ="meterSerialNumber3";
   public static final String SUPPLY_PHASE3 ="supPhase3";
   public static final String GENERAL_SUPPLY3 ="genSup3";
   public static final String CONTROL_LOAD3 ="conLod3";
   public static final String GENERATION_TYPE3 ="genType3";
   public static final String TOTAL_NETWORK_DEVICES ="totNetworkDevices";
   public static final String NETWORK_DEVICE_LOC1 ="networkDevLoc1";
   public static final String NETWORK_DEVICE_NUMBER1 ="networkDevNumber1";
   public static final String NETWORK_DEVICE_LOC2 ="networkDevLoc2";
   public static final String NETWORK_DEVICE_NUMBER2 ="networkDevNumber2";
   public static final String NETWORK_DEVICE_LOC3 ="networkDevLoc3";
   public static final String NETWORK_DEVICE_NUMBER3 ="networkDevNumber3";
   public static final String CONTROL_EQUIPMENT_NUMBER1="conEquipNumber1";
   public static final String CONTROL_EQUIPMENT_TYPE1 ="conEquipType1";
   public static final String CONTROL_CHANNEL1 ="conChannel1";
   public static final String CONTROL_CONNEC_METER1 ="conMeterNum1";
   public static final String CONTROL_EQUIPMENT_NUMBER2="conEquipNumber2";
   public static final String CONTROL_EQUIPMENT_TYPE2 ="conEquipType2";
   public static final String CONTROL_CHANNEL2 ="conChannel2";
   public static final String CONTROL_CONNEC_METER2 ="conMeterNum2";

   public static final String TRANSFORMER_NUMBER1="transNumber1";
   public static final String TRANSFORMER_TYPE1 ="transType1";
   public static final String TRANSFORMER_RATIO1 ="transRatio1";
   public static final String TRANSFORMER_CONN_NUMBER1 ="transConNumber1";
   public static final String TRANSFORMER_NUMBER2="transNumber2";
   public static final String TRANSFORMER_TYPE2 ="transType2";
   public static final String TRANSFORMER_RATIO2 ="transRatio2";
   public static final String TRANSFORMER_CONN_NUMBER2 ="transConNumber2";

   public static final String TOTAL_REMOVED_METERS="removedMeter";
   public static final String TOTAL_REMOVED_OTHERS ="removedOther";
   public static final String REMOVED_EQUIPMENT_NUMBER1 ="removedEquipNumber1";
   public static final String REMOVED_EQUIPMENT_TYPE1 ="removedEquipType1";
   public static final String REMOVED_REGISTER1="removedReg1";
   public static final String REMOVED_METER_READING1 ="removedMeterRead1";
   public static final String REMOVED_EQUIPMENT_NUMBER2 ="removedEquipNumber2";
   public static final String REMOVED_EQUIPMENT_TYPE2 ="removedEquipType2";
   public static final String REMOVED_REGISTER2="removedReg2";
   public static final String REMOVED_METER_READING2 ="removedMeterRead2";
   public static final String NOTES ="notes";
   public static final String WORK_DATE_TIME_FIELD = "workDateTime";
   public static final String PART_ROLE = "partRole";

   private  String noMWID="";
   private  String workType ="";
   private  String fieldworkDate="";
   private  String fieldworkTime ="";
   private  String custClassCode="";
   private  String partId="";
   private  String latitude="";
   private  String longitude="";
   private  String energStatus="";
   private  String primVol ="";
   private  String totInstalledMeters ="";
   private  String meterSerialNumber1 ="";
   private  String supPhase1 ="";
   private  String genSup1 ="";

   private  String genType1 ="";
   private  String meterSerialNumber2 ="";
   private  String supPhase2 ="";
   private  String genSup2 ="";

   private  String genType2 ="";
   private  String meterSerialNumber3 ="";
   private  String supPhase3 ="";
   private  String genSup3 ="";

   private  String genType3 ="";
   private String totNetworkDevices="";
   private String networkDevLoc1="";
   private String networkDevNumber1="";
   private String  networkDevLoc2="";
   private String  networkDevNumber2="";
   private String  networkDevLoc3="";
   private String  networkDevNumber3="";
   private String conEquipNumber1="";
   private String conEquipType1="";
   private String conChannel1="";
   private String conMeterNum1="";
   private String conEquipNumber2="";
   private String conEquipType2="";
   private String conChannel2="";
   private String conMeterNum2="";

   private String transNumber1="";
   private String transType1="";
   private String transRatio1="";
   private String transConNumber1="";
   private String transNumber2="";
   private String transType2="";
   private String transRatio2="";
   private String transConNumber2="";

  private String removedMeter="";
  private String removedOther="";
   private String removedEquipNumber1="";
   private String removedEquipType1="";
   private String removedMeterRead1="";
   private String removedReg1="";
   private String removedEquipNumber2="";
   private String removedEquipType2="";
   private String removedMeterRead2="";
   private String removedReg2="";
   private String notes="";
   private String workDateTime="";
   private String conLod1="";
   private String conLod2="";
   private String conLod3="";
   private String partRole="";


   public NoticeOfMeteringWorks() {
      super(TransactionType.NOTICE_OF_METERING_WORKS);
	  //Default the role to MPB for the MSATS screen.
	  //For the file it must be set otherwise it's schema invalid - so no harm defaulting here
	  setPartRole(SORRole.MPB.getCode());
   }

   public void setNoMWID(String noMWID) {
      this.noMWID = noMWID;
   }

   public String getNoMWID() {
      return noMWID;
   }

   public void setWorkType(String workType) {
      this.workType = workType;
   }

   public String getWorkType() {
      return workType;
   }

   public void setFieldworkDate(String fieldworkDate) {
      this.fieldworkDate = fieldworkDate;
   }

   public String getFieldworkDate() {
      return fieldworkDate;
   }

   public void setFieldworkTime(String fieldworkTime) {
      this.fieldworkTime = fieldworkTime;
   }

   public String getFieldworkTime() {
      return fieldworkTime;
   }

   public void setCustClassCode(String custClassCode) {
      this.custClassCode = custClassCode;
   }

   public String getCustClassCode() {
      return custClassCode;
   }

   public void setPartId(String partId) {
      this.partId = partId;
   }

   public void setPartRole(String partRole) {
      this.partRole = partRole;
   }

   public String getPartId() {
      return partId;
   }



   public void setLongitude(String longitude) {
      this.longitude = longitude;
   }

   public String getLongitude() {
      return longitude;
   }

   public void setEnergStatus(String energStatus) {
      this.energStatus = energStatus;
   }

   public String getEnergStatus() {
      return energStatus;
   }

   public void setPrimVol(String primVol) {
      this.primVol = primVol;
   }

   public String getPrimVol() {
      return primVol;
   }

   /**
    * Getter for customer types
    * @return List
    */
   public List<? extends Object> getCustomerTypes() {
      return CustomerClassCode.CUSTOMER_CLASS_CODES;
   }

   /**
    * Getter for work  types
    * @return List
    */
   public List<? extends Object> getWorkTypes() {

      return WorkType.WORK_TYPES;
   }
   /**
    * Getter for energisation status
    * @return List
    */
   public List<? extends Object> getStatuses() {

      return EnergisationStatus.ENERGISATION_STATUSES;
   }

   /**
    * Getter for work  types
    * @return List
    */
   public List<? extends Object> getVoltages() {
      return Voltage.VOLTAGES;
   }

   public void setLatitude(String latitude) {
      this.latitude = latitude;
   }

   public String getLatitude() {
      return latitude;
   }

   public void setTotInstalledMeters(String totInstalledMeters) {
      this.totInstalledMeters = totInstalledMeters;
   }

   public String getTotInstalledMeters() {
      return totInstalledMeters;
   }

   public void setMeterSerialNumber1(String meterSerialNumber1) {
      this.meterSerialNumber1 = meterSerialNumber1;
   }

   public String getMeterSerialNumber1() {
      return meterSerialNumber1;
   }

   public void setSupPhase1(String supPhase1) {
      this.supPhase1 = supPhase1;
   }

   public String getSupPhase1() {
      return supPhase1;
   }

   public void setGenSup1(String genSup1) {
      this.genSup1 = genSup1;
   }

   public String getGenSup1() {
      return genSup1;
   }



   public void setGenType1(String genType1) {
      this.genType1 = genType1;
   }

   public String getGenType1() {
      return genType1;
   }

   public void setMeterSerialNumber2(String meterSerialNumber2) {
      this.meterSerialNumber2 = meterSerialNumber2;
   }

   public String getMeterSerialNumber2() {
      return meterSerialNumber2;
   }

   public void setSupPhase2(String supPhase2) {
      this.supPhase2 = supPhase2;
   }

   public String getSupPhase2() {
      return supPhase2;
   }

   public void setGenSup2(String genSup2) {
      this.genSup2 = genSup2;
   }

   public String getGenSup2() {
      return genSup2;
   }



   public void setGenType2(String genType2) {
      this.genType2 = genType2;
   }

   public String getGenType2() {
      return genType2;
   }

   public void setMeterSerialNumber3(String meterSerialNumber3) {
      this.meterSerialNumber3 = meterSerialNumber3;
   }

   public String getMeterSerialNumber3() {
      return meterSerialNumber3;
   }

   public void setSupPhase3(String supPhase3) {
      this.supPhase3 = supPhase3;
   }

   public String getSupPhase3() {
      return supPhase3;
   }

   public void setGenSup3(String genSup3) {
      this.genSup3 = genSup3;
   }

   public String getGenSup3() {
      return genSup3;
   }



   public void setGenType3(String genType3) {
      this.genType3 = genType3;
   }

   public String getGenType3() {
      return genType3;
   }

   /**
    * Getter for Control Loads
    * @return List
    */
   public List<? extends Object> getConLods() {

      return YesNo.YESNO_CODES;

   }

   /**
    * Getter for  Supply Phases
    * @return List
    */
   public List<? extends Object> getSupPhases() {
      List<Object> phases = new ArrayList<Object>();
      for(SupplyPhase supPhase:(List<SupplyPhase>)(SupplyPhase.SUPPLY_PHASE_TYPES)){
         if(!SupplyPhase.UNKNOWN.equals(supPhase)){
            phases.add(supPhase);
         }
      }

      return phases;



   }
   /**
    * Getter for General Supply
    * @return List
    */
   public List<? extends Object> getGenTypes() {

      return GenerationType.GENERATION_TYPES;
   }


   /**
    * Getter for General Supply
    * @return List
    */
   public List<? extends Object> getGenSupplies() {

      return YesNo.YESNO_CODES;
   }

   /**
    * Getter for Network Device Locations
    * @return List
    */
   public List<? extends Object> getNetworkDevLocs() {

      return NetWorkDeviceLocation.NETWORK_DEVICES;
   }

   /**
    * Getter for Control Equipment Types
    * @return List
    */
   public List<? extends Object> getConEquipTypes() {

      return ControlEquipmentType.CONTROL_EQUIPMENT_TYPES;
   }


   /**
    * Getter for Transformer Types
    * @return List
    */
   public List<? extends Object> getTransTypes() {

      return TransformerType.TRANSFORMER_TYPES;
   }

   /**
    * Getter for Removed Equipment Types
    * @return List
    */
   public List<? extends Object> getRemovedEquipTypes() {

      return RemovedEquipmentType.REMOVED_EQUIPMENT_TYPES;
   }

   public void setNetworkDevLoc1(String networkDevLoc1) {
      this.networkDevLoc1 = networkDevLoc1;
   }

   public String getNetworkDevLoc1() {
      return networkDevLoc1;
   }

   public void setNetworkDevNumber1(String networkDevNumber1) {
      this.networkDevNumber1 = networkDevNumber1;
   }

   public String getNetworkDevNumber1() {
      return networkDevNumber1;
   }

   public void setNetworkDevLoc2(String networkDevLoc2) {
      this.networkDevLoc2 = networkDevLoc2;
   }

   public String getNetworkDevLoc2() {
      return networkDevLoc2;
   }

   public void setNetworkDevNumber2(String networkDevNumber2) {
      this.networkDevNumber2 = networkDevNumber2;
   }

   public String getNetworkDevNumber2() {
      return networkDevNumber2;
   }

   public void setNetworkDevLoc3(String networkDevLoc3) {
      this.networkDevLoc3 = networkDevLoc3;
   }

   public String getNetworkDevLoc3() {
      return networkDevLoc3;
   }

   public void setNetworkDevNumber3(String networkDevNumber3) {
      this.networkDevNumber3 = networkDevNumber3;
   }

   public String getNetworkDevNumber3() {
      return networkDevNumber3;
   }

   public void setConEquipNumber1(String conEquipNumber1) {
      this.conEquipNumber1 = conEquipNumber1;
   }

   public String getConEquipNumber1() {
      return conEquipNumber1;
   }

   public void setConEquipType1(String conEquipType1) {
      this.conEquipType1 = conEquipType1;
   }

   public String getConEquipType1() {
      return conEquipType1;
   }

   public void setConChannel1(String conChannel1) {
      this.conChannel1 = conChannel1;
   }

   public String getConChannel1() {
      return conChannel1;
   }

   public void setConMeterNum1(String conMeterNum1) {
      this.conMeterNum1 = conMeterNum1;
   }

   public String getConMeterNum1() {
      return conMeterNum1;
   }

   public void setConEquipNumber2(String conEquipNumber2) {
      this.conEquipNumber2 = conEquipNumber2;
   }

   public String getConEquipNumber2() {
      return conEquipNumber2;
   }

   public void setConEquipType2(String conEquipType2) {
      this.conEquipType2 = conEquipType2;
   }

   public String getConEquipType2() {
      return conEquipType2;
   }

   public void setConChannel2(String conChannel2) {
      this.conChannel2 = conChannel2;
   }

   public String getConChannel2() {
      return conChannel2;
   }

   public void setConMeterNum2(String conMeterNum2) {
      this.conMeterNum2 = conMeterNum2;
   }

   public String getConMeterNum2() {
      return conMeterNum2;
   }

   public void setTransNumber1(String transNumber1) {
      this.transNumber1 = transNumber1;
   }

   public String getTransNumber1() {
      return transNumber1;
   }

   public void setTransType1(String transType1) {
      this.transType1 = transType1;
   }

   public String getTransType1() {
      return transType1;
   }

   public void setTransRatio1(String transRatio1) {
      this.transRatio1 = transRatio1;
   }

   public String getTransRatio1() {
      return transRatio1;
   }

   public void setTransConNumber1(String transConNumber1) {
      this.transConNumber1 = transConNumber1;
   }

   public String getTransConNumber1() {
      return transConNumber1;
   }

   public void setTransNumber2(String transNumber2) {
      this.transNumber2 = transNumber2;
   }

   public String getTransNumber2() {
      return transNumber2;
   }

   public void setTransType2(String transType2) {
      this.transType2 = transType2;
   }

   public String getTransType2() {
      return transType2;
   }

   public void setTransRatio2(String transRatio2) {
      this.transRatio2 = transRatio2;
   }

   public String getTransRatio2() {
      return transRatio2;
   }

   public void setTransConNumber2(String transConNumber2) {
      this.transConNumber2 = transConNumber2;
   }

   public String getTransConNumber2() {
      return transConNumber2;
   }

   public void setRemovedMeter(String removedMeter) {
      this.removedMeter = removedMeter;
   }

   public String getRemovedMeter() {
      return removedMeter;
   }

   public void setRemovedOther(String removedOther) {
      this.removedOther = removedOther;
   }

   public String getRemovedOther() {
      return removedOther;
   }

   public void setRemovedEquipNumber1(String removedEquipNumber1) {
      this.removedEquipNumber1 = removedEquipNumber1;
   }

   public String getRemovedEquipNumber1() {
      return removedEquipNumber1;
   }

   public void setRemovedEquipType1(String removedEquipType1) {
      this.removedEquipType1 = removedEquipType1;
   }

   public String getRemovedEquipType1() {
      return removedEquipType1;
   }

   public void setRemovedMeterRead1(String removedMeterRead1) {
      this.removedMeterRead1 = removedMeterRead1;
   }

   public String getRemovedMeterRead1() {
      return removedMeterRead1;
   }

   public void setRemovedReg1(String removedReg1) {
      this.removedReg1 = removedReg1;
   }

   public String getRemovedReg1() {
      return removedReg1;
   }

   public void setRemovedEquipNumber2(String removedEquipNumber2) {
      this.removedEquipNumber2 = removedEquipNumber2;
   }

   public String getRemovedEquipNumber2() {
      return removedEquipNumber2;
   }

   public void setRemovedEquipType2(String removedEquipType2) {
      this.removedEquipType2 = removedEquipType2;
   }

   public String getRemovedEquipType2() {
      return removedEquipType2;
   }

   public void setRemovedMeterRead2(String removedMeterRead2) {
      this.removedMeterRead2 = removedMeterRead2;
   }

   public String getRemovedMeterRead2() {
      return removedMeterRead2;
   }

   public void setRemovedReg2(String removedReg2) {
      this.removedReg2 = removedReg2;
   }

   public String getRemovedReg2() {
      return removedReg2;
   }

   public void setNotes(String notes) {
      this.notes = notes;
   }

   public String getNotes() {
      return notes;
   }

   public void setTotNetworkDevices(String totNetworkDevices) {
      this.totNetworkDevices = totNetworkDevices;
   }

   public String getTotNetworkDevices() {
      return totNetworkDevices;
   }

   public void setWorkDateTime(String workDateTime) {
      this.workDateTime = workDateTime;
      B2BDateTimeFormatter dateTimeFormatter;
      this.workDateTime = workDateTime;
      try
      {
        dateTimeFormatter = new B2BDateTimeFormatter(workDateTime, B2BDateTimeFormatter.XMLDATE);
        setFieldworkDate(dateTimeFormatter.getB2BDate());
        setFieldworkTime(dateTimeFormatter.getB2BTime());
      }
      catch (ParseException e)
      {
        setFieldworkDate("");
        setFieldworkTime("");
      }
   }

   public String getWorkDateTime() {
      String dateString;
      B2BDateTimeFormatter dateTimeFormatter;

      try
      {
        dateTimeFormatter = new B2BDateTimeFormatter(getFieldworkDate() + " " + getFieldworkTime(), B2BDateTimeFormatter.B2BDATE);
        dateString = dateTimeFormatter.getXMLFormattedDateTime();
      }
      catch (ParseException e)
      {
        dateString = "";
      }
      return dateString;
   }

   public void setConLod1(String conLod1) {
      this.conLod1 = conLod1;
   }

   public String getConLod1() {
      return conLod1;
   }

   public void setConLod2(String conLod2) {
      this.conLod2 = conLod2;
   }

   public String getConLod2() {
      return conLod2;
   }

   public void setConLod3(String conLod3) {
      this.conLod3 = conLod3;
   }

   public String getConLod3() {
      return conLod3;
   }

   public String getPartRole() {
      return partRole;
   }
}

