/**
 * NotifiedParty.java
 * <p>
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the
 * National Electricity Market Management Company Limited.
 * <p>
 * Copyright National Electricity Market Management Company Limited. 2009
 * <p>
 * Modification History
 * <p>
 * Date Changed   Changed By        Description
 * ------------   ----------        -----------
 * 21-Jan-2009    Eddie Graham      Created.
 * 18-Mar-2011    Eddie Graham      Defect 7087. Add new method getOneWayNotificationNames()
 * to return list of One Way Notification Names.
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb;

import com.capgemini.nemmco.b2bb.enums.NotificationStatusType;
import com.capgemini.nemmco.b2bb.enums.ServiceOrderSubType;
import com.capgemini.nemmco.b2bb.enums.ServiceOrderType;
import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.lookup.XPath;
import com.capgemini.nemmco.b2bb.utils.B2BBDOMHelper;
import com.capgemini.nemmco.b2bb.utils.B2BDateFormatter;
import com.capgemini.nemmco.b2bb.utils.B2BDateTimeFormatter;
import org.apache.commons.lang3.StringUtils;
import org.w3c.dom.Node;

import java.util.List;

public class NotifiedParty extends Transaction {


	public static final String INITIATOR_ID = "initiatorID";
	public static final String RECIPIENT_ID = "recipientID";
	public static final String NMI_CHECKSUM = "NMIChecksum";
	public static final String SERVICE_ORDER_ID = "serviceOrderID";
	public static final String SERVICE_ORDER_TYPE = "serviceOrderType";
	public static final String SERVICE_ORDER_SUB_TYPE = "serviceOrderSubType";
	public static final String SCHEDULED_DATE = "scheduledDate";
	public static final String ACTUAL_DATE_AND_TIME = "actualDateAndTime";
	public static final String NOTIFICATION_STATUS = "notificationStatus";
	public static final String REF_TRANSACTION = "refTransaction";

    private String initiatorID = "";
 	 private String recipientID = "";
    private String serviceOrderID = "";
    private String serviceOrderType = "";
	private String serviceOrderSubType = "";
	
	  private boolean isScheduledDateSet = false;
	  private String scheduledDate = "";
	   private String scheduledDisplayDate = "";

	   private String actualDateAndTime = "";
    private boolean actualDateTimeSet = false;
	   private String actualDisplayDate = "";
	   private String actualDisplayTime = "";
    
    private String notificationStatus = "";
    private String refTransaction = "";

    /**
     * Default Constructor
     */
    public NotifiedParty() {
        super(TransactionType.NOTIFIED_PARTY);
    }


    public void setInitiatorID(String initiatorID) {
        this.initiatorID = initiatorID;
    }


	/**
	 * Retrieve initiatorID
	 *
	 * @return initiatorID.
	 */
    public String getInitiatorID() {
        return initiatorID;
    }

	public void setServiceOrderSubType(String serviceOrderSubType) {
		this.serviceOrderSubType = serviceOrderSubType;
	}

	public String getServiceOrderSubType() {
		return serviceOrderSubType;
	}

	public void setServiceOrderID(String serviceOrderID) {
        this.serviceOrderID = serviceOrderID;
    }

    public String getServiceOrderID() {
        return serviceOrderID;
    }

    public void setServiceOrderType(String serviceOrderType) {
        this.serviceOrderType = serviceOrderType;
    }

    public String getServiceOrderType() {
        return serviceOrderType;
    }

    /**
     * @return Returns the scheduleDate.
     */
    public String getScheduledDate() {
      if (scheduledDate == null || scheduledDate.equals("")) {
        try {
          B2BDateFormatter df = new B2BDateFormatter(getScheduledDisplayDate(), B2BDateFormatter.B2BDATE);
          this.scheduledDate = df.getXMLDate();
        } catch (Exception e) {
          this.scheduledDate = "";
        }
      }
      return this.scheduledDate;
    }
    /**
     * @param scheduleDate The scheduleDate to set.
     */
    public void setScheduledDate(String scheduleDate) {
      this.isScheduledDateSet = true;
      this.scheduledDate = scheduleDate;
      try {
        B2BDateFormatter df = new B2BDateFormatter(scheduleDate, B2BDateFormatter.XMLDATE);
        this.scheduledDisplayDate = df.getB2BDate();
      } catch (Exception e) {
        this.scheduledDisplayDate = "";
      }
    }
    /**
     * @return Returns the scheduledDisplayDate.
     */
    public String getScheduledDisplayDate() {
      return this.scheduledDisplayDate;
    }
    /**
     * @param scheduleDisplayDate The scheduleDisplayDate to set.
     */
    public void setScheduleDisplayDate(String scheduleDisplayDate) {
      this.isScheduledDateSet = false;
      this.scheduledDisplayDate = scheduleDisplayDate;
    }

    public boolean getIsScheduledDateSet() {
      return isScheduledDateSet;
    }
    
    /**
     * @return Returns true if actualDateTimeSet is set, false otherwise
     */
    public boolean isActualDateTimeSet() {
        return actualDateTimeSet;
    }

    /**
     * @param actualDateTime The actualDateTime to set.
     */
    public void setActualDateAndTime(String actualDateAndTime) {
        this.actualDateTimeSet = true;
      this.actualDateAndTime = actualDateAndTime;
      try {
        // split
        B2BDateTimeFormatter df = new B2BDateTimeFormatter(actualDateAndTime, B2BDateTimeFormatter.XMLDATE);
        this.actualDisplayDate = df.getB2BDate();
        this.actualDisplayTime = df.getB2BTime();
      } catch (Exception e) {
        this.actualDisplayDate = "";
        this.actualDisplayTime = "";
      }
    }

    /**
     * @param actualDateTimeSet The actualDateTimeSet to set.
     */
    public void setActualDateTimeSet(boolean isActualDateTimeSet) {
        this.actualDateTimeSet = isActualDateTimeSet;
    }

    /**
     * @param actualDisplayDate The actualDisplayDate to set.
     */
    public void setActualDisplayDate(String actualDisplayDate) {
        this.actualDateTimeSet = false;
      this.actualDisplayDate = actualDisplayDate;
    }

    /**
     * @param actualDisplayTime The actualDisplayTime to set.
     */
    public void setActualDisplayTime(String actualDisplayTime) {
        this.actualDateTimeSet = false;
      this.actualDisplayTime = actualDisplayTime;
    }

    /**
     * @return Returns the actualDateTime.
     */
    public String getActualDateAndTime() {
      if (actualDateAndTime == null || actualDateAndTime.equals("")) {
        try {
          B2BDateTimeFormatter df = new B2BDateTimeFormatter(getActualDisplayDateTime(), B2BDateTimeFormatter.B2BDATE);
          this.actualDateAndTime = df.getXMLFormattedDateTime();
        } catch (Exception e) {
          this.actualDateAndTime = "";
        }
      }
      return actualDateAndTime;
    }

    /**
     * @return Returns the actualDisplayDate.
     */
    public String getActualDisplayDate() {
      return actualDisplayDate;
    }

    /**
     * @return Returns the actualDisplayTime.
     */
    public String getActualDisplayDateTime() {
        if(StringUtils.isNotEmpty(actualDisplayDate.trim())) {
            return actualDisplayDate + " " +  (StringUtils.isNotEmpty(actualDisplayTime.trim()) ? actualDisplayTime : "00:00:00" );
        }
      return actualDisplayDate + " " + actualDisplayTime;
    }

    /**
     * @return Returns the actualDisplayTime.
     */
    public String getActualDisplayTime() {
      return actualDisplayTime;
    }
    
    
    public void setNotificationStatus(String notificationStatus) {
        this.notificationStatus = notificationStatus;
    }

    public String getNotificationStatus() {
        return notificationStatus;
    }

    public void setRefTransaction(String refTransaction) {
        this.refTransaction = refTransaction;
        try {
            this.refTransaction = getXmlString(REF_TRANSACTION);
        } catch (Exception e) {
            this.refTransaction = refTransaction;
        }
    }

    public String getRefTransaction() {
        return refTransaction;
    }


	public List<? extends Object> getNotificationStatusType() {
	  return NotificationStatusType.NOTIFICATION_STATUS;
	}

	public void setRecipientID(String recipientID) {
		this.recipientID = recipientID;
	}

	public String getRecipientID() {
		return recipientID;
	}

	/**
	 * @return Returns the serviceOrderType list
	 */
	public List<? extends Object> getServiceOrderTypes() {
	  return ServiceOrderType.SERVICE_ORDER_TYPES;
	}

	/**
	 * @return Returns the serviceOrderSubType list
	 */
	public List<? extends Object> getServiceOrderSubTypes() {
	  return ServiceOrderSubType.SERVICE_ORDER_SUB_TYPES;
	}

    public void setScheduledDateSet(boolean scheduledDateSet) {
        isScheduledDateSet = scheduledDateSet;
    }
    public void setScheduledDisplayDate(String scheduledDisplayDate) {
        this.scheduledDisplayDate = scheduledDisplayDate;
    }
}

