/**
 * SampleParticipant.java
 *
 * The Copyright of this work is vested in the Australian Energy Market
 * Operator Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the Australian Energy Market Operator Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the
 * Australian Energy Market Operator Limited.
 *
 * Copyright Australian Energy Market Operator Limited. 2005
 *
 * Modification History
 *
 * Date Changed   Changed By  Description
 * ------------   ----------  -----------
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 */


package com.capgemini.nemmco.b2bb;

import java.util.List;
import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.utils.EventObject;
import com.capgemini.nemmco.b2bb.validators.Validator;
import com.capgemini.nemmco.b2bb.validators.ValidatorFactory;

public class SampleParticipant 
{

  static String xmlSnippetValid = 
    "      <Transaction transactionDate=\"2005-04-08T11:50:00.0+10:00\" transactionID=\"B2BT1112925001027000\">"+
    "         <ServiceOrderRequest version=\"r17\" actionType=\"New\">"+
    "            <ServiceOrder>"+
    "               <NMI checksum=\"7\">1234567890</NMI>"+
    "               <ServiceOrderType xsi:type=\"ase:ElectricityServiceOrderType\" version=\"r17\">"+
    "                  <WorkType>Meter Reconfiguration</WorkType>"+
    "               </ServiceOrderType>"+
    "               <ServiceOrderNumber>123123141412412</ServiceOrderNumber>"+
    "            </ServiceOrder>"+
    "            <ServicePoint>"+
    "               <AccessDetails>No Details</AccessDetails>"+
    "            </ServicePoint>"+
    "            <CustomerDetail>"+
    "               <PersonName>"+
    "                  <NameTitle>Dr</NameTitle>"+
    "                  <GivenName>John</GivenName>"+
    "                  <FamilyName>Smith</FamilyName>"+
    "                  <NameSuffix>MD</NameSuffix>"+
    "               </PersonName>"+
    "               <PhoneNumber serviceType=\"Fixed Voice\">"+
    "                  <Prefix>02</Prefix>"+
    "                  <Number>95555555</Number>"+
    "                  <ServiceComment>Business</ServiceComment>"+
    "               </PhoneNumber>"+
    "            </CustomerDetail>"+
    "            <AppointmentDetail>"+
    "               <ScheduledDate>2005-06-22</ScheduledDate>"+
    "            </AppointmentDetail>"+
    "            <RequestData xsi:type=\"ase:ElectricityServiceOrderDetails\">"+
    "               <SpecialInstructions>"+
    "                  <CommentLine>No Special Instruction</CommentLine>"+
    "               </SpecialInstructions>"+
    "               <NMIStandingData xsi:type=\"ase:ElectricityStandingData\"/>"+
    "               <CustomerConsultationRequired>true</CustomerConsultationRequired>"+
    "               <ServiceTime>Any Time</ServiceTime>"+
    "            </RequestData>"+
    "         </ServiceOrderRequest>"+
    "      </Transaction>"
    ;

  static String xmlSnippetInvalid = 
    "      <Transaction transactionDate=\"2005-04-08T11:50:00.0+10:00\" transactionID=\"B2BT1112925001027000\">"+
    "         <ServiceOrderRequest version=\"r17\" actionType=\"New\">"+
    "            <ServiceOrder>"+
    "               <NMI checksum=\"7\">1234567890</NMI>"+
    "               <ServiceOrderType xsi:type=\"ase:ElectricityServiceOrderType\" version=\"r17\">"+
    "                  <WorkType>Meter Reconfiguration</WorkType>"+
    "               </ServiceOrderType>"+
    "               <ServiceOrderNumber>123123141412412</ServiceOrderNumber>"+
    "            </ServiceOrder>"+
    "            <ServicePoint>"+
    "               <AccessDetails>No Details</AccessDetails>"+
    "            </ServicePoint>"+
    "            <CustomerDetail>"+
    "               <PersonName>"+
    "                  <NameTitle>Dr</NameTitle>"+
    "                  <GivenName>John</GivenName>"+
    "                  <FamilyName>Smith</FamilyName>"+
    "                  <NameSuffix>MD</NameSuffix>"+
    "               </PersonName>"+
    "               <PhoneNumber serviceType=\"Fixed Voice\">"+
    "                  <Prefix>02</Prefix>"+
    "                  <Number>95555555</Number>"+
    "                  <ServiceComment>Business</ServiceComment>"+
    "               </PhoneNumber>"+
    "            </CustomerDetail>"+
    "            <AppointmentDetail>"+
    "               <ScheduledDate>2008-06-22</ScheduledDate>"+
    "            </AppointmentDetail>"+
    "            <RequestData xsi:type=\"ase:ElectricityServiceOrderDetails\">"+
    "               <SpecialInstructions>"+
    "                  <CommentLine>No Special Instruction</CommentLine>"+
    "               </SpecialInstructions>"+
    "               <NMIStandingData xsi:type=\"ase:ElectricityStandingData\"/>"+
    "               <CustomerConsultationRequired>true</CustomerConsultationRequired>"+
    "               <ServiceTime>Any Time</ServiceTime>"+
    "            </RequestData>"+
    "         </ServiceOrderRequest>"+
    "      </Transaction>"
    ;
  
  public static void main(String[] args) throws Exception
  {
    System.out.println("The following tests a valid transaction:-");
    checkCode(TransactionType.SOR_MSW_METER_RECONFIG, xmlSnippetValid);
    System.out.println("");
    System.out.println("");
    System.out.println("The following tests an invalid transaction");
    checkCode(TransactionType.SOR_MSW_METER_RECONFIG, xmlSnippetInvalid);
  }

  /**
   * 
   */
  private static void checkCode(TransactionType tt, String xmlSnippet) throws Exception {

    // Ask the factory for a Site Access Notification Transaction
    Transaction sampleTrans = TransactionFactory.getTransaction(tt);

    // Set the XML Snippet for this transaction
    sampleTrans.setXMLString(xmlSnippet);
    
    // Ask the validator factory for a Site Access Notification Validator
    Validator sampleValidator = ValidatorFactory.getValidator(sampleTrans);
        
    // validate the transaction
    boolean result = sampleValidator.validate();
    
    // was an error found in validation
    List<EventObject> errorList = null;
    if (result)
    {
      System.out.println("No errors found in validation");
    } else 
    {
      System.out.println("Errors found in validation");
      // extract error list
      errorList = sampleValidator.getEvents();
      
      // extract and print out all the errors
      for (int i = 0; i < errorList.size(); i++) 
      {
        EventObject tmpErr = errorList.get(i);
        System.out.println("Error Context     : " + tmpErr.getContext());
        System.out.println("Error Code        : " + tmpErr.getEventCode());
        System.out.println("Error Description : " + tmpErr.getEventDesc());
        System.out.println("Error Explanation : " + tmpErr.getExplanation());
        System.out.println("Error Severity    : " + tmpErr.getSeverity());
        System.out.println("-----------------");
      }
    }
  }
}