/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/ServiceOrderAppointmentNotification.java-arc   1.4   Apr 13 2005 16:32:20   adij  $
 * $Revision:   1.4  $
 * $Date:   Apr 13 2005 16:32:20  $
 *
 * Copyright 2005 Capgemini Australia Pty. Ltd.
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 */
package com.capgemini.nemmco.b2bb;

/**
 * The class <code>ServiceOrderAppointmentNotification</code> represents a ServiceOrderAppointmentNotification
 * Transaction.
 *
 * @author  $Author:   adij  $
 * @version $Revision:   1.4  $
 */
import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.utils.B2BDateFormatter;
import com.capgemini.nemmco.b2bb.utils.B2BTimeFormatter;
import com.capgemini.nemmco.b2bb.validators.Address;

public class ServiceOrderAppointmentNotification extends Transaction {

  public static final String RESPONSE_TYPE = "responseType";
  public static final String RET_SERVICE_ORDER = "retServiceOrder";
  public static final String NMI_FIELD = "NMI";
  public static final String NMI_CHECKSUM = "NMIChecksum";
  public static final String SERVICE_ORDER_ADDRESS = "serviceOrderAddress";
  public static final String APPOINTMENT_REFERENCE = "appointmentReference";
  public static final String CUSTOMERS_PREFERRED_DATE = "customersPreferredDate";
  public static final String CUSTOMERS_PREFERRED_TIME = "customersPreferredTime";
  public static final String SERVICE_PROVIDER_REFERENCE = "serviceProviderReference";

  private String responseType = "";
  private String retServiceOrder = "";
  private String NMI = "";
  private String NMIChecksum = "";
  private Address serviceOrderAddress;
  private String appointmentReference = "";
  private String customersPreferredDate = "";
  private String customersPreferredTime = "";
  private String serviceProviderReference = "";

  private String customersPreferredDisplayDate = "";
  private String customersPreferredDisplayTime = "";

  private boolean isCustomersPreferredDateSet = false;
  private boolean isCustomersPreferredTimeSet = false;

  /**
   * Default Constructor.
   */
  public ServiceOrderAppointmentNotification() {
    super(TransactionType.SO_APPOINTMENT_NOT);
  }

  /**
   * @return Returns the responseType.
   */
  public String getResponseType() {
    return responseType;
  }

  /**
   * @return Returns the retServiceOrder.
   */
  public String getRetServiceOrder() {
    return retServiceOrder;
  }

  /**
   * @return Returns the nMI.
   */
  public String getNMI() {
    return NMI;
  }

  /**
   * @return Returns the nMIChecksum.
   */
  public String getNMIChecksum() {
    return NMIChecksum;
  }

  /**
   * @return Returns the serviceOrderAddress.
   */
  public Address getServiceOrderAddress() {
    return serviceOrderAddress;
  }

  /**
   * @return Returns the appointmentReference.
   */
  public String getAppointmentReference() {
    return appointmentReference;
  }

  /**
   * @return Returns the customersPreferredDate.
   */
  public String getCustomersPreferredDate() {
    if (customersPreferredDate == null || customersPreferredDate.equals("")) {
      try {
        B2BDateFormatter df = new B2BDateFormatter(getCustomersPreferredDisplayDate(), B2BDateFormatter.B2BDATE);
        this.customersPreferredDate = df.getXMLDate();
      } catch (Exception e) {
        this.customersPreferredDate = "";
      }
    }
    return this.customersPreferredDate;
  }

  /**
   * @return Returns the customersPreferredTime.
   */
  public String getCustomersPreferredTime() {
    if (customersPreferredTime == null || customersPreferredTime.equals("")) {
      try {
        B2BTimeFormatter tf = new B2BTimeFormatter(getCustomersPreferredDisplayTime(), B2BTimeFormatter.B2BTIME);
        this.customersPreferredTime = tf.getXMLTime();
      } catch (Exception e) {
        this.customersPreferredTime = "";
      }
    }
    return this.customersPreferredTime;
  }

  /**
   * @return Returns the customersPreferredDisplayDate.
   */
  public String getCustomersPreferredDisplayDate() {
    return this.customersPreferredDisplayDate;
  }

  /**
   * @return Returns the customersPreferredDisplayTime.
   */
  public String getCustomersPreferredDisplayTime() {
    return this.customersPreferredDisplayTime;
  }

  /**
   * @return Returns the serviceProviderReference.
   */
  public String getServiceProviderReference() {
    return serviceProviderReference;
  }

  /**
   * @param responseType The responseType to set.
   */
  public void setResponseType(String responseType) {
    this.responseType = responseType;
  }

  /**
   * @param retServiceOrder The retServiceOrder to set.
   */
  public void setRetServiceOrder(String retServiceOrder) {
    this.retServiceOrder = retServiceOrder;
  }

  /**
   * @param nmi The nMI to set.
   */
  public void setNMI(String nmi) {
    NMI = nmi;
  }

  /**
   * @param checksum The nMIChecksum to set.
   */
  public void setNMIChecksum(String checksum) {
    NMIChecksum = checksum;
  }

  /**
   * @param serviceOrderAddress The serviceOrderAddress to set.
   */
  public void setServiceOrderAddress(Address serviceOrderAddress) {
    this.serviceOrderAddress = serviceOrderAddress;
  }

  /**
   * @param appointmentReference The appointmentReference to set.
   */
  public void setAppointmentReference(String appointmentReference) {
    this.appointmentReference = appointmentReference;
  }

  /**
   * @param customersPreferredDate The customersPreferredDate to set.
   */
  public void setCustomersPreferredDate(String customersPreferredDate) {
    this.isCustomersPreferredDateSet = true;
    this.customersPreferredDate = customersPreferredDate;
    try {
      B2BDateFormatter df = new B2BDateFormatter(customersPreferredDate, B2BDateFormatter.XMLDATE);
      this.customersPreferredDisplayDate = df.getB2BDate();
    } catch (Exception e) {
      this.customersPreferredDisplayDate = "";
    }
  }

  /**
   * @param customersPreferredTime The customersPreferredTime to set.
   */
  public void setCustomersPreferredTime(String customersPreferredTime) {
    this.isCustomersPreferredTimeSet = true;
    this.customersPreferredTime = customersPreferredTime;
    try {
      B2BTimeFormatter df = new B2BTimeFormatter(customersPreferredTime, B2BTimeFormatter.XMLTIME);
      this.customersPreferredDisplayTime = df.getB2BTime();
    } catch (Exception e) {
      this.customersPreferredDisplayTime = "";
    }
  }

  /**
   * @param customersPreferredDisplayDate The customersPreferredDisplayDate to set.
   */
  public void setPreferredDisplayDate(String customersPreferredDisplayDate) {
    this.isCustomersPreferredDateSet = false;
    this.customersPreferredDisplayDate = customersPreferredDisplayDate;
  }

  /**
   * @param preferredDisplayTime The preferredTime to set.
   */
  public void setCustomersPreferredDisplayTime(String customersPreferredDisplayTime) {
    this.isCustomersPreferredTimeSet = false;
    this.customersPreferredDisplayTime = customersPreferredDisplayTime;
  }

  /**
   * @param serviceProviderReference The serviceProviderReference to set.
   */
  public void setServiceProviderReference(String serviceProviderReference) {
    this.serviceProviderReference = serviceProviderReference;
  }

}

