 /**
  * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/ServiceOrderRequest.java-arc   1.18   Sep 15 2005 15:25:12   adij  $
  * $Revision:   1.18  $
  * $Date:   Sep 15 2005 15:25:12  $
  *
  * Copyright 2005 Capgemini Australia Pty. Ltd.
  * All rights reserved.
  * NEMMCO CONFIDENTIAL.
  *
  * Modification History
  *
  * Date Changed   Changed By        Comments
  * ------------   ----------        -----------
  * 21-Aug-2008    Bob Jones         Defect 4985 - increase each telephone number from one to three.
  * 10-Dec-2010    Oleg Pevzner      Def 6675 EVM for b2b does not validate length for role id. Allow <BusinessName> and <ContactName> when <PhoneNumber> is provided.
  * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
  * 03-Oct-2012    W.Mills           8081 - Java 7 upgrade
  * 16-Jul-2017    Hamish McNeish    Changes to support PoC changes
  * 18-July-2017   Harpreet Kaur     Code changes for LifeSupport and CustomerContact
  * 26-July-2017  SP                  changed  YesNo and TrueFalse
  * 28-Jul-2017    Harpreet Kaur     7135 Package 7B: Changes doen for SOR Meter Service Works screens as part of POC SMP
  * 31-July-2017  SP                  Changed service Time accesser to protected
  * 02-Aug-2017    Harpreet Kaur     7135 Package 7B: Changes done for EnergisationReason and DeEnergisationReason screens as part of POC SMP
  * 22-Sep-2017    Harpreet Kaur     Package 24 : Modify method getNotifiedPartyIds() to display participants depending upon sendToAnyParticipant value
  * 04-Oct-2017    Harpreet Kaur     Package 24 : Fix for Defect 8020
  * 20-Oct-2017    SP                Defect 8067
  */

 package com.capgemini.nemmco.b2bb;

 import com.capgemini.nemmco.b2bb.enums.ActionType;
 import com.capgemini.nemmco.b2bb.enums.CustomerType;
 import com.capgemini.nemmco.b2bb.enums.DeEnergisationReason;
 import com.capgemini.nemmco.b2bb.enums.InstallationType;
 import com.capgemini.nemmco.b2bb.enums.MeterInstallCode;
 import com.capgemini.nemmco.b2bb.enums.MeteringType;
 import com.capgemini.nemmco.b2bb.enums.SafetyCertMethodSentType;
 import com.capgemini.nemmco.b2bb.enums.ServiceOrderSubType;
 import com.capgemini.nemmco.b2bb.enums.ServiceOrderType;
 import com.capgemini.nemmco.b2bb.enums.ServiceTime;
 import com.capgemini.nemmco.b2bb.enums.SupplyPhase;
 import com.capgemini.nemmco.b2bb.enums.TransactionType;
 import com.capgemini.nemmco.b2bb.enums.TrueFalse;
 import com.capgemini.nemmco.b2bb.enums.YesNo;
 import com.capgemini.nemmco.b2bb.lookup.EmbeddedNetworkParentName;
 import com.capgemini.nemmco.b2bb.lookup.Participant;
 import com.capgemini.nemmco.b2bb.utils.B2BDateFormatter;
 import com.capgemini.nemmco.b2bb.utils.B2BTimeFormatter;
 import com.capgemini.nemmco.b2bb.validators.Address;
import com.capgemini.nemmco.b2bb.validators.ContactName;
import com.capgemini.nemmco.b2bb.validators.Licensee;
 import com.capgemini.nemmco.b2bb.validators.PersonName;
 import com.capgemini.nemmco.b2bb.validators.RequestData;
 import com.capgemini.nemmco.b2bb.validators.Telephone;

 import java.util.ArrayList;
 import java.util.List;

 /**
  * Defines all the fields for every type of Service Order Request (SOR). As SOR
  * messages all use common fields one parent class defining all these fields
  * is needed.
  *
  * @author $Author:   adij  $
  * @version $Revision:   1.18  $ $Date:   Sep 15 2005 15:25:12  $
  */
 public abstract class ServiceOrderRequest extends Transaction
 {

   public static String ACTIONTYPE = "actionType";
   public static String NMI_FIELD = "NMI";
   public static String NMI_CHECKSUM = "NMIChecksum";
   public static String SERVICE_ORDER_TYPE = "serviceOrderType";
   public static String SERVICE_ORDER_SUB_TYPE = "serviceOrderSubType";
   public static String METER_SERIAL_NUMBER = "meterSerialNumber";
   public static String METER_SERIAL_NUMBER1 = "meterSerialNumber1";
   public static String METER_SERIAL_NUMBER2 = "meterSerialNumber2";
   public static String METER_SERIAL_NUMBER3 = "meterSerialNumber3";
   public static String RET_SERVICE_ORDER = "retServiceOrder";
   public static String ACCESS_DETAILS = "accessDetails";

   public static String SERVICE_ORDER_ADDRESS = "serviceOrderAddress";
   // (Justina Tran) defect 3513: added the simpleRequestData variable and setter and getter methods.
   public static String REQUEST_DATA = "simpleRequestData";

   public static String HAZARD_DESCRIPTION1 = "hazardDescription1";
   public static String HAZARD_DESCRIPTION2 = "hazardDescription2";
   public static String HAZARD_DESCRIPTION3 = "hazardDescription3";

   public static String RETAILER_NAME = "initiatorName";
   public static String RETAILER_PHONENUMBER1 = "initiatorPhone1";
   public static String RETAILER_PHONENUMBER2 = "initiatorPhone2";
   public static String RETAILER_PHONENUMBER3 = "initiatorPhone3";
   public static String CUSTOMER_CONTACT_NAME = "customerContactName";
   public static String CONTACT_PHONENUMBER1 = "contactPhone1";
   public static String CONTACT_PHONENUMBER2 = "contactPhone2";
   public static String CONTACT_PHONENUMBER3 = "contactPhone3";

   public static String APPOINTMENT_REFERENCE = "appointmentReference";
   public static String SCHEDULE_DATE = "scheduleDate";
   public static String PREFERRED_DATE = "preferredDate";
   public static String PREFERRED_TIME = "preferredTime";
   public static String SPECIAL_INSTRUCTIONS1 = "specialInstructions1";
   public static String SPECIAL_INSTRUCTIONS2 = "specialInstructions2";
   public static String SPECIAL_INSTRUCTIONS3 = "specialInstructions3";
   public static String INSTALLATION_TYPE = "installationType";
   public static String SUPPLY_PHASE = "supplyPhase";
   public static String AVERAGE_DAILY_LOAD = "averageDailyLoad";
   public static String METER_INSTALL_CODE = "meterInstallCode";
   public static String CUSTOMER_TYPE = "customerType";

   public static String LICENSEE_NAME = "licenseeName";
   public static String LICENSEE_BUSINESS_NAME = "licenseeBusinessName";
   public static String LICENSEE_PHONENUMBER1 = "licenseePhone1";
   public static String LICENSEE_PHONENUMBER2 = "licenseePhone2";
   public static String LICENSEE_PHONENUMBER3 = "licenseePhone3";

   public static String REC_ID = "recId";
   public static String SAFETY_CERT_ID = "safetyCertificateId";
   public static String SAFETY_CERT_METHOD_SENT = "safetyCertificateIdMethodSent";
   public static String FORM_REF = "formReference";
   public static String FORM_NUM = "formNumber";
   public static String DOC_REFERENCE = "documentReference";
   public static String REC_ATTENDANCE_REQUIRED = "recAttendanceRequired";
   public static String METERING_REQUIRED = "meteringRequired";
   public static String OFF_PEAK_REQUIREMENT1 = "offPeakRequirement1";
   public static String OFF_PEAK_REQUIREMENT2 = "offPeakRequirement2";
   public static String OFF_PEAK_REQUIREMENT3 = "offPeakRequirement3";
   public static String MAXIMUM_DEMAND = "maximumDemand";
   public static String EMBEDDED_NET_PARENT = "embeddedNetworkParentName";
   public static String NMI_STATUS_CODE = "NMIStatusCode";
   public static String ROLE1 = "role1";
   public static String ROLE2 = "role2";
   public static String ROLE3 = "role3";
   public static String ROLE4 = "role4";
   public static String PARTY1 = "party1";
   public static String PARTY2 = "party2";
   public static String PARTY3 = "party3";
   public static String PARTY4 = "party4";
   public static String PROPOSED_TARIFF = "proposedTariff";
   public static String PROPOSED_TARIFF1 = "proposedTariff1";
   public static String PROPOSED_TARIFF2 = "proposedTariff2";
   public static String PROPOSED_TARIFF3 = "proposedTariff3";
   public static String SWITCHING_SERVICE_REQUIRED = "switchingServiceRequired";
   public static String SERVICE_TIME = "serviceTime";
   public static String CONFIRM_DEENERGISATION = "confirmDeenergisation";
   public static String CUST_CONSULTATION_REQUIRED = "customerConsultationRequired";
   public static String BUSINESS_CONTACT_NAME = "businessContactName";
   public static String BUSINESS_NAME = "businessName";
   //HM: Added for PoC B2B changes
   public static String LIFE_SUPPORT = "lifeSupport";
   public static String METERING_SAFETY_CERTIFICATE_ID = "meteringSafetyCertificateId";
   public static String METERING_SAFETY_CERT_METHOD_SENT = "meteringSafetyCertificateIdMethodSent";
   public static String COORDINATION_REQUIRED = "coOrdinationRequired";
   public static String COORDINATION_CONTACT_NAME = "coOrdinatingContactName";
   public static String COORDINATION_BUSINESS_CONTACT_NAME = "coordinatingBusinessContactName";
   public static String COORDINATION_BUSINESS_NAME = "coordinatingBusinessName";
   public static String NOTIFY_PARTY1 = "notifiedPartyId1";
   public static String NOTIFY_PARTY2 = "notifiedPartyId2";
   public static String NOTIFY_PARTY3 = "notifiedPartyId3";
   public static String COORDINATION_CONTACT_PHONENUMBER1 = "coOrdinatingContactPhone1";
   public static String COORDINATION_CONTACT_PHONENUMBER2 = "coOrdinatingContactPhone2";
   public static String COORDINATION_CONTACT_PHONENUMBER3 = "coOrdinatingContactPhone3";
   public static String DEENERGISATION_REASON = "deEnergisationReason";
     
   // private variables
   private String actionType="";
   private String serviceOrderType="";
   private String serviceOrderSubType="";
   private String meterSerialNumber="";
   private String meterSerialNumber1="";
   private String meterSerialNumber2="";
   private String meterSerialNumber3="";
   private String retServiceOrder="";
   private String accessDetails="";
   private String hazardDescription1="";
   private String hazardDescription2="";
   private String hazardDescription3="";
   private Address serviceOrderAddress;
   private RequestData simpleRequestData;
   private PersonName retailerName;
   private Telephone retailerPhone1;
   private Telephone retailerPhone2;
   private Telephone retailerPhone3;
   private PersonName customerContactName;
   private Telephone contactPhone1;
   private Telephone contactPhone2;
   private Telephone contactPhone3;
   private String appointmentReference="";
   private String scheduleDate="";
   private String preferredDate="";
   private String preferredTime="";
   private String specialInstructions1="";
   private String specialInstructions2="";
   private String specialInstructions3="";
   private String installationType="";
   private String supplyPhase="";
   private String averageDailyLoad="";
   private String meterInstallCode="";
   private String customerType="";
   private Licensee licenseeName;
   private String licenseeBusinessName="";
   private Telephone licenseePhone1;
   private Telephone licenseePhone2;
   private Telephone licenseePhone3;
   private String recId="";
   private String safetyCertificateId="";
   private String safetyCertificateIdMethodSent="";
   private String formReference="";
   private String formNumber="";
   private String meteringRequired="";
   private String offPeakRequirement1="";
   private String offPeakRequirement2="";
   private String offPeakRequirement3="";
   private String maximumDemand="";
   private String embeddedNetworkParentName="";
   private String NMIStatusCode="";
   private String role1="";
   private String role2="";
   private String role3="";
   private String role4="";
   private String party1="";
   private String party2="";
   private String party3="";
   private String party4="";
   private String proposedTariff="";
   private String proposedTariff1="";
   private String proposedTariff2="";
   private String proposedTariff3="";
   private String switchingServiceRequired="";
   private String serviceTime="";
   private String confirmDeenergisation="";
   private String customerConsultationRequired="";
   private String recAttendanceRequired="";

   private String scheduleDisplayDate="";
   private String preferredDisplayDate="";
   private String preferredDisplayTime="";

   private ContactName businessContactName;
   private String businessName="";
   //HM: Added for PoC B2B changes
   private String notifiedPartyId1="";
   private String notifiedPartyId2="";
   private String notifiedPartyId3="";
   private String lifeSupport="";
   private String meteringSafetyCertificateId="";
   private String meteringSafetyCertificateIdMethodSent="";
   private String coOrdinationRequired="";
   private PersonName coOrdinatingContactName;
   private Telephone coOrdinatingContactPhone1;
   private Telephone coOrdinatingContactPhone2;
   private Telephone coOrdinatingContactPhone3;
   private String deEnergisationReason="";
   
   private boolean isScheduleDateSet=false;
   private boolean isPreferredDateSet=false;
   private boolean isPreferredTimeSet=false;

   private String transactionDesc = "";
   private String sendToAnyParticipant = " ";
   private ContactName coordinatingBusinessContactName;
   private String coordinatingBusinessName="";

   /**
    * Default Constructor
    */
   public ServiceOrderRequest(TransactionType transactionType) {
     super(transactionType);
     setCustomerConsultationRequired("");// This is done few SORD types don't require customer consultation on screen however it is mandatory in schema. GUI options will override the default
     transactionDesc = transactionType.getDescription();
     this.serviceOrderSubType = transactionType.getSubDescription();
   }
   public ServiceOrderRequest(TransactionType transactionType, String customerConsultationFlag) {
     super(transactionType);
     setCustomerConsultationRequired(customerConsultationFlag);// This is done few SORD types don't require customer consultation on screen however it is mandatory in schema. GUI options will override the default
     transactionDesc = transactionType.getDescription();
     this.serviceOrderSubType = transactionType.getSubDescription();
   }

   public boolean getIsScheduleDateSet() {
     return isScheduleDateSet;
   }
   public boolean getIsPreferredDateSet() {
     return isPreferredDateSet;
   }
   public boolean getIsPreferredTimeSet() {
     return isPreferredTimeSet;
   }

   /**
    * @return Returns the licenseeBusinessName.
    */
   public String getLicenseeBusinessName() {
     return licenseeBusinessName;
   }
   /**
    * @param licenseeBusinessName The licenseeBusinessName to set.
    */
   public void setLicenseeBusinessName(String licenseeBusinessName) {
     this.licenseeBusinessName = licenseeBusinessName;
   }

   /**
    * @return Returns the accessDetails.
    */
   public String getAccessDetails() {
     return accessDetails;
   }
   /**
    * @param accessDetails The accessDetails to set.
    */
   public void setAccessDetails(String accessDetails) {
     this.accessDetails = accessDetails;
   }
   /**
    * @return Returns the actionType.
    */
   public String getActionType() {
     return actionType;
   }
   /**
    * @param actionType The actionType to set.
    */
   public void setActionType(String actionType) {
     this.actionType = actionType;
   }
   /**
    * @return Returns the appointmentReference.
    */
   public String getAppointmentReference() {
     return appointmentReference;
   }
   /**
    * @param appointmentReference The appointmentReference to set.
    */
   public void setAppointmentReference(String appointmentReference) {
     this.appointmentReference = appointmentReference;
   }
   /**
    * @return Returns the averageDailyLoad.
    */
   public String getAverageDailyLoad() {
     return averageDailyLoad;
   }
   /**
    * @param averageDailyLoad The averageDailyLoad to set.
    */
   public void setAverageDailyLoad(String averageDailyLoad) {
     this.averageDailyLoad = averageDailyLoad;
   }
   /**
    * @return Returns the confirmDeenergisation.
    */
   public String getConfirmDeenergisation() {
     return confirmDeenergisation;
   }
   /**
    * @param confirmDeenergisation The confirmDeenergisation to set.
    */
   public void setConfirmDeenergisation(String confirmDeenergisation) {
     this.confirmDeenergisation = confirmDeenergisation;
   }
   /**
    * @return Returns the serviceOrderAddress.
    */
   public Address getServiceOrderAddress(){
     return serviceOrderAddress;
   }
   /**
    * @param serviceOrderAddress The serviceOrderAddress to set.
    */
   public void setServiceOrderAddress(Address serviceOrderAddress) {
     this.serviceOrderAddress = serviceOrderAddress;
   }

     /**
      * @return Returns the simpleRequestData.
      */
     public RequestData getSimpleRequestData(){
       return simpleRequestData;
     }
     /**
      * @param requestData The simpleRequestData to set.
      */
     public void setSimpleRequestData(RequestData requestData) {
       this.simpleRequestData = requestData;
     }



   /**
    * @return Returns the customerContactName.
    */
   public PersonName getCustomerContactName() {
      return customerContactName;
   }
   /**
    * @param customerContactName The customerContactName to set.
    */
    public void setCustomerContactName(PersonName customerContactName) {
       this.customerContactName = customerContactName;
    }



    /**
     * @return Returns the contactPhone numbers.
     */
   public Telephone getContactPhone1() {
     return contactPhone1;
   }
   public Telephone getContactPhone2() {
     return contactPhone2;
   }
   public Telephone getContactPhone3() {
     return contactPhone3;
   }
   /**
    * @param contactPhone numbers The contactPhone numbers to set.
    */
   public void setContactPhone1(Telephone contactPhone) {
     this.contactPhone1 = contactPhone;
   }
   public void setContactPhone2(Telephone contactPhone) {
     this.contactPhone2 = contactPhone;
   }
   public void setContactPhone3(Telephone contactPhone) {
     this.contactPhone3 = contactPhone;
   }
   /**
    * @return Returns the customerConsultationRequired.
    */
   public String getCustomerConsultationRequired() {
     return customerConsultationRequired;
   }
   /**
    * @param customerConsultationRequired The customerConsultationRequired to set.
    */
   public void setCustomerConsultationRequired(
       String customerConsultationRequired) {
     this.customerConsultationRequired = customerConsultationRequired;
   }
   /**
    * @return Returns the customerType.
    */
   public String getCustomerType() {
     return customerType;
   }
   /**
    * @param customerType The customerType to set.
    */
   public void setCustomerType(String customerType) {
     this.customerType = customerType;
   }
   /**
    * @return Returns the formReference.
    */
   public String getFormReference() {
     return formReference;
   }
   /**
    * @param formReference The formReference to set.
    */
   public void setFormReference(String formReference) {
     this.formReference = formReference;
   }
   /**
    * @return Returns the formNumber.
    */
   public String getFormNumber() {
     return formNumber;
   }
   /**
    * @param formNumber The formNumber to set.
    */
   public void setFormNumber(String formNumber) {
     this.formNumber = formNumber;
   }
   /**
    * @return Returns the embeddedNetworkParentName.
    */
   public String getEmbeddedNetworkParentName() {
     return embeddedNetworkParentName;
   }
   /**
    * @param embeddedNetworkParentName The embeddedNetworkParentName to set.
    */
   public void setEmbeddedNetworkParentName(String embeddedNetworkParentName) {
     this.embeddedNetworkParentName = embeddedNetworkParentName;
   }
   /**
    * @return Returns the installationType.
    */
   public String getInstallationType() {
     return installationType;
   }
   /**
    * @param installationType The installationType to set.
    */
   public void setInstallationType(String installationType) {
     this.installationType = installationType;
   }
   /**
    * @return Returns the licenseeName.
    */
   public Licensee getLicenseeName() {
     return licenseeName;
   }
   /**
    * @param licenseeName The licenseeName to set.
    */
   public void setLicenseeName(Licensee licenseeName) {
     this.licenseeName = licenseeName;
   }
   /**
    * @return Returns the licenseePhone numbers.
    */
   public Telephone getLicenseePhone1() {
     return licenseePhone1;
   }
   public Telephone getLicenseePhone2() {
     return licenseePhone2;
   }
   public Telephone getLicenseePhone3() {
     return licenseePhone3;
   }
   /**
    * @param licenseePhone numbers The licenseePhone numbers to set.
    */
   public void setLicenseePhone1(Telephone licenseePhone) {
     this.licenseePhone1 = licenseePhone;
   }
   public void setLicenseePhone2(Telephone licenseePhone) {
     this.licenseePhone2 = licenseePhone;
   }
   public void setLicenseePhone3(Telephone licenseePhone) {
     this.licenseePhone3 = licenseePhone;
   }
   /**
    * @return Returns the maximumDemand.
    */
   public String getMaximumDemand() {
     return maximumDemand;
   }
   /**
    * @param maximumDemand The maximumDemand to set.
    */
   public void setMaximumDemand(String maximumDemand) {
     this.maximumDemand = maximumDemand;
   }
   /**
    * @return Returns the meteringRequired.
    */
   public String getMeteringRequired() {
     return this.meteringRequired;
   }
   /**
    * @param meteringRequired The meteringRequired to set.
    */
   public void setMeteringRequired(String meteringRequired) {
     this.meteringRequired = meteringRequired;
   }
   /**
    * @return Returns the meterInstallCode.
    */
   public String getMeterInstallCode() {
     return this.meterInstallCode;
   }
   /**
    * @param meterInstallCode The meterInstallCode to set.
    */
   public void setMeterInstallCode(String meterInstallCode) {
     this.meterInstallCode = meterInstallCode;
   }
 /**
    * @param meterSerialNumber The meterSerialNumber to set.
    */
   public void setMeterSerialNumber(String meterSerialNumber) {
     this.meterSerialNumber = meterSerialNumber;
   }

  /**
    * @return Returns the meterSerialNumber.
    */
   public String getMeterSerialNumber() {
     return this.meterSerialNumber;
   }


     /**
         * @param meterSerialNumber1 The meterSerialNumber1 to set.
         */
        public void setMeterSerialNumber1(String meterSerialNumber1) {
           this.meterSerialNumber1 = meterSerialNumber1;
        }
        /**
         * @return Returns the meterSerialNumber1.
         */
        public String getMeterSerialNumber1() {
           return meterSerialNumber1;
        }
        /**
         * @param meterSerialNumber2 The meterSerialNumber2 to set.
         */
        public void setMeterSerialNumber2(String meterSerialNumber2) {
           this.meterSerialNumber2 = meterSerialNumber2;
        }
        /**
         * @return Returns the meterSerialNumber2.
         */
        public String getMeterSerialNumber2() {
           return meterSerialNumber2;
        }
        /**
         * @param meterSerialNumber3 The meterSerialNumber3 to set.
         */
        public void setMeterSerialNumber3(String meterSerialNumber3) {
           this.meterSerialNumber3 = meterSerialNumber3;
        }
        /**
         * @return Returns the meterSerialNumber3.
         */
        public String getMeterSerialNumber3() {
           return meterSerialNumber3;
        }
    /**
    * @return Returns the nmi.
    */
   public String getNMI() {
     return NMI;
   }
   /**
    * @param nmi The nmi to set.
    */
   public void setNMI(String nmi) {
     this.NMI = nmi;
   }
   /**
    * @return Returns the nMIChecksum.
    */
   public String getNMIChecksum() {
     return NMIChecksum;
   }
   /**
    * @param checksum The nMIChecksum to set.
    */
   public void setNMIChecksum(String checksum) {
     NMIChecksum = checksum;
   }
   /**
    * @return Returns the nMIStatusCode.
    */
   public String getNMIStatusCode() {
     return NMIStatusCode;
   }
   /**
    * @param statusCode The nMIStatusCode to set.
    */
   public void setNMIStatusCode(String statusCode) {
     NMIStatusCode = statusCode;
   }
   /**
    * @return Returns the offPeakRequirement.
    */
   public String getOffPeakRequirement1() {
     return this.offPeakRequirement1;
   }
   /**
    * @param offPeakRequirement The offPeakRequirement to set.
    */
   public void setOffPeakRequirement1(String offPeakRequirement) {
     this.offPeakRequirement1 = offPeakRequirement;
   }
   /**
    * @return Returns the offPeakRequirement.
    */
   public String getOffPeakRequirement2() {
     return this.offPeakRequirement2;
   }
   /**
    * @param offPeakRequirement The offPeakRequirement to set.
    */
   public void setOffPeakRequirement2(String offPeakRequirement) {
     this.offPeakRequirement2 = offPeakRequirement;
   }
   /**
    * @return Returns the offPeakRequirement.
    */
   public String getOffPeakRequirement3() {
     return this.offPeakRequirement3;
   }
   /**
    * @param offPeakRequirement The offPeakRequirement to set.
    */
   public void setOffPeakRequirement3(String offPeakRequirement) {
     this.offPeakRequirement3 = offPeakRequirement;
   }
   /**
    * @return Returns the preferredDate.
    */
   public String getPreferredDate() {
     if (preferredDate == null || preferredDate.equals("")) {
       try {
         B2BDateFormatter df = new B2BDateFormatter(getPreferredDisplayDate(), B2BDateFormatter.B2BDATE);
         this.preferredDate = df.getXMLDate();
       } catch (Exception e) {
         this.preferredDate = "";
       }
     }
     return this.preferredDate;
   }
   /**
    * @param preferredDate The preferredDate to set.
    */
   public void setPreferredDate(String preferredDate) {
     this.isPreferredDateSet = true;
     this.preferredDate = preferredDate;
     try {
       B2BDateFormatter df = new B2BDateFormatter(preferredDate, B2BDateFormatter.XMLDATE);
       this.preferredDisplayDate = df.getB2BDate();
     } catch (Exception e) {
       this.preferredDisplayDate = "";
     }
   }
   /**
    * @return Returns the preferredDisplayDate.
    */
   public String getPreferredDisplayDate() {
     return this.preferredDisplayDate;
   }
   /**
    * @param preferredDisplayDate The preferredDisplayDate to set.
    */
   public void setPreferredDisplayDate(String preferredDisplayDate) {
     this.isPreferredDateSet = false;
     this.preferredDisplayDate = preferredDisplayDate;
   }
   /**
    * @return Returns the preferredTime.
    */
   public String getPreferredTime() {
     if (preferredTime == null || preferredTime.equals("")) {
       try {
         B2BTimeFormatter tf = new B2BTimeFormatter(getPreferredDisplayTime(), B2BTimeFormatter.B2BTIME);
         this.preferredTime = tf.getXMLTime();
       } catch (Exception e) {
         this.preferredTime = "";
       }
     }
     return this.preferredTime;
   }
   /**
    * @param preferredTime The preferredTime to set.
    */
   public void setPreferredTime(String preferredTime) {
     this.isPreferredTimeSet = true;
     this.preferredTime = preferredTime;
     try {
       B2BTimeFormatter df = new B2BTimeFormatter(preferredTime, B2BTimeFormatter.XMLTIME);
       this.preferredDisplayTime = df.getB2BTime();
     } catch (Exception e) {
       this.preferredDisplayTime = "";
     }
   }
   /**
    * @return Returns the preferredDisplayTime.
    */
   public String getPreferredDisplayTime() {
     return this.preferredDisplayTime;
   }
   /**
    * @param preferredDisplayTime The preferredTime to set.
    */
   public void setPreferredDisplayTime(String preferredDisplayTime) {
     this.isPreferredTimeSet = false;
     this.preferredDisplayTime = preferredDisplayTime;
   }
   /**
    * @return Returns the proposedTariff.
    */
   public String getProposedTariff() {
     return this.proposedTariff;
   }
   /**
    * @param proposedTariff The proposedTariff to set.
    */
   public void setProposedTariff(String proposedTariff) {
     this.proposedTariff = proposedTariff;
   }

    /**
     * @param proposedTariff1 The proposedTariff1 to set.
     */
    public void setProposedTariff1(String proposedTariff1) {
       this.proposedTariff1 = proposedTariff1;
    }
    /**
     * @return Returns the proposedTariff1.
     */
    public String getProposedTariff1() {
       return proposedTariff1;
    }
    /**
     * @param proposedTariff2 The proposedTariff2 to set.
     */
    public void setProposedTariff2(String proposedTariff2) {
       this.proposedTariff2 = proposedTariff2;
    }
    /**
     * @return Returns the proposedTariff2.
     */
    public String getProposedTariff2() {
       return proposedTariff2;
    }
    /**
     * @param proposedTariff3 The proposedTariff3 to set.
     */
    public void setProposedTariff3(String proposedTariff3) {
       this.proposedTariff3 = proposedTariff3;
    }
    /**
     * @return Returns the proposedTariff3.
     */
    public String getProposedTariff3() {
       return proposedTariff3;
    }

   /**
    * @return Returns the recId.
    */
   public String getRecId() {
     return recId;
   }
   /**
    * @param recId The recId to set.
    */
   public void setRecId(String recId) {
     this.recId = recId;
   }
   /**
    * @return Returns the retailerName.
    */
   public PersonName getRetailerName() {
     return this.retailerName;
   }
   /**
    * @param retailerName The retailerName to set.
    */
   public void setRetailerName(PersonName retailerName) {
     this.retailerName = retailerName;
   }
   /**
    * @return Returns the retailerPhone numbers.
    */
   public Telephone getRetailerPhone1() {
     return this.retailerPhone1;
   }
   public Telephone getRetailerPhone2() {
     return this.retailerPhone2;
   }
   public Telephone getRetailerPhone3() {
     return this.retailerPhone3;
   }
   /**
    * @param retailerPhone numbers The retailerPhone numbers to set.
    */
   public void setRetailerPhone1(Telephone retailerPhone) {
     this.retailerPhone1 = retailerPhone;
   }
   public void setRetailerPhone2(Telephone retailerPhone) {
     this.retailerPhone2 = retailerPhone;
   }
   public void setRetailerPhone3(Telephone retailerPhone) {
     this.retailerPhone3 = retailerPhone;
   }
   /**
    * @return Returns the retServiceOrder.
    */
   public String getRetServiceOrder() {
     return this.retServiceOrder;
   }
   /**
    * @param retServiceOrder The retServiceOrder to set.
    */
   public void setRetServiceOrder(String retServiceOrder) {
     this.retServiceOrder = retServiceOrder;
   }
   /**
    * @return Returns the safetyCertificateId.
    */
   public String getSafetyCertificateId() {
     return this.safetyCertificateId;
   }
   /**
    * @param safetyCertificateId The safetyCertificateId to set.
    */
   public void setSafetyCertificateId(String safetyCertificateId) {
     this.safetyCertificateId = safetyCertificateId;
   }
   /**
    * @return Returns the safetyCertificateIdMethodSent.
    */
   public String getSafetyCertificateIdMethodSent() {
     return this.safetyCertificateIdMethodSent;
   }
   /**
    * @param safetyCertificateIdMethodSent The safetyCertificateIdMethodSent to set.
    */
   public void setSafetyCertificateIdMethodSent(
       String safetyCertificateIdMethodSent) {
     this.safetyCertificateIdMethodSent = safetyCertificateIdMethodSent;
   }
   /**
    * @return Returns the scheduleDate.
    */
   public String getScheduleDate() {
     if (scheduleDate == null || scheduleDate.equals("")) {
       try {
         B2BDateFormatter df = new B2BDateFormatter(getScheduleDisplayDate(), B2BDateFormatter.B2BDATE);
         this.scheduleDate = df.getXMLDate();
       } catch (Exception e) {
         this.scheduleDate = "";
       }
     }
     return this.scheduleDate;
   }
   /**
    * @param scheduleDate The scheduleDate to set.
    */
   public void setScheduleDate(String scheduleDate) {
     this.isScheduleDateSet = true;
     this.scheduleDate = scheduleDate;
     try {
       B2BDateFormatter df = new B2BDateFormatter(scheduleDate, B2BDateFormatter.XMLDATE);
       this.scheduleDisplayDate = df.getB2BDate();
     } catch (Exception e) {
       this.scheduleDisplayDate = "";
     }
   }

   /**
    * @return Returns the scheduleDisplayDate.
    */
   public String getScheduleDisplayDate() {
     return this.scheduleDisplayDate;
   }
   /**
    * @param scheduleDisplayDate The scheduleDisplayDate to set.
    */
   public void setScheduleDisplayDate(String scheduleDisplayDate) {
     this.isScheduleDateSet = false;
     this.scheduleDisplayDate = scheduleDisplayDate;
   }
   /**
    * @return Returns the serviceOrderSubType.
    */
   public String getServiceOrderSubType() {
     return this.serviceOrderSubType;
   }
   /**
    * @param serviceOrderSubType The serviceOrderSubType to set.
    */
   public void setServiceOrderSubType(String serviceOrderSubType) {
     this.serviceOrderSubType = serviceOrderSubType;
   }
   /**
    * @return Returns the serviceOrderType.
    */
   public String getServiceOrderType() {
     return this.serviceOrderType;
   }
   /**
    * @param serviceOrderType The serviceOrderType to set.
    */
   public void setServiceOrderType(String serviceOrderType) {
     this.serviceOrderType = serviceOrderType;
   }
   /**
    * @return Returns the serviceTime.
    */
   public String getServiceTime() {
     return this.serviceTime;
   }
   /**
    * @param serviceTime The serviceTime to set.
    */
   public void setServiceTime(String serviceTime) {
     this.serviceTime = serviceTime;
   }
   /**
    * @return Returns the specialInstructions.
    */
   public String getSpecialInstructions1() {
     return this.specialInstructions1;
   }
   /**
    * @param specialInstructions1 The specialInstructions to set.
    */
   public void setSpecialInstructions1(String specialInstructions1) {
     this.specialInstructions1 = specialInstructions1;
   }
   /**
    * @return Returns the specialInstructions.
    */
   public String getSpecialInstructions2() {
     return this.specialInstructions2;
   }
   /**
    * @param specialInstructions2 The specialInstructions to set.
    */
   public void setSpecialInstructions2(String specialInstructions2) {
     this.specialInstructions2 = specialInstructions2;
   }
   /**
    * @return Returns the specialInstructions.
    */
   public String getSpecialInstructions3() {
     return this.specialInstructions3;
   }
   /**
    * @param specialInstructions3 The specialInstructions to set.
    */
   public void setSpecialInstructions3(String specialInstructions3) {
     this.specialInstructions3 = specialInstructions3;
   }
   /**
    * @return Returns the supplyPhases.
    */
   public String getSupplyPhase() {
     return this.supplyPhase;
   }
   /**
    * @param supplyPhase The supplyPhases to set.
    */
   public void setSupplyPhase(String supplyPhase) {
     this.supplyPhase = supplyPhase;
   }
   /**
    * @return Returns the switchingServiceRequired.
    */
   public String getSwitchingServiceRequired() {
     return this.switchingServiceRequired;
   }
   /**
    * @param switchingServiceRequired The switchingServiceRequired to set.
    */
   public void setSwitchingServiceRequired(String switchingServiceRequired) {
     this.switchingServiceRequired = switchingServiceRequired;
   }

   /**
    * @return Returns the hazardDescription.
    */
   public String getHazardDescription1() {
     return this.hazardDescription1;
   }
   /**
    * @param hazardDescription1 The hazardDescription to set.
    */
   public void setHazardDescription1(String hazardDescription1) {
     this.hazardDescription1 = hazardDescription1;
   }

   /**
    * @return Returns the hazardDescription.
    */
   public String getHazardDescription2() {
     return this.hazardDescription2;
   }
   /**
    * @param hazardDescription2 The hazardDescription to set.
    */
   public void setHazardDescription2(String hazardDescription2) {
     this.hazardDescription2 = hazardDescription2;
   }

   /**
    * @return Returns the hazardDescription.
    */
   public String getHazardDescription3() {
     return this.hazardDescription3;
   }
   /**
    * @param hazardDescription3 The hazardDescription to set.
    */
   public void setHazardDescription3(String hazardDescription3) {
     this.hazardDescription3 = hazardDescription3;
   }

   /**
    * @return Returns the party1.
    */
   public String getParty1() {
     return party1;
   }
   /**
    * @param party1 The party1 to set.
    */
   public void setParty1(String party1) {
     this.party1 = party1;
   }
   /**
    * @return Returns the party2.
    */
   public String getParty2() {
     return party2;
   }
   /**
    * @param party2 The party2 to set.
    */
   public void setParty2(String party2) {
     this.party2 = party2;
   }
   /**
    * @return Returns the party3.
    */
   public String getParty3() {
     return party3;
   }
   /**
    * @param party3 The party3 to set.
    */
   public void setParty3(String party3) {
     this.party3 = party3;
   }
   /**
    * @return Returns the party4.
    */
   public String getParty4() {
     return party4;
   }
   /**
    * @param party4 The party4 to set.
    */
   public void setParty4(String party4) {
     this.party4 = party4;
   }
   /**
    * @return Returns the recAttendanceRequired.
    */
   public String getRecAttendanceRequired() {
     return recAttendanceRequired;
   }
   /**
    * @param recAttendanceRequired The recAttendanceRequired to set.
    */
   public void setRecAttendanceRequired(String recAttendanceRequired) {
     this.recAttendanceRequired = recAttendanceRequired;
   }
   /**
    * @return Returns the role1.
    */
   public String getRole1() {
     return role1;
   }
   /**
    * @param role1 The role1 to set.
    */
   public void setRole1(String role1) {
     this.role1 = role1;
   }
   /**
    * @return Returns the role2.
    */
   public String getRole2() {
     return role2;
   }
   /**
    * @param role2 The role2 to set.
    */
   public void setRole2(String role2) {
     this.role2 = role2;
   }
   /**
    * @return Returns the role3.
    */
   public String getRole3() {
     return role3;
   }
   /**
    * @param role3 The role3 to set.
    */
   public void setRole3(String role3) {
     this.role3 = role3;
   }
   /**
    * @return Returns the role4.
    */
   public String getRole4() {
     return role4;
   }
   /**
    * @param role4 The role4 to set.
    */
   public void setRole4(String role4) {
     this.role4 = role4;
   }


   /**
    * @return Returns the actionType list
    */
   public List<? extends Object> getActionTypes() {
     return ActionType.ACTION_TYPE_CODES;
   }

   /**
    * @return Returns the serviceTime list
    */
   public List<? extends Object> getServiceTimes() {
     return ServiceTime.SERVICE_TIME_TYPES;
   }

   /**
    * @return Returns the yesNo list
    */
   public List<? extends Object> getYesNo() {
     return YesNo.YESNO_CODES;
   }

   /**
    * @return Returns the serviceOrderType list
    */
   public List<? extends Object> getServiceOrderTypes() {
     return ServiceOrderType.SERVICE_ORDER_TYPES;
   }

   /**
    * @return Returns the serviceOrderSubType list
    */
   public List<? extends Object> getServiceOrderSubTypes() {
     return ServiceOrderSubType.getSubTypeList(serviceOrderType);
   }

   /**
    * @return Returns the installationType list
    */
   public List<? extends Object> getInstallationTypes() {
     return InstallationType.INSTALLATION_TYPES;
   }

   /**
    * @return Returns the supplyPhases list
    */
   public List<? extends Object> getSupplyPhases() {
     return SupplyPhase.SUPPLY_PHASE_TYPES;
   }

   /**
    * @return Returns the customerType list
    */
   public List<? extends Object> getCustomerTypes() {
     return CustomerType.CUSTOMER_TYPES;
   }

   /**
    * @return Returns the safetyCertificateIdMethodSent list
    */
   public List<? extends Object> getSafetyCertificateIdMethodSents() {
     return SafetyCertMethodSentType.SAFETY_CERT_METHOD_SENT_TYPES;
   }

   /**
    * @return Returns the meteringType list
    */
   public List<? extends Object> getMeteringTypes() {
     return MeteringType.METERING_TYPES;
   }

   /**
    * @return Returns the embeddedNetworkParentName list
    */
   public List<EmbeddedNetworkParentName> getEmbeddedNetworkParentNames() {
     try {
       return EmbeddedNetworkParentName.getList();
     } catch (Exception e) {
       return (List<EmbeddedNetworkParentName>)null;
     }
   }

   /**
    * @return Returns the NMIStatusCode list
    */
   public List<? extends Object> getNMIStatusCodes() {
     return com.capgemini.nemmco.b2bb.enums.NMIStatusCode.NMI_STATUS_CODES;
   }

   /**
    * @return Returns the meterInstallCode list
    */
   public List<? extends Object> getMeterInstallCodes() {
     return MeterInstallCode.METER_INSTALL_CODES;
   }

   public List<Participant> getParticipants() {
     try {
       return Participant.getList();
     } catch (Exception e) {
       return (List<Participant>)null;
     }
   }


    /**
     * @return Returns the businessName
    */
    public String getBusinessName() {
      return businessName;
    }

   /**
    * @param businessName
    */
    public void setBusinessName(String businessName) {
      this.businessName = businessName;
    }

    /**
     * @return Returns the businessContactName
    */
    public PersonName getBusinessContactName() {
      return businessContactName;
    }

    /**
     * @param businessContactName
     */

    public void setBusinessContactName(PersonName businessContactName) {
      this.businessContactName = new ContactName(businessContactName);
    }

  /**
    * @return Returns the lifeSupport.
    */
   public String getLifeSupport() {
     return lifeSupport;
   }
   /**
    * @param lifeSupport The lifeSupport to set.
    */
   public void setLifeSupport(String lifeSupport) {
     this.lifeSupport = lifeSupport;
   }

    /**
     * Getter for yesNoTypes
     * @return List
     */
    public List<? extends Object> getYesNoTypes() {
       return  YesNo.YESNO_CODES;

    }
 /**
     * @return Returns the notified party list
     */
    public List<Participant> getNotifiedPartyIds() {
      List<Participant> toList = null;
      try {

         if(sendToAnyParticipant.equals("true")){
           toList = Participant.getAllParticipants();
         }
         else{
            toList = Participant.getList(transactionDesc,"NP");
         }

      } catch (Exception e) {
        toList = null;
      }

      return toList;
    }//end of getTos()
  /**
    * @return Returns the notifiedPartyId1.
    */
   public String getNotifiedPartyId1() {
     return notifiedPartyId1;
   }
   /**
    * @param notifiedPartyId1 The notifiedPartyId1 to set.
    */
   public void setNotifiedPartyId1(String notifiedPartyId1) {
     this.notifiedPartyId1 = notifiedPartyId1;
   }

  /**
    * @return Returns the notifiedPartyId2.
    */
   public String getNotifiedPartyId2() {
     return notifiedPartyId2;
   }
   /**
    * @param notifiedPartyId2 The notifiedPartyId1 to set.
    */
   public void setNotifiedPartyId2(String notifiedPartyId2) {
     this.notifiedPartyId2 = notifiedPartyId2;
   }
  
   /**
    * @return Returns the notifiedPartyId3.
    */
   public String getNotifiedPartyId3() {
     return notifiedPartyId3;
   }
   /**
    * @param notifiedPartyId3 The notifiedPartyId3 to set.
    */
   public void setNotifiedPartyId3(String notifiedPartyId3) {
     this.notifiedPartyId3 = notifiedPartyId3;
   }
  
   /**
    * @return Returns the meteringSafetyCertificateId.
    */
   public String getMeteringSafetyCertificateId() {
     return meteringSafetyCertificateId;
   }
   /**
    * @param meteringSafetyCertificateId The meteringSafetyCertificateId to set.
    */
   public void setMeteringSafetyCertificateId(String meteringSafetyCertificateId) {
     this.meteringSafetyCertificateId = meteringSafetyCertificateId;
   }
  
   /**
    * @return Returns the meteringSafetyCertificateIdMethodSent.
    */
   public String getMeteringSafetyCertificateIdMethodSent() {
     return meteringSafetyCertificateIdMethodSent;
   }
   /**
    * @param meteringSafetyCertificateIdMethodSent The meteringSafetyCertificateMethodSent to set.
    */
   public void setMeteringSafetyCertificateIdMethodSent(String meteringSafetyCertificateIdMethodSent) {
     this.meteringSafetyCertificateIdMethodSent = meteringSafetyCertificateIdMethodSent;
   }
  
   /**
    * @return Returns the coOrdinationRequired.
    */
   public String getCoOrdinationRequired() {
     return coOrdinationRequired;
   }
   /**
    * @param coOrdinationRequired The coOrdinationRequired to set.
    */
   public void setCoOrdinationRequired(String coOrdinationRequired) {
     this.coOrdinationRequired = coOrdinationRequired;
   }

    /**
     * @return Returns the coOrdinatingContactName
    */
    public PersonName getCoOrdinatingContactName() {
      return coOrdinatingContactName;
    }

    /**
     * @param coOrdinatingContactName
     */

    public void setCoOrdinatingContactName(PersonName coOrdinatingContactName) {
      this.coOrdinatingContactName = coOrdinatingContactName;
    }

    /**
     * @return Returns the coOrdinatingContactPhone numbers.
     */
    public Telephone getCoOrdinatingContactPhone1() {
      return coOrdinatingContactPhone1;
    }
    public Telephone getCoOrdinatingContactPhone2() {
      return coOrdinatingContactPhone2;
    }
    public Telephone getCoOrdinatingContactPhone3() {
      return coOrdinatingContactPhone3;
    }
    /**
     * @param contactPhone numbers The contactPhone numbers to set.
     */
    public void setCoOrdinatingContactPhone1(Telephone contactPhone) {
      this.coOrdinatingContactPhone1 = contactPhone;
    }
    public void setCoOrdinatingContactPhone2(Telephone contactPhone) {
      this.coOrdinatingContactPhone2 = contactPhone;
    }
    public void setCoOrdinatingContactPhone3(Telephone contactPhone) {
      this.coOrdinatingContactPhone3 = contactPhone;
    }
    
    
    /**
     * @return Returns the deEnergisationReason.
     */
    public String getDeEnergisationReason() {
      return deEnergisationReason;
    }
    /**
     * @param coOrdinationRequired The coOrdinationRequired to set.
     */
    public void setDeEnergisationReason(String deEnergisationReason) {
      this.deEnergisationReason = deEnergisationReason;
    }

    /**
     * Getter for deEnergisationReasons
     * @return List
     */
    public List<? extends Object> getDeEnergisationReasons() {
       return DeEnergisationReason.DEENERGISATION_REASON;
    }
    /**
     * @return Returns the true/false list
     */
    public List<? extends Object> getTrueFalse() {
      return TrueFalse.TRUEFALSE_CODES;
    }

    public void setSendToAnyParticipant(String sendToAnyParticipant) {
       this.sendToAnyParticipant = sendToAnyParticipant;
    }

    public String getSendToAnyParticipant() {
       return this.sendToAnyParticipant;
    }

    public void setCoordinatingBusinessContactName(PersonName coordinatingBusinessContactName) {
        this.coordinatingBusinessContactName = new ContactName(coordinatingBusinessContactName);
    }

    public PersonName getCoordinatingBusinessContactName() {
        return coordinatingBusinessContactName;
    }

    public void setCoordinatingBusinessName(String coordinatingBusinessName) {
        this.coordinatingBusinessName = coordinatingBusinessName;
    }

    public String getCoordinatingBusinessName() {
        return coordinatingBusinessName;
    }
}

