/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/SiteAccessNotification.java-arc   1.14   Mar 29 2005 11:04:20   adij  $
 * $Revision:   1.14  $
 * $Date:   Mar 29 2005 11:04:20  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 */

package com.capgemini.nemmco.b2bb;

import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.utils.B2BDateTimeFormatter;

/**
 * The class <code>SiteAccessNotification</code> represents a SiteAccessNotification
 * Transaction.
 * 
 * @author $Author:   adij  $
 * @version $Revision:   1.14  $ $Date:   Mar 29 2005 11:04:20  $
 */
public class SiteAccessNotification extends Transaction {
  public static final String ACCESS_DETAILS = "accessDetails";
  public static final String HAZARD_DESCRIPTION1 = "hazardDescription1";
  public static final String HAZARD_DESCRIPTION2 = "hazardDescription2";
  public static final String HAZARD_DESCRIPTION3 = "hazardDescription3";
  public static final String LAST_MODIFIED_DATE_TIME = "lastModifiedDateTime";
  public static final String LAST_MODIFIED_DISPLAY_DATE = "lastModifiedDisplayDate";
  public static final String LAST_MODIFIED_DISPLAY_TIME = "lastModifiedDisplayTime";

  private String accessDetails = "";
  private String hazardDescription1 = "";
  private String hazardDescription2 = "";
  private String hazardDescription3 = "";
  private String lastModifiedDateTime = "";
  private String lastModifiedDisplayDate = "";
  private String lastModifiedDisplayTime = "";
  private boolean lastModifiedDateTimeSet;
  
  /**
   * Default Constructor
   */
  public SiteAccessNotification() {
    super(TransactionType.SITE_ACCESS_NOT);
  }
  
  /**
   * @return Returns the accessDetails.
   */
  public String getAccessDetails() {
    return accessDetails;
  }
  
  /**
   * @return Returns the hazardDescription1.
   */
  public String getHazardDescription1() {
    return hazardDescription1;
  }
  
  /**
   * @return Returns the hazardDescription2.
   */
  public String getHazardDescription2() {
    return hazardDescription2;
  }
  
  /**
   * @return Returns the hazardDescription3.
   */
  public String getHazardDescription3() {
    return hazardDescription3;
  }
 
  /**
   * @return Returns the lastModifiedDateTime.
   */
  public String getLastModifiedDateTime() {
    if (lastModifiedDateTime == null || lastModifiedDateTime.equals("")) {
      try {
        B2BDateTimeFormatter df = new B2BDateTimeFormatter(getLastModifiedDisplayDateTime(), B2BDateTimeFormatter.B2BDATE);
        this.lastModifiedDateTime = df.getXMLFormattedDateTime();
      } catch (Exception e) {
        this.lastModifiedDateTime = "";
      }
    }
    
    return lastModifiedDateTime;
  }
  
  /**
   * @return Returns the lastModifiedDisplayDate.
   */
  public String getLastModifiedDisplayDate() {
    return lastModifiedDisplayDate;
  }
  
  /**
   * @return Returns the lastModifiedDisplayTime.
   */
  public String getLastModifiedDisplayDateTime() {
    return lastModifiedDisplayDate + " " + lastModifiedDisplayTime;
  }
  
  /**
   * @return Returns the lastModifiedDisplayTime.
   */
  public String getLastModifiedDisplayTime() {
    return lastModifiedDisplayTime;
  }
  
  /**
   * @param accessDetails The accessDetails to set.
   */
  public void setAccessDetails(String accessDetails) {
    this.accessDetails = accessDetails;
  }
  
  /**
   * @param hazardDescription1 The hazardDescription1 to set.
   */
  public void setHazardDescription1(String hazardDescription1) {
    this.hazardDescription1 = hazardDescription1;
  }
  
  /**
   * @param hazardDescription2 The hazardDescription2 to set.
   */
  public void setHazardDescription2(String hazardDescription2) {
    this.hazardDescription2 = hazardDescription2;
  }
  
  /**
   * @param hazardDescription3 The hazardDescription3 to set.
   */
  public void setHazardDescription3(String hazardDescription3) {
    this.hazardDescription3 = hazardDescription3;
  }
  
  /**
   * @param lastModifiedDateTime The lastModifiedDateTime to set.
   */
  public void setLastModifiedDateTime(String lastModifiedDateTime) {
    this.lastModifiedDateTimeSet = true;
    this.lastModifiedDateTime = lastModifiedDateTime;
    try {
      // split
      B2BDateTimeFormatter df = new B2BDateTimeFormatter(lastModifiedDateTime, B2BDateTimeFormatter.XMLDATE);
      this.lastModifiedDisplayDate = df.getB2BDate();
      this.lastModifiedDisplayTime = df.getB2BTime();
    } catch (Exception e) {
      this.lastModifiedDisplayDate = "";
      this.lastModifiedDisplayTime = "";
    }
  }
  
  /**
   * @param lastModifiedDisplayDate The lastModifiedDisplayDate to set.
   */
  public void setLastModifiedDisplayDate(String lastModifiedDisplayDate) {
    this.lastModifiedDateTimeSet = false;
    this.lastModifiedDisplayDate = lastModifiedDisplayDate;
  }
  
  /**
   * @param lastModifiedDisplayTime The lastModifiedDisplayTime to set.
   */
  public void setLastModifiedDisplayTime(String lastModifiedDisplayTime) {
    this.lastModifiedDateTimeSet = false;
    this.lastModifiedDisplayTime = lastModifiedDisplayTime;
  }
  
  /**
   * @return Returns the lastModifiedDateTimeSet.
   */
  public boolean isLastModifiedDateTimeSet() {
    return lastModifiedDateTimeSet;
  }
  
  /**
   * @param lastModifiedDateTimeSet The lastModifiedDateTimeSet to set.
   */
  public void setLastModifiedDateTimeSet(boolean lastModifiedDateTimeSet) {
    this.lastModifiedDateTimeSet = lastModifiedDateTimeSet;
  }
}

