/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/Transaction.java-arc   1.15   11 Apr 2005 11:10:36   RayanS  $
 * $Revision:   1.15  $
 * $Date:   11 Apr 2005 11:10:36  $
 *
 * Copyright 2005 Capgemini Australia Pty. Ltd.
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 *
 * Modification History
 *
 * Date Changed   Changed By     Comments
 * ------------   ----------     ------------------------------------------
 * 27-Mar-2009    Eddie Graham   Defect 4793. Add transaction version.
 * 27-June-2017   NG                Created for 7135 - PoC SMP Changes
 */

package com.capgemini.nemmco.b2bb;

import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.exceptions.UnableToLookupResourceException;
import com.capgemini.nemmco.b2bb.exceptions.UnableToParseXMLException;
import com.capgemini.nemmco.b2bb.lookup.XPath;
import com.capgemini.nemmco.b2bb.utils.B2BBDOMHelper;
import com.capgemini.nemmco.b2bb.utils.B2BDateTimeFormatter;
import org.w3c.dom.Node;
import javax.xml.transform.TransformerException;

/**
 * The class <code>Transaction</code> is the interface definition of a Transaction
 * object.
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.15  $ $Date:   11 Apr 2005 11:10:36  $
 */
public abstract class Transaction extends XMLObject {

  /**
   * @param transactionType
   */
  public Transaction(TransactionType transactionType) {
    super(transactionType);
  }

  public static String INIT_TRANSACTION_ID = "initiatingTransactionID";
  public static String NMI_CHECKSUM = "NMIChecksum";
  public static String NMI_FIELD = "NMI";
  public static String TRANSACTION_DATE = "transactionDate";
  public static String TRANSACTION_ID = "transactionID";

  protected int dateFormat = B2BDateTimeFormatter.XMLDATE;
  protected String initiatingTransactionID = "";
  protected String NMI = "";
  protected String NMIChecksum = "";
  protected String transactionDate = "";
  protected boolean transactionDateSet;
  protected String transactionDisplayDate = "";
  protected String transactionID = "";
  //NG Added 27-06-2017
  public String transactionType="";
  public String transactionSubType="";

  // 27-Mar-2009 Eddie Graham Defect 4793
  private String transactionVersion;

  public boolean fieldsExists() {
    return exists(this.initiatingTransactionID)
    		|| exists(this.NMI)
				|| exists(this.NMIChecksum)
				|| exists(this.transactionDate)
				|| exists(this.transactionID);
  }

  /**
   * @return Returns the initiatingTransactionID.
   */
  public String getInitiatingTransactionID() {
    return initiatingTransactionID;
  }

  /**
   * @return Returns the nMI.
   */
  public String getNMI() {
    return NMI;
  }

  /**
   * @return Returns the nMIChecksum.
   */
  public String getNMIChecksum() {
    return NMIChecksum;
  }

  /**
   * @return Returns the transactionDate.
   */
  public String getTransactionDate() {
    return transactionDate;
  }

  /**
   * @return Returns the messageDisplayDate.
   */
  public String getTransactionDisplayDate() {
    return transactionDisplayDate;
  }

  /**
   * @return Returns the transactionID.
   */
  public String getTransactionID() {
    return transactionID;
  }

  /**
   *
   * @return The transaction type of the transaction
   */
  public TransactionType getTransactionType() {
    return (TransactionType) xmlObjectType;
  }

  /**
   * @return Returns the transactionID.
   */
  public String getTransactionVersion() {
    return transactionVersion;
  }

  /**
   * @return Returns the transactionDateSet.
   */
  public boolean isTransactionDateSet() {
    return transactionDateSet;
  }

  /**
   * @param initiatingTransactionID The initiatingTransactionID to set.
   */
  public void setInitiatingTransactionID(String initiatingTransactionID) {
    this.initiatingTransactionID = initiatingTransactionID;
  }

  /**
   * @param nmi The nMI to set.
   */
  public void setNMI(String nmi) {
    NMI = nmi;
  }

  /**
   * @param checksum The nMIChecksum to set.
   */
  public void setNMIChecksum(String checksum) {
    NMIChecksum = checksum;
  }

  /**
   * @param transactionDate The transactionDate to set.
   */
  public void setTransactionDate(String transactionDate) {
    this.transactionDateSet = true;
    this.transactionDate = transactionDate;
    try {
      B2BDateTimeFormatter df = new B2BDateTimeFormatter(transactionDate, B2BDateTimeFormatter.XMLDATE);
      this.transactionDisplayDate = df.getB2BFormattedDateTime();
    } catch (Exception e) {
      this.transactionDisplayDate = "";
    }
  }

  /**
   * @param transactionDateSet The transactionDateSet to set.
   */
  public void setTransactionDateSet(boolean transactionDateSet) {
    this.transactionDateSet = transactionDateSet;
  }

  /**
   * @param transactionDisplayDate The messageDisplayDate to set.
   */
  public void setTransactionDisplayDate(String transactionDisplayDate) {
    this.transactionDateSet = false;
    this.transactionDisplayDate = transactionDisplayDate;
    try {
      B2BDateTimeFormatter df = new B2BDateTimeFormatter(transactionDisplayDate, B2BDateTimeFormatter.B2BDATE);
      this.transactionDate = df.getXMLFormattedDateTime();
    } catch (Exception e) {
      this.transactionDate = "";
    }
  }

  /**
   * @param transactionID The transactionID to set.
   */
  public void setTransactionID(String transactionID) {
    this.transactionID = transactionID;
  }

  /**
   * @param transactionVersion The transactionVersion to set.
   */
  public void setTransactionVersion(String transactionVersion) {
    this.transactionVersion = transactionVersion;
  }

  public void setDateFormat(int dateFormat)
  {
    this.dateFormat = dateFormat;
  }


  public int getDateFormat()
  {
    return dateFormat;
  }
  public String getXmlString(String fieldName) throws UnableToLookupResourceException, TransformerException, UnableToParseXMLException {
    String xPath = XPath.getXPath(xmlObjectType.getCategory(), fieldName);
    Node node = (Node) B2BBDOMHelper.getSelectedNode(this.xmlNode, XPath.removeFirst(xPath));
     return B2BBDOMHelper.nodeToString(node, "yes").
            replace("<"+fieldName.substring(0,1).toUpperCase()+fieldName.substring(1)+">", "").
            replace("</"+fieldName.substring(0,1).toUpperCase()+fieldName.substring(1)+">", "").
            replaceAll("\r\n", "").
            replaceAll("\n", "");
  }
}