/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/TransactionFactory.java-arc   1.5   11 Apr 2005 10:48:28   RayanS  $
 * $Revision:   1.5  $
 * $Date:   11 Apr 2005 10:48:28  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * Date Changed   Changed By        Comments
 * ------------   ----------        -----------
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb;

import java.util.List;

import org.w3c.dom.Node;

import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.exceptions.UnableToInstantiateClassException;
import com.capgemini.nemmco.b2bb.utils.B2BBDOMHelper;

/**
 * The class <code>TransactionFactory</code> is used to create the Transaction Objects.
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.5  $ $Date:   11 Apr 2005 10:48:28  $
 */
public class TransactionFactory {

  /**
   * Returns a Transaction object for a specified transaction
   * 
   * @param transactionType The String representation of the transaction
   * @return The correct transaction class
   * @throws UnableToInstantiateClassException
   */
  public static Transaction getTransaction(TransactionType transactionType) throws UnableToInstantiateClassException {
    try {
      //System.out.println(transactionType.getObjectClass());
      return (Transaction) Class.forName(transactionType.getObjectClass()).newInstance();
    } catch (IllegalAccessException ex) {
      throw new UnableToInstantiateClassException(ex.getMessage());
    } catch (InstantiationException ex) {
      throw new UnableToInstantiateClassException(ex.getMessage());
    } catch (ClassNotFoundException ex) {
      throw new UnableToInstantiateClassException(ex.getMessage());
    }
  }

  /**
   * Returns a Transaction object for a specified transaction
   * 
   * @param transactionNode org.w3c.dom.Node representation of the Transaction
   * @return the correct transaction class
   * @throws UnableToInstantiateClassException
   */
  public static Transaction getTransaction(Node transactionNode) throws UnableToInstantiateClassException {
    try {
      Node transactionTypeNode;
      Transaction transaction = null;

      if (transactionNode != null) {
        List<? extends Object> list = TransactionType.TRANSACTIONS;
        for (int i =0 ; i<list.size(); i++){
          TransactionType tt = (TransactionType)list.get(i);
          if ((transactionTypeNode = B2BBDOMHelper.getSelectedNode(transactionNode, tt.getXMLIdentifier())) != null) {
          	//System.out.println(tt.getObjectClass());
            transaction = (Transaction) Class.forName(tt.getObjectClass()).newInstance();
            transaction.setXMLString(transactionNode);
            return transaction;
          }
        }
      }

      return null;

    } catch (IllegalAccessException ex) {
      throw new UnableToInstantiateClassException(ex.getMessage());
    } catch (InstantiationException ex) {
      throw new UnableToInstantiateClassException(ex.getMessage());
    } catch (ClassNotFoundException ex) {
      throw new UnableToInstantiateClassException(ex.getMessage());
    } catch (Exception e) {
      throw new UnableToInstantiateClassException(e.getMessage());
    }
  }

}

