/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/XMLObject.java-arc   1.1   11 Apr 2005 11:10:36   RayanS  $
 * $Revision:   1.1  $
 * $Date:   11 Apr 2005 11:10:36  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * Date Changed   Changed By     Comments
 * ------------   ----------     -----------
 * 14-Aug-2012    Bob Jones      7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.xml.transform.TransformerException;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import com.capgemini.nemmco.b2bb.enums.XMLObjectType;
import com.capgemini.nemmco.b2bb.exceptions.UnableToLookupResourceException;
import com.capgemini.nemmco.b2bb.exceptions.UnableToParseXMLException;
import com.capgemini.nemmco.b2bb.lookup.XPath;
import com.capgemini.nemmco.b2bb.lookup.XPathComparator;
import com.capgemini.nemmco.b2bb.utils.B2BBDOMHelper;

/**
 * The class <code>XMLObject</code> is the interface definition of an XML
 * object.
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.1  $ $Date:   11 Apr 2005 11:10:36  $
 */
public abstract class XMLObject {

  protected final XMLObjectType xmlObjectType;
  protected Node xmlNode;

  public Node createXML() throws Exception {
    XPath xPath;
    String field;
    String xPathPath;
    String propertyValue;

    List<Object> xPaths = XPath.getXPaths(xmlObjectType);
    Node node = null;

    if (xPaths != null) {
      // sort the bloody thing
      Collections.sort(xPaths, new XPathComparator());

      // get first xPath and figure out root node
      xPath = (XPath) xPaths.get(0);
      node = B2BBDOMHelper.createEmptyNode(xPath.getRoot());

      Iterator<Object> iter = xPaths.iterator();

      while (iter.hasNext()) {
        xPath = (XPath) iter.next();
        field = xPath.getField();
        xPathPath = xPath.getDescription();

        if (field.indexOf(")") > -1) {
          String childCategory = XPath.getChildCategory(field);
          String fieldName = XPath.getFieldName(field);
          if (childCategory != null) {
            XMLObject child;
            child = (XMLObject) PropertyUtils.getProperty(this, fieldName);
            if (child != null && child.fieldsExists()) {
              Node childNode = child.createXML();

              Document doc = node.getOwnerDocument();
              Node toAdd = doc.importNode(childNode, true);

              Node insertHere = B2BBDOMHelper.createNode(doc, XPath.removeLast(xPathPath));
              insertHere.appendChild(toAdd);
            }
          }
        } else {
          if (xPath.getValue() != null) {
            propertyValue = xPath.getValue();
          } else {
            propertyValue = BeanUtils.getProperty(this, field);
          }
          if (propertyValue != null && !propertyValue.equals("")) {
            B2BBDOMHelper.setNodeValue(node.getOwnerDocument(), xPathPath, propertyValue);
          }
        }
      }
    }

    return node;
  }

  public boolean fieldsExists() {
    return false;
  }

	/**
	 * Checks if a value exists
	 *
	 * @param value
	 * @return True if value exists. False otherwise.
	 */
 	protected boolean exists(String value){
 	  if (value == null || value.trim().equals("")){
 	    return false;
 	  }
 	  return true;
 	}

  /**
   * 
   * @return The XML object type of the XML object
   */
  public XMLObjectType getXMLObjectType() {
    return xmlObjectType;
  }

  /**
   * This method is used to set the parameters in the XML object.
   * A Node representing the root element of the object is used to extract 
   * the necessary data.
   * 
   * @param xmlNode A Node representing the root element of an object snippet
   * from a B2B aseXML message.
   */
  public void setXMLString(Node xmlNode) throws UnableToParseXMLException {
    if (xmlNode != null) {
      try {
        this.xmlNode = xmlNode;

        // loop through each xpath mapping and set the values
        List<Object> xPaths = XPath.getXPaths(xmlObjectType);
        XPath xPath;

        if (xPaths != null) {
          Iterator<Object> iter = xPaths.iterator();

          while (iter.hasNext()) {
            xPath = (XPath) iter.next();
            if (xPath.getValue() == null) {
	            if (xPath.getField().indexOf("(") > -1) {
	              String className = XPath.getChildCategory(xPath.getField());
	              String fieldName = XPath.getFieldName(xPath.getField());
	              try {
	                Class<?> childClass = Class.forName(XMLObjectType.getXMLObjectType(className).getObjectClass());
	                XMLObject child = (XMLObject) childClass.newInstance();
	                child.setXMLString(B2BBDOMHelper.getSelectedNode(xmlNode, XPath.removeFirst(xPath.getDescription())));
	                BeanUtils.setProperty(this, fieldName, child);
	              } catch (Exception e) {
	                throw new UnableToParseXMLException("XPath properties file is invalid: " + e.getMessage());
	              }
	            } else {
	              BeanUtils.setProperty(this, xPath.getField(), B2BBDOMHelper.getSelectedNodeValue(xmlNode, XPath.removeFirst(xPath.getDescription())));
	            }
            }
          }
        }

      } catch (UnableToLookupResourceException e) {
        throw new UnableToParseXMLException("Property File not found. Exception returned: " + e.getMessage());
      } catch (TransformerException e) {
        throw new UnableToParseXMLException("XPath provided in properties file is invalid: " + e.getMessage());
      } catch (IllegalAccessException e) {
        e.printStackTrace();
      } catch (InvocationTargetException e) {
        e.printStackTrace();
      } catch (UnableToParseXMLException e) {
        e.printStackTrace();
      }

    }
  }

  /**
   * This method is used to set the parameters in the XML Object.
   * A String representing the XML snippet is parsed to extract the necessary data
   * 
   * @param xml The XML snippet of the XML object
   */
  public void setXMLString(String xml) throws UnableToParseXMLException {
    try {
      Document xmlDoc = B2BBDOMHelper.parseXML(xml);
      setXMLString(xmlDoc.getFirstChild());
    } catch (SAXException e) {
      throw new UnableToParseXMLException("Unable to parse XML exception. " + e.getMessage());
    } catch (IOException e) {
      throw new UnableToParseXMLException("Unable to parse XML exception. " + e.getMessage());
    }
  }
  
  /**
   * @param xmlObjectType
   */
  public XMLObject(final XMLObjectType xmlObjectType) {
    super();
    this.xmlObjectType = xmlObjectType;
  }
}