/*
 * Enumeration.java
 *
 * The Copyright of this work is vested in the Australian Energy Market
 * Operator (AEMO) and the software is issued in confidence for the purpose
 * only for which it is supplied. It must not be reproduced in whole or in
 * part or used for tendering or manufacturing purposes except under an
 * agreement or with the consent in writing of AEMO and then only on the
 * condition that this notice is included in any such reproduction.
 * No information as to the contents or subject matter of this software or
 * any part thereof arising directly or indirectly there from shall be
 * given orally or in writing or communicated in any manner whatsoever to
 * any third party being an individual firm or company or any employee
 * thereof without the prior consent in writing of AEMO.
 *
 * Copyright Australian Energy Market Operator Limited. 2017
 *
 * Modification History
 *
 * Date Changed   Changed By        Description
 * ------------   ----------        -----------
 * 11-Apr-2005    RayanS            Created
 * 10-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 * 27-Jun-2017    NG                7135 - PoC SMP Changes
 * 10-Jul-2017    W.Mills           7378 Transaction Type drop down is blank
 */

package com.capgemini.nemmco.b2bb.enums;

import java.io.Serializable;

import java.util.List;

/**
 * Defines the super class of an enumeration. Each enumeration requires at least a code
 */



public abstract class Enumeration implements Comparable<Object>, Serializable {

  /**
   * Returns true if the code value exists in the enumeration list
   *
   * @param code
   *          The value to find in this enumeration
   * @param list
   *          The list to check
   * @return Returns true if the value exists in the enumeration
   */
   public static boolean contains(String code, List<? extends Object> list) {
      Enumeration e;
      for (Object o : list) {
         if (o instanceof Enumeration) {
            e = (Enumeration)o;
            if (e.getCode().equals(code)) {
               return true;
            }
         }
      }
      return false;
   }

  /**
   * Returns the Enumeration object given by the code value in the enumeration list
   *
   * @param code
   *          The value to find in this enumeration
   * @param list
   *          The list to check
   * @return The Enumeration object associated with the key; null otherwise
   */
   public static Enumeration getEnumeration(String code, List<? extends Object> list) {
      Enumeration e;
      for (Object o : list) {
         if (o instanceof Enumeration) {
            e = (Enumeration)o;
            if (e.getCode().equals(code)) {
               return e;
            }
         }
      }
      return null;
   }

  protected String code;
  protected String description;
  protected String subDescription;
  protected int number;

  public int compareTo(Object o) {
    if (o != null && o instanceof Enumeration) {
      return (this.getCode()).compareTo(((Enumeration) o).getCode());
    } else {
      throw new ClassCastException("Object must be of type Enumeration");
    }
  }

  public boolean equals(Object o) {
    return this == o;
  }

  public String getCode() {
    return code;
  }

  public String getDescription() {
    return description;
  }

  public int getNumber() {
    return number;
  }

  public String toString() {
    if (description != null) {
      return code + "-" + description;
    } else {
      return code;
    }
  }

   public String getSubDescription() {
      return subDescription;
   }
}

