/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/enums/EventCode.java-arc   1.8   21 Apr 2005 11:38:26   RayanS  $
 * $Revision:   1.8  $
 * $Date:   21 Apr 2005 11:38:26  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 *
 * 
 * Date Changed   Changed By     Comments
 * ------------   ----------     -----------
 * 22-May-2009    Tushar Gupta   Defect 5439 - Added error codes for ServiceTime validation.
 * 21-Aug-2009    Eddie Graham   Defect 4793 - Added event code 2003.
 * 14-Aug-2012    Bob Jones      7984 Java upgrade from 6 to 7.
 * 31-May-2013    Eddie Graham   Defect 2169 - Added event code 1970.
 */

package com.capgemini.nemmco.b2bb.enums;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * Defines all event codes which can be used in the B2B Browser application
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.8  $ $Date:   21 Apr 2005 11:38:26  $
 */
public class EventCode extends Enumeration implements Comparable<Object>, java.io.Serializable {

  //count will be seen as zero despite its value unless it's before the
  // public constant declarations
  private static int count = 0;

  public static final EventCode CODE_0000 = new EventCode("0"   ,"Accept.", Severity.INFO);
  public static final EventCode CODE_0001 = new EventCode("1"   ,"Not well formed", Severity.ERROR);
  public static final EventCode CODE_0002 = new EventCode("2"   ,"Schema validation failure", Severity.ERROR);
  public static final EventCode CODE_0003 = new EventCode("3"   ,"Transaction not supported within Transaction Group", Severity.ERROR);
  public static final EventCode CODE_0004 = new EventCode("4"   ,"Transaction version not supported", Severity.ERROR);
  public static final EventCode CODE_0005 = new EventCode("5"   ,"Uncompression failure", Severity.ERROR);
  public static final EventCode CODE_0006 = new EventCode("6"   ,"Message too big", Severity.ERROR);
  public static final EventCode CODE_0007 = new EventCode("7"   ,"Header mismatch", Severity.ERROR);
  public static final EventCode CODE_0008 = new EventCode("8"   ,"Incorrect market", Severity.ERROR);
  public static final EventCode CODE_0009 = new EventCode("9"   ,"Unknown Transaction Group", Severity.ERROR);
  public static final EventCode CODE_0101 = new EventCode("101" ,"Database update error, typically the result of data integrity rule violation ", Severity.ERROR);
  public static final EventCode CODE_0102 = new EventCode("102" ,"Database system error, typically due to major database problem", Severity.ERROR);
  public static final EventCode CODE_0111 = new EventCode("111" ,"Participant Outbox full, message cannot be delivered", Severity.ERROR);
  public static final EventCode CODE_0200 = new EventCode("200" ,"Record(s) not found", Severity.ERROR);
  public static final EventCode CODE_0201 = new EventCode("201" ,"Data missing.", Severity.ERROR);
  public static final EventCode CODE_0202 = new EventCode("202" ,"Invalid data.", Severity.ERROR);
  public static final EventCode CODE_0203 = new EventCode("203" ,"Unknown report", Severity.ERROR);
  public static final EventCode CODE_0204 = new EventCode("204" ,"Missing or invalid report parameters", Severity.ERROR);
  public static final EventCode CODE_0206 = new EventCode("206" ,"Recipient did not initiate request", Severity.ERROR);
  public static final EventCode CODE_0207 = new EventCode("207" ,"Unrecognised EventCode", Severity.ERROR);
  public static final EventCode CODE_0999 = new EventCode("999" ,"Unexpected Error", Severity.ERROR);
  public static final EventCode CODE_1156 = new EventCode("1156","NMI and Checksum do not match", Severity.ERROR);
  public static final EventCode CODE_1910 = new EventCode("1910","ServiceOrderSubType does not match ServiceOrderType.", Severity.ERROR);
  public static final EventCode CODE_1912 = new EventCode("1912","Unable to perform the work within the required timeframe, alternate date provided in Explanation.", Severity.WARN);
  public static final EventCode CODE_1913 = new EventCode("1913","New request with previously used RequestID.", Severity.ERROR);
  public static final EventCode CODE_1914 = new EventCode("1914","New request with previously used RetServiceOrder.", Severity.ERROR);
  public static final EventCode CODE_1915 = new EventCode("1915","Service Provider does not support this ServiceOrderType or SubType", Severity.ERROR);
  public static final EventCode CODE_1916 = new EventCode("1916","Invalid AppointmentReference.", Severity.ERROR);
  public static final EventCode CODE_1917 = new EventCode("1917","Unable to cancel ServiceOrderRequest. Requested work has commenced or is completed.", Severity.ERROR);
  public static final EventCode CODE_1918 = new EventCode("1918","NMI already allocated for this address.", Severity.ERROR);
  public static final EventCode CODE_1919 = new EventCode("1919","Requested metering configuration is incorrect.", Severity.ERROR);
  public static final EventCode CODE_1921 = new EventCode("1921","ActualDateAndTime is after the date and time the ServiceOrderResponse was sent.", Severity.ERROR);
  public static final EventCode CODE_1922 = new EventCode("1922","Appointment Notification does not match a ServiceOrderRequest.", Severity.ERROR);
  public static final EventCode CODE_1923 = new EventCode("1923","Recipient is not responsible for the supplied NMI.", Severity.ERROR);
  public static final EventCode CODE_1924 = new EventCode("1924","NMIChecksum invalid.", Severity.ERROR);
  public static final EventCode CODE_1925 = new EventCode("1925","Format problem found in MDFF", Severity.ERROR);
  public static final EventCode CODE_1931 = new EventCode("1931","No data found.", Severity.ERROR);
  public static final EventCode CODE_1932 = new EventCode("1932","Participant is not authorised to receive the requested data.", Severity.ERROR);
  public static final EventCode CODE_1933 = new EventCode("1933","Participant is not entitled to requested data for date range requested.", Severity.ERROR);
  public static final EventCode CODE_1934_INFO = new EventCode("1934","Participant is not entitled to requested data for part of the date range requested", Severity.INFO );
  public static final EventCode CODE_1934_ERROR = new EventCode("1934","Participant is not entitled to requested data for part of the date range requested", Severity.ERROR);
  public static final EventCode CODE_1936 = new EventCode("1936","Requested data has previously been sent in response to a previous ProvideMeterDataRequest.", Severity.ERROR);
  public static final EventCode CODE_1937 = new EventCode("1937","Unable To Cancel, Original Request Not Received", Severity.ERROR);
  public static final EventCode CODE_1938 = new EventCode("1938","Previous Cancellation Already Processed", Severity.ERROR);
  public static final EventCode CODE_1939 = new EventCode("1939","Not Current FRMP", Severity.ERROR);
  public static final EventCode CODE_1940 = new EventCode("1940","Unable to perform the work after hours, alternative time provided in Explanation.", Severity.WARN);
  public static final EventCode CODE_1941 = new EventCode("1941","No Meter At Site", Severity.ERROR);
  public static final EventCode CODE_1942 = new EventCode("1942","Address identical to address held by DNSP", Severity.INFO);
  public static final EventCode CODE_1943 = new EventCode("1943","Address not accepted. MSATS correct", Severity.ERROR);
  public static final EventCode CODE_1944 = new EventCode("1944","Rejection - site already de-energised.", Severity.ERROR);
  public static final EventCode CODE_1945 = new EventCode("1945","Retailer Is Not Permitted To Raise This Service Order Type", Severity.ERROR);
  public static final EventCode CODE_1946 = new EventCode("1946","Requested data is no longer on-line", Severity.ERROR);
  public static final EventCode CODE_1947 = new EventCode("1947","Address not accepted. MSATS to be updated.", Severity.ERROR);
  public static final EventCode CODE_1948 = new EventCode("1948","NSRD not past yet (ie allowed timeframe to provide reading has not expired yet).", Severity.ERROR);
  public static final EventCode CODE_1949 = new EventCode("1949","Medical Certificate not received in time", Severity.ERROR);
  public static final EventCode CODE_1950 = new EventCode("1950","Mandatory field not populated. Missing field(s) listed in Explanation.", Severity.ERROR);
  public static final EventCode CODE_1951 = new EventCode("1951","Product Code does not match requested work", Severity.WARN);
  public static final EventCode CODE_1952 = new EventCode("1952","Invalid Multiple Service Order Combination", Severity.ERROR);
  public static final EventCode CODE_1953 = new EventCode("1953","Documentation required, see Explanation for details", Severity.WARN);
  public static final EventCode CODE_1954 = new EventCode("1954","ScheduledDate greater than 100 calendar days in the future.", Severity.ERROR);
  public static final EventCode CODE_1955 = new EventCode("1955","RetServiceOrder value of the rejected Request is not in SpecialInstructions", Severity.ERROR);
  public static final EventCode CODE_1956 = new EventCode("1956","Request submitted by another Retailer", Severity.ERROR);
  public static final EventCode CODE_1957 = new EventCode("1957","The Request falls outside the Service Provider's regulatory and contractual obligations.", Severity.ERROR);
  public static final EventCode CODE_1958 = new EventCode("1958","Insufficient information provided to action Request.", Severity.ERROR);
  public static final EventCode CODE_1959 = new EventCode("1959","Query has been investigated and no change made to the MDFF Data.", Severity.ERROR);
  public static final EventCode CODE_1960 = new EventCode("1960","StartReadDate is before the MDP SLR on-line storage requirement.", Severity.INFO);
  public static final EventCode CODE_1961 = new EventCode("1961","NMI abolished", Severity.ERROR);
  public static final EventCode CODE_1962 = new EventCode("1962","No active meters", Severity.ERROR);
  public static final EventCode CODE_1963_INFO = new EventCode("1963","Recipient is not the MDP for the whole period.", Severity.INFO);
  public static final EventCode CODE_1963_ERROR = new EventCode("1963","Recipient is not the MDP for the whole period.", Severity.ERROR);
  public static final EventCode CODE_1964 = new EventCode("1964","Unable to Cancel, Original Request Rejected.", Severity.ERROR);
  public static final EventCode CODE_1965 = new EventCode("1965","Request matches an existing Request.  The TransactionID of the related Request is provided in Explanation.", Severity.ERROR);
  public static final EventCode CODE_1966 = new EventCode("1966","No further data available.", Severity.INFO);
  public static final EventCode CODE_1967 = new EventCode("1967","'Replace' ServiceOrderRequest sent without the prior agreement of the Receiver.", Severity.ERROR);
  public static final EventCode CODE_1968 = new EventCode("1968","Required timeframe for updating MSATS has not passed.", Severity.ERROR);
  public static final EventCode CODE_1969 = new EventCode("1969","Invalid Request.", Severity.ERROR);
  
  // 31-May-2013 Eddie Graham Defect 2169 - Added event code 1970.  
  public static final EventCode CODE_1970 = new EventCode("1970","Data not fit for purpose. Details provided in Explanation.", Severity.ERROR);
  
  public static final EventCode CODE_1999_INFO = new EventCode("1999","Miscellaneous error, not covered by the other Event Codes.", Severity.INFO);
  public static final EventCode CODE_1999_ERROR = new EventCode("1999","Miscellaneous error, not covered by the other Event Codes.", Severity.ERROR);
 
  // TG 27 May 2009 - Added 2000,2001,2002 error code for defect 5439 
  public static final EventCode CODE_2000_ERROR = new EventCode("2000","Unable to perform the work within the required timeframe, alternative date provided in Explanation.", Severity.ERROR);
  public static final EventCode CODE_2001_ERROR = new EventCode("2001","Unable to perform the work after hours, alternative time provided in Explanation.", Severity.ERROR);
  public static final EventCode CODE_2002_ERROR = new EventCode("2002","Unable to perform the work due to unacceptable notice period provided, alternative time provided in Explanation.", Severity.ERROR);
  
  // 21-Aug-2009 Eddie Graham Defect 4793 - Added event code 2003.
  public static final EventCode CODE_2003_ERROR = new EventCode("2003","The format of the CSV Payload is not as defined in the Procedure. Please correct and re-send.", Severity.ERROR);

  public static final EventCode CODE_2004_ERROR = new EventCode("2004","", Severity.ERROR);
  private static final EventCode[] EVENT_CODE_ARRAY = { 
      CODE_0000,
      CODE_0001,
      CODE_0002,
      CODE_0003,
      CODE_0004,
      CODE_0005,
      CODE_0006,
      CODE_0007,
      CODE_0008,
      CODE_0009,
      CODE_0101,
      CODE_0102,
      CODE_0111,
      CODE_0200,
      CODE_0201,
      CODE_0202,
      CODE_0203,
      CODE_0204,
      CODE_0206,
      CODE_0207,
      CODE_0999,
      CODE_1156,
      CODE_1910,
      CODE_1912,
      CODE_1913,
      CODE_1914,
      CODE_1915,
      CODE_1916,
      CODE_1917,
      CODE_1918,
      CODE_1919,
      CODE_1921,
      CODE_1922,
      CODE_1923,
      CODE_1924,
      CODE_1925,
      CODE_1931,
      CODE_1932,
      CODE_1933,
      CODE_1934_INFO,
      CODE_1934_ERROR,
      CODE_1936,
      CODE_1937,
      CODE_1938,
      CODE_1939,
      CODE_1940,
      CODE_1941,
      CODE_1942,
      CODE_1943,
      CODE_1944,
      CODE_1945,
      CODE_1946,
      CODE_1947,
      CODE_1948,
      CODE_1949,
      CODE_1950,
      CODE_1951,
      CODE_1952,
      CODE_1953,
      CODE_1954,
      CODE_1955,
      CODE_1956,
      CODE_1957,
      CODE_1958,
      CODE_1959,
      CODE_1960,
      CODE_1961,
      CODE_1962,
      CODE_1963_INFO,
      CODE_1963_ERROR,
      CODE_1964,
      CODE_1965,
      CODE_1966,
      CODE_1967,
      CODE_1968,
      CODE_1969,
      CODE_1970,
      CODE_1999_INFO,
      CODE_1999_ERROR,
      CODE_2000_ERROR,
      CODE_2001_ERROR,
      CODE_2002_ERROR,
      CODE_2003_ERROR,
      CODE_2004_ERROR
      };
  
  public static final List<? extends Object> EVENT_CODES = Collections.unmodifiableList(Arrays.asList(EVENT_CODE_ARRAY));

  private final Severity severity;

  /**
   * Return the EventCode object given by code (key)
   * 
   * @param code
   *          The key to find in this enumeration
   * @return The object associated with the key. Null otherwise
   */
  public static EventCode getEventCode(String code) {
    for (int i = 0; i < EVENT_CODES.size(); i++) {
      if ((((Enumeration) EVENT_CODES.get(i)).getCode()).equals(code)) return (EventCode) EVENT_CODES.get(i);
    }
    return null;
  }

  private EventCode(String code, String description, Severity severity) {
    this.code = code;
    this.description = description;
    this.severity = severity;
    this.number = ++count;
  }

  public Severity getSeverity() {
    return severity;
  }
}

