/*
 * TransactionType.java
 *
 * The Copyright of this work is vested in the Australian Energy Market
 * Operator (AEMO) and the software is issued in confidence for the purpose
 * only for which it is supplied. It must not be reproduced in whole or in
 * part or used for tendering or manufacturing purposes except under an
 * agreement or with the consent in writing of AEMO and then only on the
 * condition that this notice is included in any such reproduction.
 * No information as to the contents or subject matter of this software or
 * any part thereof arising directly or indirectly there from shall be
 * given orally or in writing or communicated in any manner whatsoever to
 * any third party being an individual firm or company or any employee
 * thereof without the prior consent in writing of AEMO.
 *
 * Copyright Australian Energy Market Operator Limited. 2017
 *
 * Modification History
 *
 * Date Changed   Changed By        Description
 * ------------   ----------        -----------
 * 26-Apr-2005    RayanS            Created
 * 03-Sep-2008    Eddie Graham      Defect 4864 - Modify the ordering of the transactions in array TRANSACTION_ARRAY so that SOR_CANCEL is first.
 * 23-Jan-2009    Eddie Graham      Defect 4793. Add new constant ONE_WAY_NOT.
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 * 15-Jun-2017    SP                SMP  workpkg7b- adding the RemoteAccess Request screen
 * 27-Jun-2017    NG                7135 - PoC SMP Changes
 * 27-Jun-2017    Harpreet Kaur     7135 Package 7B:Add new constant SITE_ACCESS_REQ
 * 30-Jun-2017    Harpreet Kaur     7135 Package 7B:Add new constant PLANNED_INTERRUPT_NOT
 * 10-Jul-2017    W.Mills           7378 Transaction Type drop down is blank
 * 10-Jul-2017    Harpreet Kaur    7135 Package 7B:Add new constant METER_FAULT_AND_ISSUE_NOT
 * 10-Jul-2017    SP              SMP  workpkg7b- adding the Notice Of metering works
 * 28-Jul-2017    Harpreet Kaur    7135 Package 7B: Changes doen for SOR Meter Service Works screens as part of POC SMP

 */

package com.capgemini.nemmco.b2bb.enums;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * A typesafe enumeration of the TransactionTypes available in the B2B Browser Application.
 *
 * Immutable, Thread safe, and resolves properly on deserialization.
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.27  $ $Date:   26 Apr 2005 14:22:46  $
 */
public final class TransactionType extends XMLObjectType implements Comparable<Object>, java.io.Serializable {

  private static int count = 0;

  public static final TransactionType SOR_ALLOCATE_NMI         = new TransactionType("SORAllocateNMI"                    , "Supply Service Works"           , "Allocate NMI"                         , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORAllocateNMI"                      , "com.capgemini.nemmco.b2bb.validators.SORAllocateNMIBasicValidator"                       , "com.capgemini.nemmco.b2bb.validators.SORAllocateNMIExtValidator"                  , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Supply Service Works' and @workSubType='Allocate NMI']");
  public static final TransactionType SOR_NEW_CONNECTION       = new TransactionType("SORNewConnection"                  , "Supply Service Works"           , "Establish Permanent Supply"           , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORNewConnection"                    , "com.capgemini.nemmco.b2bb.validators.SORNewConnectionBasicValidator"                     , "com.capgemini.nemmco.b2bb.validators.SORNewConnectionExtValidator"                , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Supply Service Works' and @workSubType='Establish Permanent Supply']");
  public static final TransactionType SOR_NEW_CONNECTION_T     = new TransactionType("SORNewConnectionTemp"              , "Supply Service Works"           , "Establish Temporary Supply"           , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORNewConnectionTemp"                , "com.capgemini.nemmco.b2bb.validators.SORNewConnectionBasicValidator"                     , "com.capgemini.nemmco.b2bb.validators.SORNewConnectionExtValidator"                , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Supply Service Works' and @workSubType='Establish Temporary Supply']");
  public static final TransactionType SOR_NEW_CONNECTION_TP    = new TransactionType("SORNewConnectionTempPmt"           , "Supply Service Works"           , "Establish Temporary In Permanent"     , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORNewConnectionTempPmt"             , "com.capgemini.nemmco.b2bb.validators.SORNewConnectionBasicValidator"                     , "com.capgemini.nemmco.b2bb.validators.SORNewConnectionExtValidator"                , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Supply Service Works' and @workSubType='Establish Temporary In Permanent']");
  public static final TransactionType SOR_SUPPLY_ALTERATION    = new TransactionType("SORSupplyAlteration"               , "Supply Service Works"           , "Supply Alteration"                    , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORSupplyAlteration"                 , "com.capgemini.nemmco.b2bb.validators.SORNewConnectionBasicValidator"                     , "com.capgemini.nemmco.b2bb.validators.SORNewConnectionExtValidator"                , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Supply Service Works' and @workSubType='Supply Alteration']");
  public static final TransactionType SOR_SUPPLY_ABOLISHMENT   = new TransactionType("SORSupplyAbolishment"              , "Supply Service Works"           , "Supply Abolishment"                   , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORSupplyAbolishment"                , "com.capgemini.nemmco.b2bb.validators.SORSupplyAbolishmentBasicValidator"                 , "com.capgemini.nemmco.b2bb.validators.SORSupplyAbolishmentExtValidator"            , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Supply Service Works' and @workSubType='Supply Abolishment']");
  public static final TransactionType SOR_TEMP_ISOLATION       = new TransactionType("SORTempIsolation"                  , "Supply Service Works"           , "Temporary Isolation"                  , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORTemporaryIsolation"               , "com.capgemini.nemmco.b2bb.validators.SORTemporaryIsolationBasicValidator"                , "com.capgemini.nemmco.b2bb.validators.SORTemporaryIsolationExtValidator"           , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Supply Service Works' and @workSubType='Temporary Isolation']");
  public static final TransactionType SOR_TEMP_ISOLATION_GRP   = new TransactionType("SORTempIsolationGrp"               , "Supply Service Works"           , "Temporary Isolation-Group Supply"     , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORTemporaryIsolationGroup"          , "com.capgemini.nemmco.b2bb.validators.SORTemporaryIsolationBasicValidator"                , "com.capgemini.nemmco.b2bb.validators.SORTemporaryIsolationExtValidator"           , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Supply Service Works' and @workSubType='Temporary Isolation-Group Supply']");
  public static final TransactionType SOR_TARIFF_CHANGE        = new TransactionType("SORTariffChange"                   , "Supply Service Works"           , "Tariff Change"                        , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORTariffChange"                     , "com.capgemini.nemmco.b2bb.validators.SORTariffChangeBasicValidator"                      , "com.capgemini.nemmco.b2bb.validators.SORTariffChangeExtValidator"                 , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Supply Service Works']");  //catch all others -  the only one left is @workSubType='Tariff Change'

  //Meter Service Works
  public static final TransactionType SOR_MSW_INSTALL_METER     = new TransactionType("SORMetServWrksInstallMeter"       , "Metering Service Works"         , "Install Meter"                        , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORMetServWrksInstallMeter"          , "com.capgemini.nemmco.b2bb.validators.SORMeterInstallBasicValidator"                      , "com.capgemini.nemmco.b2bb.validators.SORMeterInstallExtValidator"                 , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Metering Service Works' and @workSubType='Install Meter']");
  public static final TransactionType SOR_MSW_MOVE_METER        = new TransactionType("SORMetServWrksMoveMeter"          , "Metering Service Works"         , "Move Meter"                           , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORMetServWrksMoveMeter"             , "com.capgemini.nemmco.b2bb.validators.SORMeterMoveBasicValidator"                         , "com.capgemini.nemmco.b2bb.validators.SORMeterMoveExtValidator"                    , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Metering Service Works' and @workSubType='Move Meter']");
  public static final TransactionType SOR_MSW_REMOVE_METER      = new TransactionType("SORMetServWrksRemoveMeter"        , "Metering Service Works"         , "Remove Meter"                         , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORMetServWrksRemoveMeter"           , "com.capgemini.nemmco.b2bb.validators.SORMeterRemoveBasicValidator"                       , "com.capgemini.nemmco.b2bb.validators.SORMeterRemoveExtValidator"                  , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Metering Service Works' and @workSubType='Remove Meter']");
  public static final TransactionType SOR_MSW_EXCHANGE_METER    = new TransactionType("SORMetServWrksExchangeMeter"      , "Metering Service Works"         , "Exchange Meter"                       , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORMetServWrksExchangeMeter"         , "com.capgemini.nemmco.b2bb.validators.SORMeterExchangeBasicValidator"                     , "com.capgemini.nemmco.b2bb.validators.SORMeterExchangeExtValidator"                , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Metering Service Works' and @workSubType='Exchange Meter']");
  public static final TransactionType SOR_MSW_INSTALL_CLOAD     = new TransactionType("SORMetServWrksInstallCLoad"       , "Metering Service Works"         , "Install Controlled Load"              , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORMetServWrksInstallCLoad"          , "com.capgemini.nemmco.b2bb.validators.SORMeterInstallControlledLoadBasicValidator"        , "com.capgemini.nemmco.b2bb.validators.SORMeterInstallControlledLoadExtValidator"   , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Metering Service Works' and @workSubType='Install Controlled Load']");
  public static final TransactionType SOR_MSW_METER_RECONFIG    = new TransactionType("SORMetServWrksMeterReconfig"      , "Metering Service Works"         , "Meter Reconfiguration"                , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORMetServWrksMeterReconfig"         , "com.capgemini.nemmco.b2bb.validators.SORMeterReconfigurationBasicValidator"              , "com.capgemini.nemmco.b2bb.validators.SORMeterReconfigurationExtValidator"         , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Metering Service Works' and @workSubType='Meter Reconfiguration']");
  public static final TransactionType SOR_MSW_CHG_TSWITCH       = new TransactionType("SORMetServWrksChgTSwitch"         , "Metering Service Works"         , "Change Timeswitch Settings"           , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORMetServWrksChgTSwitch"            , "com.capgemini.nemmco.b2bb.validators.SORMeterReconfigurationBasicValidator"              , "com.capgemini.nemmco.b2bb.validators.SORMeterReconfigurationExtValidator"         , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Metering Service Works' and @workSubType='Change Timeswitch Settings']");
  public static final TransactionType SOR_MSW_METER_INV_INS     = new TransactionType("SORMetServWrksInvInspect"         , "Metering Service Works"         , "Meter Investigation-Inspect"          , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORMetServWrksInvInspect"            , "com.capgemini.nemmco.b2bb.validators.SORMeterInvestigationBasicValidator"                , "com.capgemini.nemmco.b2bb.validators.SORMeterInvestigationExtValidator"           , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Metering Service Works' and @workSubType='Meter Investigation-Inspect']");
  public static final TransactionType SOR_MSW_METER_INV_TEST    = new TransactionType("SORMetServWrksInvTest"            , "Metering Service Works"         , "Meter Investigation-Test"             , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORMetServWrksInvTest"               , "com.capgemini.nemmco.b2bb.validators.SORMeterInvestigationBasicValidator"                , "com.capgemini.nemmco.b2bb.validators.SORMeterInvestigationExtValidator"           , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Metering Service Works' and @workSubType='Meter Investigation-Test']");
  public static final TransactionType SOR_MSW_RESEAL            = new TransactionType("SORMetServWrksReseal"             , "Metering Service Works"         , "Reseal Device"                        , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORMetServWrksReseal"                , "com.capgemini.nemmco.b2bb.validators.SORMeterInvestigationBasicValidator"                , "com.capgemini.nemmco.b2bb.validators.SORMeterInvestigationExtValidator"           , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Metering Service Works']"); //catch all others -  the only one left is @workSubType='Reseal Device'

  public static final TransactionType SOR_REENERGISATION              = new TransactionType("SORReenergisation"             , "Re-energisation"             , "Remote"                               , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORReEnRemote"                       , "com.capgemini.nemmco.b2bb.validators.SORReenergisationBasicValidator"                    , "com.capgemini.nemmco.b2bb.validators.SORReenergisationExtValidator"               , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Re-energisation' and @workSubType='Remote']");
  public static final TransactionType SOR_REENERGISATION_PHY_VISIT    = new TransactionType("SORReenergisationPhyVisit"     , "Re-energisation"             , "Physical Visit"                       , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORReEnPhysicalVisit"                , "com.capgemini.nemmco.b2bb.validators.SORReenergisationBasicValidator"                    , "com.capgemini.nemmco.b2bb.validators.SORReenergisationExtValidator"               , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Re-energisation' and @workSubType='Physical Visit']");
  public static final TransactionType SOR_REENERGISATION_RETRO_MOVEIN = new TransactionType("SORReenergisationRetroMoveIn"  , "Re-energisation"             , "Retrospective Move-in"                , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORReEnRetroMoveIn"                  , "com.capgemini.nemmco.b2bb.validators.SORReenergisationBasicValidator"                    , "com.capgemini.nemmco.b2bb.validators.SORReenergisationExtValidator"               , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Re-energisation' and @workSubType='Retrospective Move-in']");
  public static final TransactionType SOR_REENERGISATION_REC_DISCR    = new TransactionType("SORReenergisationRecDiscr"     , "Re-energisation"             , "Recipient Discretion"                 , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORReEnRecipientDiscretion"          , "com.capgemini.nemmco.b2bb.validators.SORReenergisationBasicValidator"                    , "com.capgemini.nemmco.b2bb.validators.SORReenergisationExtValidator"               , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Re-energisation' and @workSubType='Recipient Discretion']");
  public static final TransactionType SOR_REENERGISATION_AFT_DISCONN  = new TransactionType("SORReenergisationAftDisconn"   , "Re-energisation"             , "After Disconnection For Non-Payment"  , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORReEnAfterDisconnection"           , "com.capgemini.nemmco.b2bb.validators.SORReenergisationBasicValidator"                    , "com.capgemini.nemmco.b2bb.validators.SORReenergisationExtValidator"               , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Re-energisation' and @workSubType='After Disconnection For Non-Payment']");
  public static final TransactionType SOR_REENERGISATION_NEW_READ  = new TransactionType("SORReenergisationNewRead"         , "Re-energisation"             , "New Reading Required"                 , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORReEnNewRead"                      , "com.capgemini.nemmco.b2bb.validators.SORReenergisationBasicValidator"                    , "com.capgemini.nemmco.b2bb.validators.SORReenergisationExtValidator"               , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Re-energisation' and @workSubType='New Reading Required']");
  public static final TransactionType SOR_REENERGISATION_MOVEIN       = new TransactionType("SORReenergisationMoveIn"       , "Re-energisation"             , "Move-in"                              , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORReEnMoveIn"                       , "com.capgemini.nemmco.b2bb.validators.SORReenergisationBasicValidator"                    , "com.capgemini.nemmco.b2bb.validators.SORReenergisationExtValidator"               , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Re-energisation']"); //catch all others -  the only one left is @workSubType='Move-in'

  public static final TransactionType SOR_DEENERGISATION                 = new TransactionType("SORDeenergisation"                , "De-energisation"       , "Disconnect at Pillar-Box Pit Or Pole-Top" , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORDeEnDisconnectAtP"              , "com.capgemini.nemmco.b2bb.validators.SORDeenergisationBasicValidator"                    , "com.capgemini.nemmco.b2bb.validators.SORDeenergisationExtValidator"               , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='De-energisation' and @workSubType='Disconnect at Pillar-Box Pit Or Pole-Top']");
  public static final TransactionType SOR_DEENERGISATION_REMOTE          = new TransactionType("SORDeenergisationRemote"          , "De-energisation"       , "Remote"                                 , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORDeEnRemote"                     , "com.capgemini.nemmco.b2bb.validators.SORDeenergisationBasicValidator"                    , "com.capgemini.nemmco.b2bb.validators.SORDeenergisationExtValidator"               , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='De-energisation' and @workSubType='Remote']");
  public static final TransactionType SOR_DEENERGISATION_LOC_MET_DISCONN = new TransactionType("SORDeenergisationLocMetDisconn"   , "De-energisation"       , "Local Meter Disconnection"              , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORDeEnLocalMeterDisconnection"    , "com.capgemini.nemmco.b2bb.validators.SORDeenergisationBasicValidator"                    , "com.capgemini.nemmco.b2bb.validators.SORDeenergisationExtValidator"               , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='De-energisation' and @workSubType='Local Meter Disconnection']");
  public static final TransactionType SOR_DEENERGISATION_REC_DISCR       = new TransactionType("SORDeenergisationRecDiscr"        , "De-energisation"       , "Recipient Discretion"                   , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORDeEnRecipientDiscretion"        , "com.capgemini.nemmco.b2bb.validators.SORDeenergisationBasicValidator"                    , "com.capgemini.nemmco.b2bb.validators.SORDeenergisationExtValidator"               , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='De-energisation' and @workSubType='Recipient Discretion']");
  public static final TransactionType SOR_DEENERGISATION_REMOVE_FUSE      = new TransactionType("SORDeenergisationRemoveFuse"     , "De-energisation"       , "Remove Fuse"                            , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORDeEnRemoveFuse"                , "com.capgemini.nemmco.b2bb.validators.SORDeenergisationBasicValidator"                    , "com.capgemini.nemmco.b2bb.validators.SORDeenergisationExtValidator"               , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='De-energisation']"); //catch all others - the only one left is @workSubType='Remove Fuse'

  public static final TransactionType SOR_SPECIAL_READ_FINAL      = new TransactionType("SORSpecialReadFinal"                    , "Special Read"           , "Final Read"                         , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORSpecialReadFinal"                   , "com.capgemini.nemmco.b2bb.validators.SORSpecialReadBasicValidator"                       , "com.capgemini.nemmco.b2bb.validators.SORSpecialReadExtValidator"                  , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Special Read' and @workSubType='Final Read']");
  public static final TransactionType SOR_SPECIAL_READ_CHECK            = new TransactionType("SORSpecialReadCheck"                  , "Special Read"           , "Check Read"                         , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORSpecialReadCheck"                   , "com.capgemini.nemmco.b2bb.validators.SORSpecialReadBasicValidator"                       , "com.capgemini.nemmco.b2bb.validators.SORSpecialReadExtValidator"                  , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Special Read' and @workSubType='Check Read']");
  public static final TransactionType SOR_SPECIAL_READ            = new TransactionType("SORSpecialRead"                         , "Special Read"           , ""                                   , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORSpecialRead"                        , "com.capgemini.nemmco.b2bb.validators.SORSpecialReadBasicValidator"                       , "com.capgemini.nemmco.b2bb.validators.SORSpecialReadExtValidator"                  , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Special Read']"); //Catch all - When no subtype is provided

  public static final TransactionType SOR_MISCELLANEOUS        = new TransactionType("SORMiscellaneous"           , "Miscellaneous"                   , ""            , "ServiceOrderRequest"         , "com.capgemini.nemmco.b2bb.SORMiscellaneous"                    , "com.capgemini.nemmco.b2bb.validators.SORMiscellaneousBasicValidator"                     , "com.capgemini.nemmco.b2bb.validators.SORMiscellaneousExtValidator"       , "ServiceOrderRequest/ServiceOrder/ServiceOrderType/WorkType[.='Miscellaneous']");
  public static final TransactionType SOR_CANCEL               = new TransactionType("SORCancel"                  , "Service Order Request Cancel"    , ""             , "ServiceOrderRequestCancel"   , "com.capgemini.nemmco.b2bb.SORCancel"                           , "com.capgemini.nemmco.b2bb.validators.SORCancelBasicValidator"                            , "com.capgemini.nemmco.b2bb.validators.SORCancelExtValidator"              , "ServiceOrderRequest/@actionType[.='Cancel']");
  public static final TransactionType SOR_RESPONSE             = new TransactionType("SORResponse"                , "Service Order Response"          , ""            , "SORResponse"                 , "com.capgemini.nemmco.b2bb.SORResponse"                         , "com.capgemini.nemmco.b2bb.validators.SORResponseBasicValidator"                          , "com.capgemini.nemmco.b2bb.validators.SORResponseExtValidator"            , "ServiceOrderResponse/@responseType[.='Closure']");


  public static final TransactionType CUST_DETAIL_NOT          = new TransactionType("CustDetailNotification"     , "Customer Details Notification"   , ""            , "CustDetailNotification"      , "com.capgemini.nemmco.b2bb.CustDetailNotification"              , "com.capgemini.nemmco.b2bb.validators.CustDetailNotificationBasicValidator"               , "com.capgemini.nemmco.b2bb.validators.CustDetailNotificationExtValidator" , "CustomerDetailsNotification");
  public static final TransactionType SITE_ACCESS_REQ          = new TransactionType("SiteAccessRequest"          , "Site Access Request"             , ""            , "SiteAccessRequest"           , "com.capgemini.nemmco.b2bb.SiteAccessRequest"                   , "com.capgemini.nemmco.b2bb.validators.SiteAccessRequestBasicValidator"                    , "com.capgemini.nemmco.b2bb.validators.SiteAccessRequestExtValidator"      , "SiteAccessRequest");
  public static final TransactionType SITE_ACCESS_NOT          = new TransactionType("SiteAccessNotification"     , "Site Access Notification"        , ""             , "SiteAccessNotification"      , "com.capgemini.nemmco.b2bb.SiteAccessNotification"              , "com.capgemini.nemmco.b2bb.validators.SiteAccessNotificationBasicValidator"               , "com.capgemini.nemmco.b2bb.validators.SiteAccessNotificationExtValidator" , "AmendMeterRouteDetails/AmendSiteAccessDetails");
  public static final TransactionType SITE_ADDRESS_NOT         = new TransactionType("SiteAddressNotification"    , "Site Address Notification"       , ""            , "SiteAddressNotification"     , "com.capgemini.nemmco.b2bb.SiteAddressNotification"             , "com.capgemini.nemmco.b2bb.validators.SiteAddressNotificationBasicValidator"              , "com.capgemini.nemmco.b2bb.validators.SiteAddressNotificationExtValidator", "AmendMeterRouteDetails/AmendSiteAddressDetails");
  public static final TransactionType PROVIDE_METER_DATA_REQ   = new TransactionType("ProvideMeterDataRequest"    , "Provide Meter Data Request"      , ""            , "ProvideMeterDataRequest"     , "com.capgemini.nemmco.b2bb.ProvideMeterDataRequest"             , "com.capgemini.nemmco.b2bb.validators.ProvideMeterDataRequestBasicValidator"              , "com.capgemini.nemmco.b2bb.validators.ProvideMeterDataRequestExtValidator", "MeterDataMissingNotification/MissingMeterData");
  public static final TransactionType METER_DATA_VERIFY_REQ    = new TransactionType("MeterDataVerifyRequest"     , "Verify Meter Data Request"       , ""            , "MeterDataVerifyRequest"      , "com.capgemini.nemmco.b2bb.MeterDataVerifyRequest"              , "com.capgemini.nemmco.b2bb.validators.MeterDataVerifyRequestBasicValidator"               , "com.capgemini.nemmco.b2bb.validators.MeterDataVerifyRequestExtValidator" , "MeterDataVerifyRequest/VerifyRequestData");
  public static final TransactionType CUST_DETAIL_REQUEST      = new TransactionType("CustDetailsRequest"         , "Customer Details Request"        , ""            , "CustDetailsRequest"          , "com.capgemini.nemmco.b2bb.CustDetailsRequest"                  , "com.capgemini.nemmco.b2bb.validators.CustDetailsRequestBasicValidator"                   , ""                                                                        , "CustomerDetailsRequest");
  public static final TransactionType METER_DATA_NOT           = new TransactionType("MeterDataNotification"      , "Meter Data Notification"         , ""            , "MeterDataNotification"       , "com.capgemini.nemmco.b2bb.MeterDataNotification"               , "com.capgemini.nemmco.b2bb.validators.MeterDataNotificationBasicValidator"                , ""                                                                        , "MeterDataNotification");
  public static final TransactionType SO_APPOINTMENT_NOT       = new TransactionType("SOAppointmentNotification"  , "Service Order Appointment Notification" , ""     , "SOAppointmentNotification"   , "com.capgemini.nemmco.b2bb.ServiceOrderAppointmentNotification" , "com.capgemini.nemmco.b2bb.validators.ServiceOrderAppointmentNotificationBasicValidator"  , ""                                                                        , "ServiceOrderResponse/@responseType[.='Appointment']");
  // 23-Jan-2009 Eddie Graham Defect 4793. Add new constant ONE_WAY_NOT.
  public static final TransactionType ONE_WAY_NOT              = new TransactionType("OneWayNotification"         , "One Way Notification"                        , "OneWayNotification"           , "com.capgemini.nemmco.b2bb.OneWayNotification"                 , "com.capgemini.nemmco.b2bb.validators.OneWayNotificationBasicValidator"                   , "com.capgemini.nemmco.b2bb.validators.OneWayNotificationExtValidator"     , "OneWayNotification");
  public static final TransactionType REMOTE_SERVICE_REQ        = new TransactionType("RemoteServiceRequest","Remote Service Request", "RemoteServiceRequest", "com.capgemini.nemmco.b2bb.RemoteServiceRequest", "com.capgemini.nemmco.b2bb.validators.RemoteServiceRequestBasicValidator"                    , "com.capgemini.nemmco.b2bb.validators.RemoteServiceRequestExtValidator", "RemoteServiceRequest");
  public static final TransactionType REMOTE_SERVICE_RES        = new TransactionType("RemoteServiceResponse","Remote Service Response", "RemoteServiceResponse", "com.capgemini.nemmco.b2bb.RemoteServiceResponse", "com.capgemini.nemmco.b2bb.validators.RemoteServiceResponseBasicValidator", "com.capgemini.nemmco.b2bb.validators.RemoteServiceResponseExtValidator", "RemoteServiceResponse");
  public static final TransactionType PLANNED_INTERRUPT_NOT    = new TransactionType("PlannedInterruptionNotification"      , "Planned Interruption Notification"  , "PlannedInterruptionNotification" , "com.capgemini.nemmco.b2bb.PlannedInterruptionNotification"       , "com.capgemini.nemmco.b2bb.validators.PlannedInterruptionNotificationBasicValidator"         , "com.capgemini.nemmco.b2bb.validators.PlannedInterruptionNotificationExtValidator"      , "PlannedInterruptionNotification");
  public static final TransactionType METER_FAULT_AND_ISSUE_NOT    = new TransactionType("MeterFaultAndIssueNotification"          , "Meter Fault And Issue Notification"     , "MeterFaultAndIssueNotification"     , "com.capgemini.nemmco.b2bb.MeterFaultAndIssueNotification"           , "com.capgemini.nemmco.b2bb.validators.MeterFaultAndIssueNotificationBasicValidator"             , "com.capgemini.nemmco.b2bb.validators.MeterFaultAndIssueNotificationExtValidator"          , "MeterFaultAndIssueNotification");
  public static final TransactionType NOTICE_OF_METERING_WORKS       = new TransactionType("NoticeOfMeteringWorks","Notice Of Metering Works", "NoticeOfMeteringWorks", "com.capgemini.nemmco.b2bb.NoticeOfMeteringWorks", "com.capgemini.nemmco.b2bb.validators.NoticeOfMeteringWorksBasicValidator", "com.capgemini.nemmco.b2bb.validators.NoticeOfMeteringWorksExtValidator", "NoticeOfMeteringWorks");
  public static final TransactionType NOTIFIED_PARTY            = new TransactionType("NotifiedParty"      ,                  "Notified Party"  ,                       "NotifiedParty" ,                   "com.capgemini.nemmco.b2bb.NotifiedParty"       ,                   "com.capgemini.nemmco.b2bb.validators.NotifiedPartyBasicValidator"         ,                        "com.capgemini.nemmco.b2bb.validators.NotifiedPartyExtValidator"      ,     "NotifiedParty");
  public static final TransactionType PEER_TO_PEER             = new TransactionType("PTPDataExchange"      ,                  "Peer To Peer Data Exchange"                              , "PTPDataExchange"                   , "com.capgemini.nemmco.b2bb.PTPDataExchange"                      , "com.capgemini.nemmco.b2bb.validators.PTPDataExchangeBasicValidator"                              , "com.capgemini.nemmco.b2bb.validators.PTPDataExchangeBasicValidator"      ,     "PTPDataExchange");

  private static final TransactionType[] TRANSACTION_ARRAY = {
     //Matches by xpath in the order below
     SOR_CANCEL,  //Keep first

     SOR_ALLOCATE_NMI,
     SOR_NEW_CONNECTION,
     SOR_NEW_CONNECTION_T,
     SOR_NEW_CONNECTION_TP,
     SOR_SUPPLY_ALTERATION,
     SOR_SUPPLY_ABOLISHMENT,
     SOR_TEMP_ISOLATION,
     SOR_TEMP_ISOLATION_GRP,
     SOR_TARIFF_CHANGE, //Keep last

     SOR_MSW_INSTALL_CLOAD,
     SOR_MSW_INSTALL_METER,
     SOR_MSW_REMOVE_METER,
     SOR_MSW_MOVE_METER,
     SOR_MSW_EXCHANGE_METER,
     SOR_MSW_METER_RECONFIG,
     SOR_MSW_CHG_TSWITCH,
     SOR_MSW_METER_INV_INS,
     SOR_MSW_METER_INV_TEST,
     SOR_MSW_RESEAL, //Keep last

     SOR_REENERGISATION,
     SOR_REENERGISATION_PHY_VISIT,
     SOR_REENERGISATION_RETRO_MOVEIN,
     SOR_REENERGISATION_REC_DISCR,
     SOR_REENERGISATION_AFT_DISCONN,
	 SOR_REENERGISATION_NEW_READ,
     SOR_REENERGISATION_MOVEIN, //Keep last

      SOR_DEENERGISATION,
      SOR_DEENERGISATION_REMOTE,
      SOR_DEENERGISATION_LOC_MET_DISCONN,
      SOR_DEENERGISATION_REC_DISCR,
      SOR_DEENERGISATION_REMOVE_FUSE, //Keep last

      SOR_SPECIAL_READ_CHECK,
      SOR_SPECIAL_READ_FINAL,
      SOR_SPECIAL_READ, //Keep last

      SOR_MISCELLANEOUS,
      SOR_RESPONSE,
      SO_APPOINTMENT_NOT,

      CUST_DETAIL_NOT,
      SITE_ACCESS_REQ,
      SITE_ACCESS_NOT,
      SITE_ADDRESS_NOT,
      PROVIDE_METER_DATA_REQ,
      METER_DATA_VERIFY_REQ,
      CUST_DETAIL_REQUEST,
      METER_DATA_NOT,
      REMOTE_SERVICE_REQ,
      REMOTE_SERVICE_RES,
      SO_APPOINTMENT_NOT,
      ONE_WAY_NOT,
      PLANNED_INTERRUPT_NOT,
      METER_FAULT_AND_ISSUE_NOT,
      NOTICE_OF_METERING_WORKS,
	  NOTIFIED_PARTY,
	  PEER_TO_PEER

      };

  public static final List<TransactionType> TRANSACTIONS = Collections.unmodifiableList(Arrays.asList(TRANSACTION_ARRAY));

  protected TransactionType(String code,
      String description,
      String category,
      String objectClass,
      String basicValidatorClass,
      String extValidatorClass,
      String XMLIdentifier) {
    super(code, description, category, objectClass, basicValidatorClass, extValidatorClass, XMLIdentifier);
    this.number = ++count;
  }

   protected TransactionType(String code,
       String description,
       String subDescription,
       String category,
       String objectClass,
       String basicValidatorClass,
       String extValidatorClass,
       String XMLIdentifier) {
     super(code, description, subDescription, category, objectClass, basicValidatorClass, extValidatorClass, XMLIdentifier);
     this.number = ++count;
   }

  protected TransactionType(String code,
      String description,
      String category,
      String objectClass,
      String basicValidatorClass,
      String extValidatorClass) {
    super(code, description, category, objectClass, basicValidatorClass, extValidatorClass);
    this.number = ++count;
  }

   /**
    * Return the TransactionType object given by code (key)
    *
    * @param code
    *          The key to find in this enumeration
    * @return The object associated with the key. Null otherwise
    */
   public static TransactionType getTransactionType(String code) {
      for (TransactionType transactionType : TRANSACTIONS)
         if (transactionType.getCode().equals(code))
            return transactionType;
      return null;
  }

   public static String getCodeForDescSubDesc(String description, String subDescription) {
      for (TransactionType transactionType : TRANSACTIONS)
         if (transactionType.getDescription().equals(description) && transactionType.getSubDescription().equals(subDescription))
           return transactionType.getCode();
      return null;
   }
}
