/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/lookup/ScreenName.java-arc   1.1   21 Mar 2005 15:57:30   RayanS  $
 * $Revision:   1.1  $
 * $Date:   21 Mar 2005 15:57:30  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * Date Changed   Changed By        Comments
 * ------------   ----------        -----------
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb.lookup;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import com.capgemini.nemmco.b2bb.exceptions.UnableToLookupResourceException;

/**
 * Defines the lookup class for field to screen name
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.1  $ $Date:   21 Mar 2005 15:57:30  $
 */
public class ScreenName extends Lookup implements Comparable<Object>, Serializable {
  private static int count = 0;
  private static final String BASE_NAME = "com.capgemini.nemmco.b2bb.lookup.ScreenName";
  private static List<Object> list = null;
  
  private static String getChildCategory(String field) {
    int openIndex = field.indexOf("(");
    int closeIndex = field.lastIndexOf(")");
    
    if (openIndex > -1 && closeIndex > -1) {
      return field.substring(openIndex + 1, closeIndex);
    } else {
      return null;
    }
  }

  /**
   * @param category
   * @param field
   * @return
   * @throws UnableToLookupResourceException
   */
  public static String getScreenName(String category, String field) throws UnableToLookupResourceException {
    List<Object> list = getScreenNames();
    ScreenName screenName;
    String[] fields = null;
    
    if (field != null && field.indexOf(".") > -1) {
      fields = field.split("\\.");
    
	    if (fields != null) {
	      if (fields.length == 2) {
	        Iterator<Object> iter = list.iterator();
	        while (iter.hasNext()) {
	          screenName = (ScreenName) iter.next();
	          
	          if (screenName.getCode().matches(category + "\\.\\(\\w+\\)" + fields[0])) {
	            return screenName.getDescription() + " " + getScreenName(getChildCategory(screenName.getCode()), fields[1]);
	          }
	        }
	      }
	    }
    } else {
      Iterator<Object> iter = list.iterator();
      while (iter.hasNext()) {
        screenName = (ScreenName) iter.next();
        if (screenName.getCode().matches(category + "\\.\\(\\w+\\)" + field)) {
          return screenName.getDescription();
        } else if (screenName.getCode().equals(category + "." + field)) {
          return screenName.getDescription();
        }
      }
    }
    
    return null;
  }

  /**
   * @return
   * @throws UnableToLookupResourceException
   */
  public static List<Object> getScreenNames() throws UnableToLookupResourceException {
    if (list != null) return list;
    
    ArrayList<Object> values = new ArrayList<Object>();
    ResourceBundle resources = null;
    
    try {
      resources = ResourceBundle.getBundle(BASE_NAME);
    } catch (MissingResourceException e) {
      throw new UnableToLookupResourceException(e.getMessage());
    }
    
    java.util.Enumeration<String> en = resources.getKeys();
    while (en.hasMoreElements()) {
      String tmpKey = en.nextElement();
      values.add(new ScreenName(tmpKey, (String) resources.getObject(tmpKey)));
    }

    // create a list to return
    list = Collections.unmodifiableList(values);
    return list;
  }

  private ScreenName(String code, String description) {
    this.code = code;
    this.description = description;
    this.number = ++count;
  }
}

