/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/ValidationModule/src/com/capgemini/nemmco/b2bb/test/ValidationModuleTestHarness.java-arc   1.1   11 Apr 2005 09:47:04   RayanS  $
 * $Revision:   1.1  $
 * $Date:   11 Apr 2005 09:47:04  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * Date Changed   Changed By     Comments
 * ------------   ----------     -----------
 * 14-Aug-2012    Bob Jones      7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb.test;

import java.util.List;

import org.w3c.dom.Node;

import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.TransactionFactory;
import com.capgemini.nemmco.b2bb.enums.FlatOrUnitType;
import com.capgemini.nemmco.b2bb.enums.FloorOrLevelType;
import com.capgemini.nemmco.b2bb.enums.PostalDeliveryType;
import com.capgemini.nemmco.b2bb.enums.StreetSuffix;
import com.capgemini.nemmco.b2bb.enums.StreetType;
import com.capgemini.nemmco.b2bb.enums.TransactionType;
import com.capgemini.nemmco.b2bb.utils.B2BBDOMHelper;
import com.capgemini.nemmco.b2bb.utils.EventObject;
import com.capgemini.nemmco.b2bb.validators.Validator;
import com.capgemini.nemmco.b2bb.validators.ValidatorFactory;

import com.sun.xml.xsom.XSSchema;
import com.sun.xml.xsom.XSSchemaSet;
import com.sun.xml.xsom.parser.XSOMParser;

import java.io.File;

/**
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.1  $ $Date:   11 Apr 2005 09:47:04  $
 */
public class ValidationModuleTestHarness 
{
 
  public static void main(String[] args) throws Exception
  {
    // given a file name it will extract
    if (args.length!=3) {
      System.out.println("Usage: java program <Transaction Type> <Transaction filename> <Enumerations filename>");
      System.out.println("  Available Transaction Types are:");
      for (int i =0;i<TransactionType.TRANSACTIONS.size();i++){
        System.out.println("    "+((TransactionType) TransactionType.TRANSACTIONS.get(i)).getCode());
      }
     return;
    }
    
    String transactionType = args[0];
    String filename = args[1];
    String enumerations = args[2];
     
    try {      
      setB2BEnumerations(enumerations); //Def 7366 Eddie Graham     
    
      TransactionType tt = (TransactionType)TransactionType.getEnumeration(transactionType
                                                                           ,TransactionType.TRANSACTIONS);
      checkCode(tt, B2BBDOMHelper.getSelectedNode(B2BBDOMHelper.parseFile(filename),"aseXML/Transactions/Transaction"));
    } catch ( Exception e ) {
      System.out.println( e.getMessage() );
    }      
  }

  /**
    * 
    * @param tt
    * @param xmlSnippet
    * @throws Exception
    */
  private static void checkCode(TransactionType tt, Node xmlSnippet) throws Exception {

    // Ask the factory for a Site Access Notification Transaction
    Transaction sampleTrans = TransactionFactory.getTransaction(tt);

    // Set the XML Snippet for this transaction
    sampleTrans.setXMLString(xmlSnippet);
    
    // Ask the validator factory for a Site Access Notification Validator
    Validator sampleValidator = ValidatorFactory.getValidator(sampleTrans);
        
    // validate the transaction
    boolean result = sampleValidator.validate();
    
    // was an error found in validation
    List<EventObject> errorList = null;
    if (result)
    {
      System.out.println("No errors found in validation");
    } else 
    {
      System.out.println("Errors found in validation");
      // extract error list
      errorList = sampleValidator.getEvents();
      
      // extract and print out all the errors
      for (int i = 0; i < errorList.size(); i++) 
      {
        EventObject tmpErr = errorList.get(i);
        System.out.println("Error Context     : " + tmpErr.getContext());
        System.out.println("Error Code        : " + tmpErr.getEventCode());
        System.out.println("Error Description : " + tmpErr.getEventDesc());
        System.out.println("Error Explanation : " + tmpErr.getExplanation());
        System.out.println("Error Severity    : " + tmpErr.getSeverity());
        System.out.println("-----------------");
      }
    }
  }
  
   /**
   * Loads the enumerations - Defect 7366 - Eddie Graham
   * @param enumerations
   * @throws Exception
   */
   private static void setB2BEnumerations(String enumerations) throws Exception {
      File file = new File(enumerations);
      
      if (!file.exists()) {
         throw new Exception("Enumerations file does not exist - " + enumerations);
      }      
   
      XSOMParser parser = new XSOMParser();
      parser.parse(file);
      XSSchemaSet schemaSet = parser.getResult();
      XSSchema xsSchema = schemaSet.getSchema(1);
         
      FlatOrUnitType.setFlatOrUnitType(xsSchema);
      if ((FlatOrUnitType.FLAT_OR_UNIT_TYPES == null) || FlatOrUnitType.FLAT_OR_UNIT_TYPES.isEmpty()) {
         throw new Exception("Flat Or Unit Type - Enumerated list is empty." );            
      }
      
      FloorOrLevelType.setFloorOrLevelType(xsSchema);
      if ((FloorOrLevelType.FLOOR_OR_LEVEL_TYPES == null) || FloorOrLevelType.FLOOR_OR_LEVEL_TYPES.isEmpty()) {
         throw new Exception("Floor Or Level Type - Enumerated list is empty.");            
      }      
      
      PostalDeliveryType.setPostalDeliveryType(xsSchema);
      if ((PostalDeliveryType.POSTAL_DELIVERY_TYPE_CODES == null) || PostalDeliveryType.POSTAL_DELIVERY_TYPE_CODES.isEmpty()) {
         throw new Exception("Postal Delivery Type - Enumerated list is empty.");            
      }       
      
      StreetSuffix.setStreetSuffix(xsSchema);
      if ((StreetSuffix.STREET_SUFFIXES == null) || StreetSuffix.STREET_SUFFIXES.isEmpty()) {
         throw new Exception("Street Suffix - Enumerated list is empty.");            
      }      
      
      StreetType.setStreetType(xsSchema);  
      if ((StreetType.STREET_TYPES == null) || StreetType.STREET_TYPES.isEmpty()) {
         throw new Exception("Street Type - Enumerated list is empty.");            
      }      
   } // end setEnumerations    
}