/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/utils/BasicValidationUtility.java-arc   1.54   04 May 2005 16:01:02   RayanS  $
 * $Revision:   1.54  $
 * $Date:   04 May 2005 16:01:02  $
 *
 * Copyright 2005 Capgemini Australia Pty. Ltd.
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * 
 * Modification History
 * 
 * Date Changed   Changed By        Comments
 * ------------   ----------        -----------
 * 24-Mar-2006    Eddie Graham      Defect 3700 - Validation of NMI Checksum not allow entry of space. 
 * 26-Nov-2008    Bob Jones         d4826 Make private validateDateRange method public for use by ExtValidationUtility.
 * 01-Nov-2010    Bob Jones         6982 Fix Scheduled Date "not in past" validation in validateScheduleDateRange method.
 * 18-Mar-2011    Eddie Graham      Defect 7087. Add new method validateOneWayNotificationName() to validate One Way 
 *                                  Notification Name against One Way Notification Name enumeration.
 * 26-Jul-2011    Eddie Graham      Defect 7366. Modify method validateEnumeratedValue() to raise error if enumerated list is empty.
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 * 26-June-2017  SP                 SMP new screen- wk pkg 7b.
 * 27-Jun-2017    Harpreet Kaur     7135 Package 7B:Add new method validateMandatoryConditionally1
 * 30-Jun-2017    Harpreet Kaur     7135 Package 7B:Add new method to validate Duration
 * 10-Jul-2017  SP                 SMP new screen- wk pkg 7b.
 * 18-Jul-2017    Harpreet Kaur     7135 Package 7B: Fix for defects 7473, 7478, 7479
 * 31-Jul-2017    Hamish McNeish    Added DurationInterval validator (support 00:00 - 99:99)
 * 11-Sep-2017    Harpreet Kaur     Defect fix 7482 as part of POC SMP.
 * 28-Sep-2017    Harpreet Kaur     Defect fix 7482 as part of POC SMP.
 * 13-Nov-2017    Harpreet Kaur     Defect fix 8379 as part of POC SMP.
 */

package com.capgemini.nemmco.b2bb.utils;

import com.capgemini.nemmco.b2bb.XMLObject;
import com.capgemini.nemmco.b2bb.enums.ActionType;
import com.capgemini.nemmco.b2bb.enums.CustomerType;
import com.capgemini.nemmco.b2bb.enums.DeEnergisationReason;
import com.capgemini.nemmco.b2bb.enums.Enumeration;
import com.capgemini.nemmco.b2bb.enums.EventClass;
import com.capgemini.nemmco.b2bb.enums.EventCode;
import com.capgemini.nemmco.b2bb.enums.ExceptionCode;
import com.capgemini.nemmco.b2bb.enums.FlatOrUnitType;
import com.capgemini.nemmco.b2bb.enums.FloorOrLevelType;
import com.capgemini.nemmco.b2bb.enums.InstallationType;
import com.capgemini.nemmco.b2bb.enums.Market;
import com.capgemini.nemmco.b2bb.enums.MessageStatus;
import com.capgemini.nemmco.b2bb.enums.MeterInstallCode;
import com.capgemini.nemmco.b2bb.enums.MeteringType;
import com.capgemini.nemmco.b2bb.enums.NMIStatusCode;
import com.capgemini.nemmco.b2bb.enums.NameTitle;
import com.capgemini.nemmco.b2bb.enums.NameType;
import com.capgemini.nemmco.b2bb.enums.OneWayNotificationName;
import com.capgemini.nemmco.b2bb.enums.PhoneServiceComment;
import com.capgemini.nemmco.b2bb.enums.PhoneServiceType;
import com.capgemini.nemmco.b2bb.enums.PostalDeliveryType;
import com.capgemini.nemmco.b2bb.enums.Priority;
import com.capgemini.nemmco.b2bb.enums.ResponseType;
import com.capgemini.nemmco.b2bb.enums.SORRole;
import com.capgemini.nemmco.b2bb.enums.SafetyCertMethodSentType;
import com.capgemini.nemmco.b2bb.enums.ServiceOrderStatus;
import com.capgemini.nemmco.b2bb.enums.ServiceOrderSubType;
import com.capgemini.nemmco.b2bb.enums.ServiceOrderType;
import com.capgemini.nemmco.b2bb.enums.ServiceTime;
import com.capgemini.nemmco.b2bb.enums.Severity;
import com.capgemini.nemmco.b2bb.enums.State;
import com.capgemini.nemmco.b2bb.enums.Status;
import com.capgemini.nemmco.b2bb.enums.StreetSuffix;
import com.capgemini.nemmco.b2bb.enums.StreetType;
import com.capgemini.nemmco.b2bb.enums.SupplyPhase;
import com.capgemini.nemmco.b2bb.enums.TransactionGroup;
import com.capgemini.nemmco.b2bb.enums.TrueFalse;
import com.capgemini.nemmco.b2bb.enums.YesNo;
import com.capgemini.nemmco.b2bb.exceptions.UnableToLookupResourceException;
import com.capgemini.nemmco.b2bb.lookup.EventMessage;
import com.capgemini.nemmco.b2bb.lookup.ScreenName;
import com.capgemini.nemmco.b2bb.lookup.XPath;

import java.text.MessageFormat;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * The class <code>BasicValidationUtility</code> contains all validation
 * required by the Basic Validation classes.
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.54  $ $Date:   04 May 2005 16:01:02  $
 */
public class BasicValidationUtility {
  private static final String TIME24HOURS_PATTERN = "([01]?[0-9]|2[0-3]):[0-5][0-9]";
  private static final String TIMEINTERVAL_PATTERN = "([0-9][0-9]):[0-9][0-9]";
  private static final String MESSAGE_AND_TRANS_ID_PATTERN = "[<>&'\"]";

  static private BasicValidationUtility basicValidationUtility = null;
  protected String messagePrefix = "error.";  
  protected String errorDesc = "";
   
  /**
   * define the constructor as private so noone can instantiate this class
   * except through the getInstance method.
   *
   */
  protected BasicValidationUtility(){
  }

  /**
   * Returns an instance of the BasicValidationUtility
   *
   * @return an instance of the BasicValidationUtility
   */
  public static BasicValidationUtility getBasicInstance(){
    if (basicValidationUtility == null) {
      basicValidationUtility = new BasicValidationUtility();
    }
    return basicValidationUtility;
  }

 /**
  * Perform basic validation on a participant
  *
  * @param errorList list of events to add any generated events to
  * @param transactionType being validated
  * @param fieldname in the transaction
  * @param participant participant id to validate
  * @return True if validation passes. False otherwise.
  */
  public boolean validateParticipant(List<EventObject> errorList,
                                     String transactionType,
                                     String fieldname,
                                     String participant){
    return validateLengthRange(errorList, transactionType, fieldname, participant, 0, 10);
  }

 /**
  * Perform length validation on a string
  *
  * @param errorList list of events to add any generated events to
  * @param transactionType being validated
  * @param fieldname in the transaction
  * @param value to validate
  * @param minlength to check the value adheres to (inclusive)
  * @param maxlength to check the value adheres to (inclusive)
  * @return True if validation passes. False otherwise.
  */
  public boolean validateLengthRange(List<EventObject> errorList,
                                     String transactionType,
                                     String fieldname,
                                     String value,
                                     int minlength,
                                     int maxlength){
    try {
      // set null to an empty string
      if (value == null) value = "";
	    if (value.length() >= minlength && value.length() <= maxlength){
	      return true;
	    }

	    // define variable to add to explanation
	    String description = minlength + "";
	    if (minlength != maxlength) {
	      description = minlength + " to " + maxlength;
	    }
	    Object[] args = {description, new Long(value.length())};
      addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_0202,"invalidLength");

    } catch (UnableToLookupResourceException e){
      unexpectedException(errorList, fieldname,e.getMessage());
    }

    return false;
  }

  /**
   * Perform length validation on a string
   *
   * @param errorList list of events to add any generated events to
  * @param transactionType being validated
  * @param fieldname in the transaction
   * @param value to validate
   * @param length to check the value adheres to (inclusive)
   * @return True if validation passes. False otherwise.
   */
   public boolean validateLength(List<EventObject> errorList,
                                 String transactionType,
                                 String fieldname,
                                 String value,
                                 int length){
     return validateLengthRange(errorList, transactionType, fieldname, value, length, length);
   }

 /**
  * Perform priority validation on a string
  *
  * @param errorList list of events to add any generated events to
  * @param transactionType being validated
  * @param fieldname in the transaction
  * @param value to validate
  * @return True if validation passes. False otherwise.
  */
  public boolean validatePriority(List<EventObject> errorList,
                                  String transactionType,
                                  String fieldname,
                                  String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Priority",
        Priority.PRIORITIES,
        value);
  }

  /**
   * Perform priority validation on a string
   *
   * @param errorList list of events to add any generated events to
   * @param transactionType being validated
   * @param fieldname in the transaction
   * @param value to validate
   * @return True if validation passes. False otherwise.
   */
   public boolean validateActionType(List<EventObject> errorList,
                                   String transactionType,
                                   String fieldname,
                                   String value){
     return validateEnumeratedValue(errorList,
         transactionType,
         fieldname,
         "Action Type",
         ActionType.ACTION_TYPE_CODES,
         value);
   }

 /**
  * Perform market validation on a string
  *
  * @param errorList list of events to add any generated events to
  * @param transactionType being validated
  * @param fieldname in the transaction
  * @param value to validate
  * @return True if validation passes. False otherwise.
  */
  public boolean validateMarket(List<EventObject> errorList,
                                String transactionType,
                                String fieldname,
                                String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Market",
        Market.MARKETS,
        value);
  }

 /**
  * Perform validation on a transaction id string
  *
  * @param errorList list of events to add any generated events to
  * @param transactionType being validated
  * @param fieldname in the transaction
  * @param value to validate
  * @return True if validation passes. False otherwise.
  */
  public boolean validateTransactionId(List<EventObject> errorList,
                                       String transactionType,
                                       String fieldname,
                                       String value){
    return validateLengthRange(errorList, transactionType, fieldname, value, 0, 36);
  }

 /**
  * Perform validation on a nmi.
  *
  * @param errorList list of events to add any generated events to
  * @param transactionType being validated
  * @param fieldname in the transaction
  * @param checksum to validate
  * @return True if validation passes. False otherwise.
  */
  public boolean validateNMI(List<EventObject> errorList,
                             String transactionType,
                             String fieldname,
                             String nmi,
                             String checksum){

    if(nmi==null) nmi="";
      // validate the nmi format
    if (!validateLength(errorList, transactionType, fieldname, nmi, 10)) return false;
    if (!validateAlphaNumeric(errorList, transactionType, fieldname, nmi)) return false;


    return true;
  }

  /**
   * Perform validation on a nmi checksum.
   *
   * @param errorList list of events to add any generated events to
   * @param transactionType being validated
   * @param fieldname in the transaction
   * @param checksum to validate
   * @return True if validation passes. False otherwise.
   */
   public boolean validateNMIChecksum(List<EventObject> errorList,
                                      String transactionType,
                                      String fieldname,
                                      String nmi,
                                      String checksum){

     // if checksum doesnt exist then it is valid
     // 24Mar2006 Eddie Graham Defect 3700 Validation of NMI Checksum not allow entry of space 
     if (checksum == null || checksum.equals("")) return true;     
     // validate the checksum length
     if (!validateLength(errorList, transactionType, fieldname, checksum, 1)) return false;
     // validate the checksum is a number
     if (!validateNumeric(errorList, transactionType, fieldname, checksum)) return false;
     // validate the checksum value

     int inputChecksum = Integer.parseInt(checksum);
     int d;
     int s = 0;
     char[] stringChar = nmi.toCharArray();
     for (int i = 0 ;i < stringChar.length; i++){
       char nextChar = stringChar[i];
       //take each digit in turn
       d = nextChar;
       // if its even double id
       if ((i+1) % 2 == 0) {
         d = d * 2;
       }
       int digitsum = 0;
       // sum each digit of the ascii value
       while (d > 0) {
         digitsum = digitsum + (d % 10);
         d = d / 10;
       }
       s = s + digitsum;
     }

     int calculatedChecksum = ((((s/10) + 1) * 10) - s) % 10;

     if (calculatedChecksum == inputChecksum) {
       return true;
     }

     try {

       // define variable to add to explanation
       Object[] args = {"NMI Checksum", checksum};
       addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_1156,"invalidValue");

     } catch (UnableToLookupResourceException e){
       unexpectedException(errorList, fieldname, e.getMessage());
     }

     return false;
   }

 /**
  * Checks a field exists. This test applies for elements which are mandatory
  * on the basis of other data elements being provided.
  *
  * @param errorList list of events to add any generated events to
  * @param transactionType being validated
  * @param fieldname in the transaction
  * @param value to validate
  * @return True if validation passes. False otherwise.
  */
  public boolean validateMandatory(List<EventObject> errorList,
                                     String transactionType,
                                     String fieldname,
                               		   String value){
    return validateMandatory(errorList,transactionType,fieldname,value, EventCode.CODE_0201);
  }

  /**
   * Checks a field exists. This is used for elements that are mandatory in all cases in
   * the B2B procedure.
   *
   * @param errorList list of events to add any generated events to
   * @param transactionType being validated
   * @param fieldname in the transaction
   * @param value to validate
   * @return True if validation passes. False otherwise.
   */
   public boolean validateMandatoryAllCases(List<EventObject> errorList,
                                            String transactionType,
                                            String fieldname,
                                            String value){
     return validateMandatory(errorList,transactionType,fieldname,value, EventCode.CODE_1950);
   }

 /**
  * Checks a field exists
  */
  private boolean validateMandatory(List<EventObject> errorList,
                                   String transactionType,
                                   String fieldname,
                                   String value,
                                   EventCode eventCode){
    try {
      if (value!=null && !(value.trim()).equals("")){
        return true;
      }

      // define variable to add to explanation
      Object[] args = {fieldname};
      addEvent(errorList,transactionType,fieldname,args,eventCode,"mandatoryMissing");

    } catch (UnableToLookupResourceException e){
      unexpectedException(errorList, fieldname,e.getMessage());
    }

    return false;
  }

 /**
  * Checks a value is numeric
  *
  * @param errorList list of events to add any generated events to
  * @param transactionType being validated
  * @param fieldname in the transaction
  * @param value to validate
  * @return True if validation passes. False otherwise.
  */
  public boolean validateNumeric(List<EventObject> errorList,
                                 String transactionType,
                                 String fieldname,
                                 String value){
    boolean result;
    String numericPattern = "[\\d]+";

    result = value.matches(numericPattern);
    if (!result)
    {
      // define variable to add to explanation
      Object[] args = {value};
      try
      {
        addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "notNumeric");
      }
      catch (UnableToLookupResourceException e)
      {
        unexpectedException(errorList, fieldname, e.getMessage());
      }
    }
    return result;
  }

 /**
  * Checks a value is numeric
  *
  * @param errorList list of events to add any generated events to
  * @param transactionType being validated
  * @param fieldname in the transaction
  * @param value to validate
  * @return True if validation passes. False otherwise.
  */
  public boolean validateNumericDecimal(List<EventObject> errorList,
                                        String transactionType,
                                        String fieldname,
                                        String value,
                                        int length,
                                        int decimalPlaces)
  {
    boolean result = true;
    String tmpValue = value;
    // check we have only 1 decimal point
    if (value.indexOf('.')!=value.lastIndexOf('.')){
      result = false;

    // check length is correct
    } else if (tmpValue.replaceAll("[.]","").length()>length){
      result = false;

    } else {
      // remove the decimal and check its a numeric
      tmpValue=value;
      if (!validateNumeric(errorList,transactionType,fieldname,tmpValue.replaceAll("[.]",""))){
        return false;
      }
      // finally check we have the correct number of decimals after the decimal point
      if (value.indexOf('.')==-1) return true;
      tmpValue = value.substring(value.lastIndexOf('.')+1);
      if (!(tmpValue.length()<=decimalPlaces && tmpValue.length()>0)){
        result = false;
      }
    }
 	  if (!result)
    {
      Object[] args = {value};
      try
      {
        addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "notNumeric");
      }
      catch (UnableToLookupResourceException e)
      {
        unexpectedException(errorList, fieldname, e.getMessage());
      }
    }
    return result;
  }

  /**
   * Checks a date is in the B2B Date format
   *
   * @param errorList list of events to add any generated events to
   * @param transactionType being validated
   * @param fieldname in the transaction
   * @param date to validate
   * @return True if validation passes. False otherwise.
   */
   public boolean validateB2BDate(List<EventObject> errorList,
                                  String transactionType,
                                  String fieldname,
                                  String date) {
     Object[] args = new Object[2]; 
     
     try {          
       // check the date is parsable into an XML date string
       try {
         new B2BDateFormatter(date, B2BDateFormatter.B2BDATE);
         // all ok
         return true;
       } catch (Exception e){
         ;
       }

       // define variable to add to explanation
       if (errorDesc.length() > 0) { // customised date description set by wrapper method: use this
         args[0] = errorDesc;
         args[1] = date;
       } else { // not customised date description: use B2B default for backward compatibility
         args[0] = "B2B Date";
         args[1] = date;
       }
       addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_0202,"invalidDateTime");

     } catch (UnableToLookupResourceException e){
       unexpectedException(errorList, fieldname, e.getMessage());
     }

     return false;
   }
   
  /**
   * Wrapper method to check that a date is in the B2B Date format 
   * but allow a customised date description in the error message
   *
   * @param errorList list of events to add any generated events to
   * @param transactionType being validated
   * @param fieldname in the transaction
   * @param date to validate
   * @param dateDesc customised date description for error message
   * @return True if validation passes. False otherwise.
   */   
   
   public boolean validateDate(List<EventObject> errorList,
                               String transactionType,
                               String fieldname,
                               String date,
                               String dateDesc) {
     boolean valid = false;
     
     errorDesc = dateDesc;
     
     try {
       valid = validateB2BDate(errorList, transactionType, fieldname, date);
     } catch (Exception e){
       ;
     }

     return valid;
   }
   
   /**
    * Checks a time is in the B2B Time format
    *
    * @param errorList list of events to add any generated events to
    * @param transactionType being validated
    * @param fieldname in the transaction
    * @param date to validate
    * @return True if validation passes. False otherwise.
    */
    public boolean validateB2BTime(List<EventObject> errorList,
                                   String transactionType,
                                   String fieldname,
                                   String date) {
      Object[] args = new Object[2]; 
       
      try {
        // check the date is parsable into an XML date string
        try {
          new B2BTimeFormatter(date, B2BTimeFormatter.B2BTIME);
          // all ok
          return true;
        } catch (Exception e){
          ;        
        }

        // define variable to add to explanation
        if (errorDesc.length() > 0) { // customised date description set by wrapper method: use this
           args[0] = errorDesc;
           args[1] = date;
        } else { // not customised date description: use B2B default for backward compatibility
           args[0] = "B2B Time";
           args[1] = date;
        }        
        addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_0202,"invalidDateTime");

      } catch (UnableToLookupResourceException e){
        unexpectedException(errorList, fieldname, e.getMessage());
      }

      return false;
    }
    
   /**
    * Wrapper method to check that a date is in the B2B Time format 
    * but allow a customised time description in the error message
    *
    * @param errorList list of events to add any generated events to
    * @param transactionType being validated
    * @param fieldname in the transaction
    * @param time to validate
    * @param timeDesc customised date description for error message
    * @return True if validation passes. False otherwise.
    */   
    
    public boolean validateTime(List<EventObject> errorList,
                                String transactionType,
                                String fieldname,
                                String time,
                                String timeDesc) {
      boolean valid = false;
      
      errorDesc = timeDesc;
      
      try {
        valid = validateB2BTime(errorList, transactionType, fieldname, time);
      } catch (Exception e){
        e = null;
      }

      return valid;
    }    

  /**
   * Checks a date is in the XML Date format
   *
   * @param errorList list of events to add any generated events to
   * @param transactionType being validated
   * @param fieldname in the transaction
   * @param date to validate
   * @return True if validation passes. False otherwise.
   */
   public boolean validateXMLDate(List<EventObject> errorList,
                                  String transactionType,
                                  String fieldname,
                                  String date){
     try {
       // check the date is parsable into an XML date string
       try {
         new B2BDateFormatter(date, B2BDateFormatter.XMLDATE);
         // all ok
         return true;
       } catch (Exception e){
         ;       
       }

       // define variable to add to explanation
       Object[] args = {"XML Date", date};
       addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_0202,"invalidDateTime");

     } catch (UnableToLookupResourceException e){
       unexpectedException(errorList, fieldname, e.getMessage());
     }

     return false;
   }
   /**
    * Checks a time is in the XML Time format
    *
    * @param errorList list of events to add any generated events to
    * @param transactionType being validated
    * @param fieldname in the transaction
    * @param date to validate
    * @return True if validation passes. False otherwise.
    */
    public boolean validateXMLTime(List<EventObject> errorList,
                                   String transactionType,
                                   String fieldname,
                                   String date){
      try {
        // check the date is parsable into an XML date string
        try {
         new B2BTimeFormatter(date, B2BTimeFormatter.XMLTIME);
          // all ok
          return true;
        } catch (Exception e){
          ;        
        }

        // define variable to add to explanation
        Object[] args = {"XML Time", date};
        addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_0202,"invalidDateTime");

      } catch (UnableToLookupResourceException e){
        unexpectedException(errorList, fieldname, e.getMessage());
      }

      return false;
    }

 /**
  * Checks a date is in the XML Date format
  *
  * @param errorList list of events to add any generated events to
  * @param transactionType being validated
  * @param fieldname in the transaction
  * @param date to validate
  * @return True if validation passes. False otherwise.
  */
  public boolean validateXMLDateTime(List<EventObject> errorList,
                                 String transactionType,
                                 String fieldname,
                                 String date){
    try {
      // check the date is parsable into an XML date string
      try {
        new B2BDateTimeFormatter(date, B2BDateTimeFormatter.XMLDATE);
        // all ok
        return true;
      } catch (Exception e){
        ;      
      }

      // define variable to add to explanation
      Object[] args = {"XML Date Time", date};
      addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_0202,"invalidDateTime");

    } catch (UnableToLookupResourceException e){
      unexpectedException(errorList, fieldname, e.getMessage());
    }

    return false;
  }

  /**
   * Checks a date is in the B2B screen date
   *
   * @param errorList list of events to add any generated events to
   * @param transactionType being validated
   * @param fieldname in the transaction
   * @param date to validate
   * @return True if validation passes. False otherwise.
   */
   public boolean validateB2BDateTime(List<EventObject> errorList,
                                   String transactionType,
                                  String fieldname,
                                  String date){
     try {
       // check the date is parsable into an B2B date string
       try {
         new B2BDateTimeFormatter(date, B2BDateTimeFormatter.B2BDATE);
         // all ok
         return true;
       } catch (Exception e){
  	      ;
  	    }

  	   // define variable to add to explanation
  	   Object[] args = {"B2B Date Time", date};
       addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_0202,"invalidDateTime");

     } catch (UnableToLookupResourceException e){
       unexpectedException(errorList, fieldname, e.getMessage());
     }

     return false;
   }

 /**
  * Checks a value using regular expressions
  *
  * @param errorList list of events to add any generated events to
  * @param transactionType being validated
  * @param fieldname in the transaction
  * @param value to validate
  * @param pattern the regular expression pattern
  * @return True if validation passes. False otherwise.
  */
  public boolean validatePattern(List<EventObject> errorList,
                                 String transactionType,
                                 String fieldname,
                                 String value,
                                 String pattern){
    try {
      if(Pattern.matches(pattern, value)){
        return true;
      }

 	    // define variable to add to explanation
 	    Object[] args = {value,pattern};
      addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_0202,"invalidPattern");

    } catch (UnableToLookupResourceException e){
      unexpectedException(errorList, fieldname, e.getMessage());
    }

    return false;
  }
  
  /**
   * Validates the Status
   *
   * @param errorList list of events to add any generated events to
   * @param transactionType being validated
   * @param fieldname in the transaction
   * @param value to validate
   * @return True if validation passes. False otherwise.
   */
   public boolean validateStatus(List<EventObject> errorList,
                                 String transactionType,
                                 String fieldname,
                                 String value){
     return validateEnumeratedValue(errorList,
         transactionType,
         fieldname,
         "Status",
         Status.STATUSES,
         value);
   }
   
 /**
  * Validates the Status
  *
  * @param errorList list of events to add any generated events to
  * @param transactionType being validated
  * @param fieldname in the transaction
  * @param value to validate
  * @return True if validation passes. False otherwise.
  */
  public boolean validateMessageStatus(List<EventObject> errorList,
                                String transactionType,
                                String fieldname,
                                String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Status",
        MessageStatus.STATUSES,
        value);
  }

  /**
   * Validates the Transaction Group
   *
   * @param errorList list of events to add any generated events to
   * @param transactionType being validated
   * @param fieldname in the transaction
   * @param value to validate
   * @return True if validation passes. False otherwise.
   */
   public boolean validateTransactionGroup(List<EventObject> errorList,
                                           String transactionType,
                                           String fieldname,
                                           String value){
     return validateEnumeratedValue(errorList,
         transactionType,
         fieldname,
         "Transaction Group",
         TransactionGroup.TRANSACTION_GROUPS,
         value);
   }

  /**
   * validate flatOrUnitType
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateFlatOrUnitType(List<EventObject> errorList,
                                  String transactionType,
                                  String fieldname,
                                  String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Flat Or Unit Type",
        FlatOrUnitType.FLAT_OR_UNIT_TYPES,
        value);
  }

  /**
   * validate postalDeliveryType
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validatePostalDeliveryType(List<EventObject> errorList,
                                            String transactionType,
                                            String fieldname,
                                            String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "PostalDeliveryType",
        PostalDeliveryType.POSTAL_DELIVERY_TYPE_CODES,
        value);
  }

  /**
   * validate service order type
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param serviceOrderType
   * @param validType If null will check the transaction type list otherwise
   * will compare to the service order type provided
   * @return
   */
  public boolean validateServiceOrderType(List<EventObject> errorList,
                                          String transactionType,
                                          String fieldname,
                                          String serviceOrderType,
                                          String validType){
    if (!serviceOrderType.equals(validType)){
      try {
        Object[] args = new Object[2];
        args[0] = "Service Order Type";
        args[1] = serviceOrderType;
        addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "invalidValue");
      } catch (UnableToLookupResourceException e){
        unexpectedException(errorList, fieldname, e.getMessage());
      }
      return false;
    }
    return true;
  }

  /**
   * validate service order type
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateServiceOrderType(List<EventObject> errorList,
                                          String transactionType,
                                          String fieldname,
                                          String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Service Order Type",
        ServiceOrderType.SERVICE_ORDER_TYPES,
        value);
  }

  /**
   * validate service order sub type
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param serviceOrderType
   * @param serviceOrderSubType
   * @return
   */
  public boolean validateServiceOrderSubType(List<EventObject> errorList,
                                             String transactionType,
                                             String fieldname,
                                             String serviceOrderType,
                                             String serviceOrderSubType){
    try {
      //Enumeration tmpEnum = ServiceOrderSubType.getEnumeration(serviceOrderSubType,ServiceOrderSubType.SERVICE_ORDER_SUB_TYPES);
      //Match by ordertype and subtype
      ServiceOrderSubType tmpServiceOrderSubType = null;
      for (int i=0;i<ServiceOrderSubType.SERVICE_ORDER_SUB_TYPES.size(); i++) {
      	ServiceOrderSubType sub = (ServiceOrderSubType) ServiceOrderSubType.SERVICE_ORDER_SUB_TYPES.get(i);
      	if (sub.getServiceOrderType().getCode().equals(serviceOrderType) && sub.getCode().equals(serviceOrderSubType)) {
      		tmpServiceOrderSubType = sub;
      		break;
      	}
      }
      if (tmpServiceOrderSubType == null){
        // define variable to add to explanation
        Object[] args = {serviceOrderSubType, serviceOrderType};
        addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_1910,"invalidServiceOrderSubTypeMatch");
        return false;
      }

      if (((tmpServiceOrderSubType.getServiceOrderType()).getCode()).equals(serviceOrderType)){
        return true;
      }
      // define variable to add to explanation
      Object[] args = {serviceOrderSubType, serviceOrderType};
      addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_1910,"invalidServiceOrderSubTypeMatch");

    } catch (UnableToLookupResourceException e){
      unexpectedException(errorList, fieldname, e.getMessage());
    }

    return false;
  }

  /**
   * validate yes/no type
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateYesNo(List<EventObject> errorList,
                               String transactionType,
                               String fieldname,
                               String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Yes/No",
        YesNo.YESNO_CODES,
        value);
  }

  /**
   * validate true/false type
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateTrueFalse(List<EventObject> errorList,
                               String transactionType,
                               String fieldname,
                               String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "True/False",
        TrueFalse.TRUEFALSE_CODES,
        value);
  }
  
  /**
   * validate event class type
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateEventClass(List<EventObject> errorList,
                               String transactionType,
                               String fieldname,
                               String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Event Class",
        EventClass.EVENT_CLASSES,
        value);
  }
  
  /**
   * validate Severity type
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateSeverity(List<EventObject> errorList,
                                 String transactionType,
                                 String fieldname,
                                 String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Severity",
        Severity.SEVERITY,
        value);
  }

  /**
   * validate Service Order Request Role
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateSORRole(List<EventObject> errorList,
                                 String transactionType,
                                 String fieldname,
                                 String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Service Order Request Role",
        SORRole.SOR_ROLES,
        value);
  }

  /**
   * validate service time
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateServiceTime(List<EventObject> errorList,
                                          String transactionType,
                                          String fieldname,
                                          String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Service Time",
        ServiceTime.SERVICE_TIME_TYPES,
        value);
  }

  /**
   * validate InstallationType
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateInstallationType(List<EventObject> errorList,
                                          String transactionType,
                                          String fieldname,
                                          String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Installation Type",
        InstallationType.INSTALLATION_TYPES,
        value);
  }

  /**
   * validate Supply Phase
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateSupplyPhase(List<EventObject> errorList,
                                      String transactionType,
                                      String fieldname,
                                      String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Supply Phase",
        SupplyPhase.SUPPLY_PHASE_TYPES,
        value);
  }

  /**
   * validate Metering Required
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateMeteringRequired(List<EventObject> errorList,
                                          String transactionType,
                                          String fieldname,
                                          String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Meter Required",
        MeteringType.METERING_TYPES,
        value);
  }

  /**
   * validate Meter Install Code
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateMeterInstallCode(List<EventObject> errorList,
                                          String transactionType,
                                          String fieldname,
                                          String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Meter Install Code",
        MeterInstallCode.METER_INSTALL_CODES,
        value);
  }

 /**
   * validate floorOrLevelType
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateFloorOrLevelType(List<EventObject> errorList,
                                  String transactionType,
                                  String fieldname,
                                  String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Floor Or Level Type",
        FloorOrLevelType.FLOOR_OR_LEVEL_TYPES,
        value);
  }

  /**
   * Validate NMI Status Code
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateNMIStatusCode(List<EventObject> errorList,
                                      String transactionType,
                                      String fieldname,
                                      String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "NMI Status Code",
        NMIStatusCode.NMI_STATUS_CODES,
        value);
  }

  /**
   * Validate DeEnergisation Reason
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateDeEnergisationReason(List<EventObject> errorList,
                                      String transactionType,
                                      String fieldname,
                                      String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "De-Enerisation Reason",
        DeEnergisationReason.DEENERGISATION_REASON,
        value);
  }

  /**
   * Validate Customer Type
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateCustomerType(List<EventObject> errorList,
                                      String transactionType,
                                      String fieldname,
                                      String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Customer Type",
        CustomerType.CUSTOMER_TYPES,
        value);
  }

  /**
   * Validate streetType
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateStreetType(List<EventObject> errorList,
                                  String transactionType,
                                  String fieldname,
                                  String value){
    return validateEnumeratedValue(errorList,
        transactionType,
        fieldname,
        "Street Type",
        StreetType.STREET_TYPES,
        value);
  }

  /**
   * Validate streetSuffix
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateStreetSuffix(List<EventObject> errorList,
                                  String transactionType,
                                  String fieldname,
                                  String value){
    return validateEnumeratedValue(errorList,
                                   transactionType,
                                   fieldname,
                                   "Street Suffix",
                                   StreetSuffix.STREET_SUFFIXES,
                                   value);
  }

  /**
   * Validate Phone Service Type
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validatePhoneServiceType(List<EventObject> errorList,
                                     String transactionType,
                                     String fieldname,
                                     String value){

    return validateEnumeratedValue(errorList,
                                   transactionType,
                                   fieldname,
                                   "Phone Service Type",
                                   PhoneServiceType.PHONE_SERVICE_TYPES,
                                   value);
  }

  /**
   * Validate Phone Service Comment
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validatePhoneServiceComment(List<EventObject> errorList,
                                             String transactionType,
                                             String fieldname,
                                             String value){

    return validateEnumeratedValue(errorList,
                                   transactionType,
                                   fieldname,
                                   "Phone Service Comment",
                                   PhoneServiceComment.PHONE_SERVICE_COMMENTS,
                                   value);
  }

/**
   * Validate state
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateState(List<EventObject> errorList,
                               String transactionType,
                               String fieldname,
                               String value){

    return validateEnumeratedValue(errorList,
                                   transactionType,
                                   fieldname,
                                   "State",
                                   State.STATES,
                                   value);
  }

  /**
   * Validate Name Title
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateNameTitle(List<EventObject> errorList,
                                   String transactionType,
                                   String fieldname,
                                   String value){

    return validateEnumeratedValue(errorList,
                                   transactionType,
                                   fieldname,
                                   "Name Title",
                                   NameTitle.NAME_TITLES,
                                   value);
  }

  /**
   * If one value exists both have to exist
   * @param errorList
   * @param transactionType
   * @param fieldname1
   * @param fieldname2
   * @param value1
   * @param value2
   * @return
   */
  public boolean validateDependancy(List<EventObject> errorList,
                                   String transactionType,
                                   String fieldname1,
                                   String fieldname2,
                                   String value1,
                                   String value2){
    boolean tmpResult = true;

    if (value1 == null || value2 == null) tmpResult = false;
    else {
      // check none are blank
      // 24Mar2006 Eddie Graham Defect 3700 Remove trim to ensure validation failure if space is entered         
      //if (value1.trim().equals("") || value2.trim().equals("")) tmpResult = false;
      if (value1.equals("") || value2.equals("")) tmpResult = false;      
    }
    if (!tmpResult){
      try {
        // add and event
        Object[] args = new Object[2];
        args[0] = fieldname1;
        args[1] = fieldname2;
        addEvent(errorList, transactionType, fieldname1, args, EventCode.CODE_0202, "invalidDependancy");
      } catch (UnableToLookupResourceException e)
      {
        unexpectedException(errorList, fieldname1, e.getMessage());
      }
    }

    return tmpResult;
  }

  /**
   * If field1 is dependant on field 2 but not vice versa
   * @param errorList
   * @param transactionType
   * @param fieldname1
   * @param fieldname2
   * @param value1
   * @param value2
   * @return
   */
  public boolean validateOneWayDependancy(List<EventObject> errorList,
                                   String transactionType,
                                   String fieldname1,
                                   String fieldname2,
                                   String value1,
                                   String value2){
    boolean tmpResult = true;

    if (value1 == null || value2 == null) tmpResult = false;
    else {
      // check none are blank
      // 24Mar2006 Eddie Graham Defect 3700 Remove trim to ensure validation failure if space is entered       
      //if (value1.trim().equals("") || value2.trim().equals("")) tmpResult = false;
      if (value1.equals("") || value2.equals("")) tmpResult = false;      
    }
    if (!tmpResult){
      try {
        // add and event
        Object[] args = new Object[2];
        args[0] = ScreenName.getScreenName(transactionType, fieldname1);
        args[1] = ScreenName.getScreenName(transactionType, fieldname2);
        addEvent(errorList, transactionType, fieldname1, args, EventCode.CODE_0202, "invalidOneWayDependancy");
      } catch (UnableToLookupResourceException e)
      {
        unexpectedException(errorList, fieldname1, e.getMessage());
      }
    }

    return tmpResult;
  }

  /**
   * Validates a unique identifier
   *
   * @param errorList list of events to add any generated events to
   * @param transactionType being validated
   * @param fieldname in the transaction
   * @param value to validate
   * @return True if validation passes. False otherwise.
   */
   public boolean validateUniqueIdentifier(List<EventObject> errorList,
                                           String transactionType,
                                           String fieldname,
                                           String value) {

    return validateLengthRange(errorList, transactionType, fieldname, value, 1, 36);
  }


  /**
   * Validates the Transaction Group
   *
   * @param errorList list of events to add any generated events to
   * @param transactionType being validated
   * @param fieldname in the transaction
   * @param value to validate
   * @param allowedValue value allowed
   * @return True if validation passes. False otherwise.
   */
   public boolean validateAllowedValue(List<EventObject> errorList,
                                           String transactionType,
                                           String fieldname,
                                           String value,
                                           String allowedValue) {
    try {
      if (value.equals(allowedValue))
        return true;

 	  // define variable to add to explanation
 	  Object[] args = {allowedValue, value};
      addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_0202,"notAllowedValue");

    } catch (UnableToLookupResourceException e){
      unexpectedException(errorList, fieldname, e.getMessage());
    }

    return false;

   }

  /**
   * Only one value in a list of choices may exist
   * @param errorList
   * @param transactionType
   * @param fieldnames
   * @param values
   * @param mandatory True or false depending on whether the choice is mandatory
   * @return
   */
  public boolean validateChoice(List<EventObject> errorList,
                                String transactionType,
                                String[] fieldnames,
                                String[] values,
                                boolean mandatory) {  // Is it mandatory to provide a value

    int count = 0;
    for (int i = 0; i < values.length; i++) {
      if (exists(values[i])) {
        count++;
      }
    }

    boolean tmpResult = (mandatory ? count == 1 : count <= 1);

    if (!tmpResult){
      try {
        String choices = "[";
        for (int i = 0; i < fieldnames.length; i++) {
          choices += ScreenName.getScreenName(transactionType, fieldnames[i]);
          if (i < fieldnames.length - 1)
            choices += ", ";
        }
        choices += "]";
        // add and event
        EventCode eventCode = (count == 0 ? EventCode.CODE_0201 : EventCode.CODE_0202);
        String tmpMandatory = (mandatory ? "invalidChoiceMandatory" : "invalidOptional");
        addEvent(errorList, transactionType, fieldnames[0], new String[]{choices}, eventCode, tmpMandatory);
      } catch (UnableToLookupResourceException e)
      {
        unexpectedException(errorList, fieldnames[0], e.getMessage());
      }
    }

    return tmpResult;
  }

  /**
   * Validate Name Type
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateNameType(List<EventObject> errorList,
                                  String transactionType,
                                  String fieldname,
                                  String value){

    return validateEnumeratedValue(errorList, transactionType, fieldname, "Name Type", NameType.NAME_TYPES, value);
  }

  /**
   * Validate Safety Certificate Method Sent
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateSafetyCertMethodSent(List<EventObject> errorList,
                                              String transactionType,
                                              String fieldname,
                                              String value){

    return validateEnumeratedValue(errorList,
                                   transactionType,
                                   fieldname,
                                   "Safety Certificate Method Sent",
                                   SafetyCertMethodSentType.SAFETY_CERT_METHOD_SENT_TYPES,
                                   value);
  }


  /**
   * Validate Metering Safety Certificate Method Sent
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @return
   */
  public boolean validateMeteringSafetyCertMethodSent(List<EventObject> errorList,
                                              String transactionType,
                                              String fieldname,
                                              String value){

    return validateEnumeratedValue(errorList,
                                   transactionType,
                                   fieldname,
                                   "Metering Safety Certificate Method Sent",
                                   SafetyCertMethodSentType.SAFETY_CERT_METHOD_SENT_TYPES,
                                   value);
  }

  public boolean checkDuplicate(List<EventObject> errorList,
                                String transactionType,
                                String fieldname,
                                String FieldType,
                                HashMap<Object,Object> hash,
                                String value){
    boolean result = true;
    if (hash.containsKey(value)){
      try {
        // duplicate
        result = false;
        Object[] args = new Object[2];
        args[0] = FieldType;
        args[1] = value;
        addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "duplicateRole");
      }
      catch (UnableToLookupResourceException e)
      {
        unexpectedException(errorList, fieldname, e.getMessage());
      }
    } else {
      hash.put(value,value);
    }
    return result;
  }

 /**
  * Checks if a value exists
  *
  * @param value
  * @return True if value exists. False otherwise.
  */
  public boolean exists(String value){
    // 24Mar2006 Eddie Graham Defect 3700 Remove trim so if space is entered returns true
    return (value!=null && !value.equals(""));
  }

  /**
   * Checks if XML object exists
   *
   * @param value
   * @return True if value exists. False otherwise.
   */
   public boolean exists(XMLObject value){
     return (value!=null && value.fieldsExists());    
   }

   public boolean validateResponseType(List<EventObject> errorList,
                                           String transactionType,
                                           String fieldname,
                                           String value){
     return validateEnumeratedValue(errorList,
         transactionType,
         fieldname,
         "Response Type",
         ResponseType.RESPONSE_TYPE,
         value);
   }

   public boolean validateServiceOrderStatus(List<EventObject> errorList,
                                           String transactionType,
                                           String fieldname,
                                           String value){
     return validateEnumeratedValue(errorList,
         transactionType,
         fieldname,
         "Service Order Status",
         ServiceOrderStatus.SERVICE_ORDER_STATUS_TYPE,
         value);
   }

   public boolean validateResponseCode(List<EventObject> errorList,
                                           String transactionType,
                                           String fieldname,
                                           String value){
     return validateEnumeratedValue(errorList,
         transactionType,
         fieldname,
         "Response Code",
         ExceptionCode.EXCEPTION_CODES,
         value);
   }

  /**
   * Used to create an error message if fieldname1 and fieldname2 cant exist concurrently
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param fieldname2
   */
  public void addConcurrencyError(List<EventObject> errorList,
                                     String transactionType,
                                     String fieldname,
                                     String fieldname2){

    try
    {
		Object[] args = new Object[2];
		args[0] = ScreenName.getScreenName(transactionType, fieldname);
		args[1] = ScreenName.getScreenName(transactionType, fieldname2);
      addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "concurrentError");
    }
    catch (UnableToLookupResourceException e)
    {
      unexpectedException(errorList, fieldname, e.getMessage());
    }
  }

 /**
  * Used to define an error if an unexpected exception is thrown during validation
  * of any field.
  *
  * @param errorList to add event to.
  * @param context
  * @param explanation of the unexpected error
  */
  public void unexpectedException(List<EventObject> errorList,
                                  String context,
                                  String explanation){
    EventObject event = new EventObject(context,
        EventCode.CODE_0999,
        explanation);
    errorList.add(event);
  }

  /**
   * Creates and adds and event to the errorList
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param args used to format the explanation
   * @param eventcode
   * @param msgType used to look up an explanation
   * @throws UnableToLookupResourceException
   */
  public void addEvent(List<EventObject> errorList,
                          String transactionType,
                          String fieldname,
                          Object[] args,
                          EventCode eventcode,
                          String msgType) throws UnableToLookupResourceException{
    String context = "";
    MessageFormat msgFormat = new MessageFormat(EventMessage.getEventMessage(messagePrefix+msgType).getDescription());

    if (transactionType != null)
    {
      context = XPath.getXPath(transactionType, fieldname);
    }

    String screenName = ScreenName.getScreenName(transactionType, fieldname);
    EventObject event = new EventObject(context,
                            eventcode,
                            screenName + ": " + msgFormat.format(args));

    errorList.add(event);

  }

  public boolean validateEnumeratedValue(List<EventObject> errorList, String transactionType, String fieldname, String enumType, List<? extends Object> enumList, String value)
  {
    boolean result;
    Object[] args;

    // 26-Jul-2011 Eddie Graham Defect 7366 Raise error if enumerated list is empty
    if ((enumList == null) || enumList.isEmpty()) {
      unexpectedException(errorList, fieldname, enumType + ": Enumerated list is empty");
      return false;
    }

    // if value is blank return a mandatory field error.
    if (!validateMandatory(errorList,transactionType,fieldname,value)){
      return false;
    }
    result = Enumeration.contains(value, enumList);
    if (!result)
    {
      try
      {
        args = new Object[2];
        args[0] = enumType;
        args[1] = value;
        addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "invalidValue");
      }
      catch (UnableToLookupResourceException e)
      {
        unexpectedException(errorList, fieldname, e.getMessage());
      }
    }
    return result;
  }

  /**
   * Trim a string to remove all leading and trailing white spaces
   * @param value
   * @return a trimed string value
   */
  public static String trim(String value)
  {
    String trimValue;

    trimValue = null;
    if (value != null)
    {
      trimValue = value.trim();
    }
    return trimValue;
  }

  public boolean validateAlphaNumeric(List<EventObject> errorList,
                                           String transactionType,
                                           String fieldname,
                                           String value)
  {
    boolean result;
    String alphaNumericPattern = "[a-zA-Z0-9]+";

    result = value.matches(alphaNumericPattern);
    if (!result)
    {
      // define variable to add to explanation
      Object[] args = {fieldname, value};
      try
      {
        addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_0202,"notAlphaNumeric");
      }
      catch (UnableToLookupResourceException e)
      {
        unexpectedException(errorList, fieldname,e.getMessage());
      }
    }
    return result;
  }

  /**
   * Validate a date (which could either be an XML or B2B date)
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @param dateType
   * @return The result of the validation
   */
  public boolean validateDate(List<EventObject> errorList, String transactionType, String fieldname, String value, int dateType)
  {
    boolean result = false;

    if (dateType == B2BDateFormatter.B2BDATE)
    {
      result = validateB2BDate(errorList, transactionType, fieldname, value);
    }
    else if (dateType == B2BDateFormatter.XMLDATE)
    {
      result = validateXMLDate(errorList, transactionType, fieldname, value);
    }
    return result;
  }

  /**
   * Validate a datetime (which could either be an XML or B2B date)
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param value
   * @param dateType
   * @return The result of the validation
   */
  public boolean validateDateTime(List<EventObject> errorList, String transactionType, String fieldname, String value, int dateType)
  {
    boolean result = false;

    if (dateType == B2BDateTimeFormatter.B2BDATE)
    {
      result = validateB2BDateTime(errorList, transactionType, fieldname, value);
    }
    else if (dateType == B2BDateTimeFormatter.XMLDATE)
    {
      result = validateXMLDateTime(errorList, transactionType, fieldname, value);
    }
    return result;
  }

  /**
   * Given a date procedure will verify it is a valid Schedule Date
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param startDate Start date
   * @param startDateType
   * @return true if validation was fine false otherwise
   */
   public boolean validateScheduleDateRange(List<EventObject> errorList,
                                            String transactionType,
                                            String fieldname,
                                            String startDate,
                                            String transactionDate,
                                            int startDateType) {                                                 
     int numberOfDays = 100;
     // check the dates are valid
     if (!validateDate(errorList, transactionType, fieldname, startDate, startDateType)) {
        return false;
     }
     try {
       // get java.util.Date representation of startdate
       B2BDateFormatter startDateFormatter = new B2BDateFormatter(startDate, startDateType);  
       if (startDateType == B2BDateFormatter.XMLDATE) {
          transactionDate = transactionDate.substring(0,10);
       }
       B2BDateFormatter transactionDateFormatter = new B2BDateFormatter(transactionDate, startDateType);

       Calendar c = Calendar.getInstance();
       c.setTime(new Date());
       c.add(Calendar.DATE,numberOfDays);
       Date futureDate = c.getTime();

       Object[] args1 = {startDate, new Integer(numberOfDays)};
       Object[] args2 = {startDate};
       
       // validate date is less than a set number of days in the future
       return validateDateRange( errorList
                                ,transactionType
                                ,fieldname
                                ,startDateFormatter.getDate()
                                ,futureDate
                                ,EventCode.CODE_1954
                                ,args1
                                ,"invalidScheduleDate")
              &&
              // validate the date is not in the past
              validateDateRange( errorList
                                ,transactionType
                                ,fieldname
                                ,transactionDateFormatter.getDate()
                                ,startDateFormatter.getDate()
                                ,EventCode.CODE_0202
                                ,args2
                                ,"invalidDateInPast")
                                ;

     } catch (Exception e) {
       unexpectedException(errorList, fieldname, e.getMessage());
     }
     return false;
   }

 /**
  *
  * @param errorList
  * @param transactionType
  * @param fieldname
  * @param startDate Start date
  * @param startDateType Start date type
  * @param endDate end date
  * @param endDateType end date type
  * @return true if validation was fine false otherwise
  */
  public boolean validateDateTimeRange(List<EventObject> errorList,
                                       String transactionType,
                                       String fieldname,
                                       String startDate,
                                       int startDateType,
                                       String endDate,
                                       int endDateType){
    // check the dates are valid
    if (!validateDateTime(errorList, transactionType, fieldname, startDate, startDateType)) return false;
    if (!validateDateTime(errorList, transactionType, fieldname, endDate, endDateType)) return false;
    try {
      // get java.util.Date representation of start and end date
      B2BDateTimeFormatter startDateFormatter = new B2BDateTimeFormatter(startDate, startDateType);
      B2BDateTimeFormatter endDateFormatter = new B2BDateTimeFormatter(endDate, endDateType);
      // define variable to add to explanation
      Object[] args = {startDate, endDate};

      return validateDateRange( errorList
                               ,transactionType
                               ,fieldname
                               ,startDateFormatter.getDate()
                               ,endDateFormatter.getDate()
                               ,EventCode.CODE_0202
                               ,args
                               ,"invalidStartEndDateRange");
    } catch (Exception e) {
      unexpectedException(errorList, fieldname, e.getMessage());
    }
    return false;
  }

  /**
   *
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param startDate Start date
   * @param startDateType date type
   * @return true if validation was fine false otherwise
   */
   public boolean validateDateInFuture(List<EventObject> errorList,
                                    String transactionType,
                                    String fieldname,
                                    String startDate,
                                    int startDateType){
     // check the dates are valid
     if (!validateDate(errorList, transactionType, fieldname, startDate, startDateType)) return false;
     try {
       // get java.util.Date representation of start and end date
       B2BDateFormatter startDateFormatter = new B2BDateFormatter(startDate, startDateType);

       Calendar c = Calendar.getInstance();
       c.setTime(new Date());
       c.add(Calendar.DATE,-1);
       Date yesterday = c.getTime();

       // define variable to add to explanation
       Object[] args = {startDate};

       // validate the date is not in the past
       return  validateDateRange( errorList
                                 ,transactionType
                                 ,fieldname
                                 ,yesterday
                                 ,startDateFormatter.getDate()
                                 ,EventCode.CODE_0202
                                 ,args
                                 ,"invalidDateInPast")
                                 ;
     } catch (Exception e) {
       unexpectedException(errorList, fieldname, e.getMessage());
     }
     return false;
   }

  /**
   *
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param startDate Start date
   * @param startDateType Start date type
   * @param endDate end date
   * @param endDateType end date type
   * @return true if validation was fine false otherwise
   */
   public boolean validateDateRange(List<EventObject> errorList,
                                    String transactionType,
                                    String fieldname,
                                    String startDate,
                                    int startDateType,
                                    String endDate,
                                    int endDateType){
     // check the dates are valid
     if (!validateDate(errorList, transactionType, fieldname, startDate, startDateType)) {
        return false;
     }
     if (!validateDate(errorList, transactionType, fieldname, endDate, endDateType)) { 
        return false;
     }
     try {
       // get java.util.Date representation of start and end date
       B2BDateFormatter startDateFormatter = new B2BDateFormatter(startDate, startDateType);
       B2BDateFormatter endDateFormatter = new B2BDateFormatter(endDate, endDateType);
       // define variable to add to explanation
       Object[] args = {startDate, endDate};

       return validateDateRange( errorList
                                ,transactionType
                                ,fieldname
                                ,startDateFormatter.getDate()
                                ,endDateFormatter.getDate()
                                ,EventCode.CODE_0202
                                ,args
                                ,"invalidStartEndDateRange");
     } catch (Exception e) {
       unexpectedException(errorList, fieldname, e.getMessage());
     }
     return false;
   }

  /**
   *
   * @param errorList
   * @param transactionType
   * @param fieldname
   * @param startDate Start date
   * @param endDate enddate
   * @param eventCode event code to use
   * @param eventObjectList list of objects to be added to the event code if an event is produced
   * @param explanationCode event code to get from the properties file on an error
   * @return true if validation was fine false otherwise
   */
   public boolean validateDateRange(List<EventObject> errorList,
                                    String transactionType,
                                    String fieldname,
                                    Date startDate,
                                    Date endDate,
                                    EventCode eventCode,
                                    Object[] eventObjectList,
                                    String explanationCode) {
     try {       
       if (!startDate.after(endDate)) {
          return true;
       }
       addEvent(errorList,transactionType,fieldname,eventObjectList,eventCode,explanationCode);
     } catch (UnableToLookupResourceException e){
       unexpectedException(errorList, fieldname, e.getMessage());
     }

     return false;
   }

   /**
    *
    * @param errorList
    * @param transactionType
    * @param fieldname
    * @param value to validate
    * @return true if validation was fine false otherwise
    */
   public boolean validateOneWayNotificationName(List<EventObject> errorList,
                                                 String transactionType,
                                                 String fieldname,
                                                 String value) {
     return validateEnumeratedValue(errorList,
                                    transactionType,
                                    fieldname,
                                    "Name",
                                    OneWayNotificationName.ONE_WAY_NOTIFICATION_NAME_CODES,
                                    value);
   }
   
    /**
    * Checks for conditional mandatory validation. This test applies for elements which are mandatory
    * on the basis of other data elements being provided.
    * If the condition field equals matchValue then the field is mandatory.
    *
    * @param errorList list of events to add any generated events to
    * @param transactionType being validated
    * @param conditionalFieldname to check value of
    * @param conditionalFieldValue to match to matchValue
    * @param matchValue if the contional field matches this then fieeldname must be mandatory
    * @param fieldname being validated
	* @param value being validated
    * @return True if validation passes. False otherwise.
    */
   public boolean validateMandatoryConditionally1(List<EventObject> errorList,
                                                  String transactionType,
                                                  String fieldname,
												  String value,
                                                  String conditionalFieldname,
                                                  String conditionalFieldValue,
                                                  String matchValue) {
      boolean result = true;
      if (!exists(value) && conditionalFieldValue != null && conditionalFieldValue.equals(matchValue)) {
         result = false;
         try {
    	   String conditionalScreenName = ScreenName.getScreenName(transactionType, conditionalFieldname);
	       Object[] args = { conditionalScreenName, matchValue };
           addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0201, "mandatoryConditionally1");
         } catch (UnableToLookupResourceException e) {
            unexpectedException(errorList, fieldname, e.getMessage());
         }
      }
      return result;
   }
   
   /**
    * Checks the symantics of an email address
    * @param errorList
    * @param transactionType
    * @param fieldname
    * @param emailAddress
    * @return true if validation was fine false otherwise
    */
    public boolean validateEmailAddress(List<EventObject> errorList,
                                     String transactionType,
                                     String fieldname,
                                     String emailAddress){
      boolean isValid = true;
      try {
              if(Pattern.matches("[^@]+@[^\\.]+\\..+", emailAddress)){
                 isValid = true;
                 return isValid;
              }

               // define variable to add to explanation
               Object[] args = {emailAddress};
               addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_0202,"invalidEmailAddress");

      } catch (UnableToLookupResourceException e){
        unexpectedException(errorList, fieldname,e.getMessage());
      }

      return false;
    }

   /**
     * Validate duration in 24 hours (HH:MM) format with regular expression
      * @param errorList list of events to add any generated events to
      * @param transactionType being validated
      * @param fieldname in the transaction
      * @param value to validate duration
      *   @return true valid duration(time) format, false invalid format
     */
    public boolean validateDuration(List<EventObject> errorList,
                                      String transactionType,
                                      String fieldname,
                                      String value) {
       boolean isValid = false;
       try {
          Pattern pattern = Pattern.compile(TIME24HOURS_PATTERN);
          Matcher matcher = pattern.matcher(value);
          if(matcher.matches()){
             isValid = true;
             return isValid;
           }

          // define variable to add to explanation
          Object[] args = { fieldname };
          addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "invalidDuration");
       } catch (UnableToLookupResourceException e) {
          unexpectedException(errorList, fieldname, e.getMessage());
       }

       return isValid;
    }

    /**
     * Validate duration in the format 00:00 - 99:99 with regular expression.
      * @param errorList list of events to add any generated events to
      * @param transactionType being validated
      * @param fieldname in the transaction
      * @param value to validate duration
      *   @return true valid duration(time) format, false invalid format
     */
    public boolean validateDurationInterval(List<EventObject> errorList,
                                      String transactionType,
                                      String fieldname,
                                      String value) {
       boolean isValid = false;
       try {
          Pattern pattern = Pattern.compile(TIMEINTERVAL_PATTERN);
          Matcher matcher = pattern.matcher(value);
          if(matcher.matches()){
             isValid = true;
             return isValid;
           }

          // define variable to add to explanation
          Object[] args = { fieldname };
          addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "invalidDuration");
       } catch (UnableToLookupResourceException e) {
          unexpectedException(errorList, fieldname, e.getMessage());
       }

       return isValid;
    }

    /**
    * Checks for conditional mandatory validation. This test applies for elements which are mandatory
    * on the basis of other data elements being provided.
    * If the condition field does not equal matchValue then the field is mandatory.
    *
    * @param errorList list of events to add any generated events to
    * @param transactionType being validated
    * @param fieldname being validated
	* @param value being validated
    * @param conditionalFieldname to check value of
    * @param conditionalFieldValue to match to matchValue
    * @param matchValue if the contional field matches this then fieeldname must be mandatory
    * @return True if validation passes. False otherwise.
    */
   public boolean validateMandatoryConditionally2(List<EventObject> errorList,
                                                  String transactionType,
												  String fieldname,
												  String value,
                                                  String conditionalFieldname,
                                                  String conditionalFieldValue,
                                                  String matchValue
                                                  ) {
      boolean result = true;
      if(exists(value) && !matchValue.equals(conditionalFieldValue) && !exists(value) ){
         result = false;
         try {
  		   String conditionalScreenName = ScreenName.getScreenName(transactionType, conditionalFieldname);
           Object[] args = { conditionalScreenName, matchValue };
            addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0201, "mandatoryConditionally2");
         } catch (UnableToLookupResourceException e) {
            unexpectedException(errorList, fieldname, e.getMessage());
         }
      }
      return result;
   }

  /**
    * Validate allowed values
    *
    * @param errorList list of events to add any generated events to
    * @param transaction type for the validation
    * @param fieldname in the transaction
    * @param value to validate
    * @param allowedValue value allowed
    * @return True if validation passes. False otherwise.
    */
    public boolean validateAllowedValues(List<EventObject> errorList,
                                            String transactionType,
                                            String fieldname,
                                            List<? extends Object> allowedValues,
                                            String value) {
     try {
        if (allowedValues.contains(value)) {
         return true;
        }else{
           // define variable to add to explanation

            Object[] args = {allowedValues.toArray(), value};
            addEvent(errorList,transactionType,fieldname,args,EventCode.CODE_0202,"notAllowedValue");
        }


     } catch (UnableToLookupResourceException e){
       unexpectedException(errorList, fieldname, e.getMessage());
     }

     return false;

    }

   /**
       * If value1 must exist to have value2
       * @param errorList
       * @param transactionType
       * @param fieldname1
       * @param value1
       * @param fieldname2
       * @param value2
       * @return true if validation was fine false otherwise
       */
      public  boolean validateDependency2(List<EventObject> errorList,
                                       String transactionType,
                                       String fieldname1,
                                       String value1,
                                       String fieldname2,
                                       String value2){
         boolean result=true;
         if(exists(value2) &&  !exists(value1) ){

           result = false;
           Object[] args = {fieldname2, fieldname1};
           try
           {
             addEvent(errorList,transactionType,fieldname2,args,EventCode.CODE_0202,"conditional2");
           }
           catch (UnableToLookupResourceException e)
           {
             unexpectedException(errorList, fieldname1,e.getMessage());
           }
         }
         return result;
      }

   public boolean validateEnumeratedValue2(List<EventObject> errorList, String transactionType, String fieldname, String enumType, List<? extends Object> enumList, String value)
   {
     boolean result;
     Object[] args;


     if ((enumList == null) || enumList.isEmpty()) {
       unexpectedException(errorList, fieldname, enumType + ": Enumerated list is empty");
       return false;
     }

     result = Enumeration.contains(value, enumList);
     if (!result)
     {
       try
       {
         args = new Object[2];
         args[0] = enumType;
         args[1] = value;
         addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "invalidValue");
       }
       catch (UnableToLookupResourceException e)
       {
         unexpectedException(errorList, fieldname, e.getMessage());
       }
     }
     return result;
   }

   /**
    * Checks a value is numeric and check the lenght before and after decimal is correct
    *
    * @param errorList list of events to add any generated events to
    * @param transactionType being validated
    * @param fieldname in the transaction
    * @param value to validate
    * @return True if validation passes. False otherwise.
    */
    public boolean validateNumericDecimal2(List<EventObject> errorList,
                                          String transactionType,
                                          String fieldname,
                                          String value,
                                          String pattern){

       boolean result = true;

       if(Pattern.matches(pattern, value)){
          result = true;
       }else{
         result = false;
          Object[] args = {fieldname,value};

          try
          {
            addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "invalidValue");
          }
          catch (UnableToLookupResourceException e)
          {
            unexpectedException(errorList, fieldname, e.getMessage());
            return false;
          }

       }

      return result;
    }
	  /**
    * Checks the specialNotes
    * @param errorList list of events to add any generated events to
    * @param transactionType being validated
    * @param fieldname in the transaction
    * @param specialNotes to validate specialNotes
    * @return true if validation was fine false otherwise
    */
   public boolean validateSpecialNotes(List<EventObject> errorList,
                                       String transactionType,
                                       String fieldname,
                                       String specialNotes) {
      try {
         if (!isBlank(specialNotes)) return true;

         // define variable to add to explanation
         Object[] args = { fieldname };
         addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "invalidSpecialNotes");

      } catch (UnableToLookupResourceException e) {
         unexpectedException(errorList, fieldname, e.getMessage());
      }

      return false;
   }

   public boolean isBlank(String str) {
      int strLen;
      if (str == null || (strLen = str.trim().length()) == 0) {
         return true;
      }
      for (int i = 0; i < strLen; i++) {
         if ((Character.isWhitespace(str.charAt(i)) == false)) {
            return false;
         }
      }
      return true;
   }
    public boolean validateXmlString(List<EventObject> errorList, String transactionType, String fieldname, String value, int minlength, int maxlength) {
        boolean isValidSting = false;
        boolean isValidlength = false;
        try {
            if (value == null) value = "";
            isValidSting = isValidXmlString(value);
            isValidlength = (value.length() >= minlength && value.length() <= maxlength);
            if (isValidSting && isValidlength) {
                return true;
            }
            if (!isValidlength) {
                String description = minlength + "";
                if (minlength != maxlength) {
                    description = minlength + " to " + maxlength;
                }
                Object[] args = {description, new Long(value.length())};
                addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "invalidLength");
            }
            if (!isValidSting) {
                Object[] args = {fieldname, value};
                addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "invalidXmlFormat");
            }
        } catch (UnableToLookupResourceException e) {
            unexpectedException(errorList, fieldname, e.getMessage());
        }
        return false;
    }
    public boolean isValidXmlString(String xmlString) {
        for (int i = 0; i < xmlString.length(); i++) {
            char c = xmlString.charAt(i);
            switch (c) {
                case '<':
                    return false;
                case '>':
                    return false;
                case '\"':
                    return false;
                case '&':
                    return false;
                case '\'':
                    return false;
                case '`':
                    return false;
                default:
            }
        }
        return true;
    }
    public  boolean validateDependency(List<EventObject> errorList,
                                        String transactionType,
                                        String fieldname1,
                                        String value1,
                                        String fieldname2,
                                        String value2){
        boolean result=true;
        if( (exists(value1) &&  !exists(value2) ) || (!exists(value1) &&  exists(value2)) ){
            result = false;
            Object[] args = {exists(value1) ? fieldname1 : fieldname2, !exists(value2) ? fieldname2 : fieldname1};
            try
            {
                addEvent(errorList,transactionType,fieldname2,args,EventCode.CODE_0202,"conditional2");
            }
            catch (UnableToLookupResourceException e)
            {
                unexpectedException(errorList, fieldname1,e.getMessage());
            }
        }
        return result;
    }
    public boolean validateAttachmentOrder(List<EventObject> errorList, String transactionType, String fieldname, String value) {
        try {
            Object[] args = {fieldname, value};
            addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "invalidAttachment");
        } catch (UnableToLookupResourceException e) {
            unexpectedException(errorList, fieldname, e.getMessage());
        }
        return false;
    }

   /**
    * Validate messageId and transcationId in the format consisting only alphanumeric characters and the hyphen character with regular expression.
     * @param errorList list of events to add any generated events to
     * @param transactionType being validated
     * @param fieldname in the transaction
     * @param value to validate messageId
     *   @return true valid messageId format, false invalid format
    */
   public boolean validateId(List<EventObject> errorList,
                                     String transactionType,
                                     String fieldname,
                                     String value) {
      boolean isValid = false;
      try {
         Pattern pattern = Pattern.compile(MESSAGE_AND_TRANS_ID_PATTERN);
         Matcher matcher = pattern.matcher(value);
         if(!matcher.find()){
            isValid = true;
            return isValid;
          }

         // define variable to add to explanation
         Object[] args = { fieldname };
         addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "invalidId");
      } catch (UnableToLookupResourceException e) {
         unexpectedException(errorList, fieldname, e.getMessage());
      }

      return isValid;
   }
    public boolean validateEmptyField(List<EventObject> errorList, String transactionType, String fieldname, String fieldValue) {
        try {
            if (!isBlank(fieldValue)) return true;
            Object[] args = {fieldname};
            addEvent(errorList, transactionType, fieldname, args, EventCode.CODE_0202, "invalidSpecialNotes");
        } catch (UnableToLookupResourceException e) {
            unexpectedException(errorList, fieldname, e.getMessage());
        }
        return false;
    }
}
