/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/Address.java-arc   1.18   Sep 27 2005 15:45:56   adij  $
 * $Revision:   1.18  $
 * $Date:   Sep 27 2005 15:45:56  $
 *
 * Copyright 2005 Capgemini Australia Pty. Ltd.
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * Defect 3657 : 22-12-05 : NG : validateLocality() to allow for punctuation characters
 *
 * Modification History
 *
 * Date Changed   Changed By        Description
 * ------------   ----------        -----------
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb.validators;

import com.capgemini.nemmco.b2bb.enums.EventCode;
import java.util.ArrayList;
import java.util.List;

import com.capgemini.nemmco.b2bb.XMLObject;
import com.capgemini.nemmco.b2bb.enums.FlatOrUnitType;
import com.capgemini.nemmco.b2bb.enums.FloorOrLevelType;
import com.capgemini.nemmco.b2bb.enums.PostalDeliveryType;
import com.capgemini.nemmco.b2bb.enums.State;
import com.capgemini.nemmco.b2bb.enums.StreetSuffix;
import com.capgemini.nemmco.b2bb.enums.StreetType;
import com.capgemini.nemmco.b2bb.enums.XMLObjectType;
import com.capgemini.nemmco.b2bb.exceptions.UnableToLookupResourceException;
import com.capgemini.nemmco.b2bb.utils.BasicValidationUtility;
import com.capgemini.nemmco.b2bb.utils.EventObject;

/**
 * Define the address object for transaction
 *
 * @author $Author:   adij  $
 * @version $Revision:   1.18  $ $Date:   Sep 27 2005 15:45:56  $
 */
public class Address extends XMLObject implements ValidatorInterface {

  public static String ADDRESS_CODE = "Address";
  public static String BUILDING_OR_PROPERTY_NAME1 = "buildingOrPropertyName1";
  public static String BUILDING_OR_PROPERTY_NAME2 = "buildingOrPropertyName2";
  public static String FLAT_OR_UNIT_NUM = "flatOrUnitNum";
  public static String FLAT_OR_UNIT_TYPE = "flatOrUnitType";
  public static String FLOOR_OR_LEVEL_NUM = "floorOrLevelNum";
  public static String FLOOR_OR_LEVEL_TYPE = "floorOrLevelType";
  public static String HOUSE_NUM_SUFFIX1 = "houseNumSuffix1";
  public static String HOUSE_NUM_SUFFIX2 = "houseNumSuffix2";
  public static String HOUSE_NUM1 = "houseNum1";
  public static String HOUSE_NUM2 = "houseNum2";
  public static String LOCALITY = "locality";
  public static String LOCATION_DESCRIPTOR = "locationDescriptor";
  public static String LOT_NUM = "lotNum";
  public static String POSTAL_DELIVERY_NUM_PREFIX = "postalDeliveryNumPrefix";
  public static String POSTAL_DELIVERY_NUM_SUFFIX = "postalDeliveryNumSuffix";
  public static String POSTAL_DELIVERY_NUM_VALUE = "postalDeliveryNumValue";
  public static String POSTAL_DELIVERY_TYPE = "postalDeliveryType";
  public static String POSTCODE = "postcode";
  public static String SITE_DPID = "siteDPID";
  public static String STATE = "state";
  public static String STREET_NAME1 = "streetName1";
  public static String STREET_NAME2 = "streetName2";
  public static String STREET_SUFFIX1 = "streetSuffix1";
  public static String STREET_SUFFIX2 = "streetSuffix2";
  public static String STREET_TYPE1 = "streetType1";
  public static String STREET_TYPE2 = "streetType2";
  public static String UNSTRUCTURED_ADDRESS1 = "unstructuredAddress1";
  public static String UNSTRUCTURED_ADDRESS2 = "unstructuredAddress2";
  public static String UNSTRUCTURED_ADDRESS3 = "unstructuredAddress3";
  
  private String buildingOrPropertyName1 = "";
  private String buildingOrPropertyName2 = "";
  private List<EventObject> events = new ArrayList<EventObject>();
  private String flatOrUnitNum = "";
  private String flatOrUnitType = "";
  private String floorOrLevelNum = "";
  private String floorOrLevelType = "";
  private String houseNum1 = "";
  private String houseNum2 = "";
  private String houseNumSuffix1 = "";
  private String houseNumSuffix2 = "";
  private String locality = "";
  private String locationDescriptor = "";
  private String lotNum = "";
  private String parentCategory = "";
  private String parentField = "";
  private String postalDeliveryNumPrefix = "";
  private String postalDeliveryNumSuffix = "";
  private String postalDeliveryNumValue = "";
  private String postalDeliveryType = "";
  private String postcode = "";
  private String siteDPID = "";
  private String state = "";
  private String streetName1 = "";
  private String streetName2 = "";
  private String streetSuffix1 = "";
  private String streetSuffix2 = "";
  private String streetType1 = "";
  private String streetType2 = "";
  private String unstructuredAddress1 = "";
  private String unstructuredAddress2 = "";
  private String unstructuredAddress3 = "";
  private BasicValidationUtility validator = null;

  /**
   * Default constructor
   */
  public Address() {
    super(XMLObjectType.ADDRESS);
    validator = BasicValidationUtility.getBasicInstance();
  }

  /**
   * Constructor
   * @param xmlObjectType The xmlObjectType to set
   */
  public Address(XMLObjectType xmlObjectType) {
    super(xmlObjectType);
    validator = BasicValidationUtility.getBasicInstance();
  }

  /**
   * 
   * @return Return true if at least one field exists
   */
  public boolean fieldsExists() {
    return structuredAddressExists() 
    		|| unstructuredAddressExists() 
    		|| locPostcodeStateExists();
  }

  /**
   * 
   * @return Return the buildingOrPropertyName1
   */
  public String getBuildingOrPropertyName1() {
    return buildingOrPropertyName1;
  }

  /**
   * 
   * @return Return the buildingOrPropertyName2
   */
  public String getBuildingOrPropertyName2() {
    return buildingOrPropertyName2;
  }

  /**
   * @see com.capgemini.nemmco.b2bb.validators.ValidatorInterface#getEvents()
   */
  public List<EventObject> getEvents() {
    return events;
  }

  /**
   * 
   * @return Return the flatOrUnitNumber
   */
  public String getFlatOrUnitNum() {
    return flatOrUnitNum;
  }

  /**
   * 
   * @return Return the flatOrUnitType
   */
  public String getFlatOrUnitType() {
    return flatOrUnitType;
  }
  
  /**
   * 
   * @return Return the List of flatOrUnitType
   */
  public List<? extends Object> getFlatOrUnitTypes() {
    return FlatOrUnitType.FLAT_OR_UNIT_TYPES;
  }

  /**
   * 
   * @return Return the floorOrLevelNumber
   */
  public String getFloorOrLevelNum() {
    return floorOrLevelNum;
  }

  /**
   * 
   * @return Return the floorOrLevelType
   */
  public String getFloorOrLevelType() {
    return floorOrLevelType;
  }

  /**
   * 
   * @return Return the List of floorOrLevelType
   */
  public List<? extends Object> getFloorOrLevelTypes() {
    return FloorOrLevelType.FLOOR_OR_LEVEL_TYPES;
  }

  /**
   * 
   * @return Return the houseNumber1
   */
  public String getHouseNum1() {
    return houseNum1;
  }

  /**
   * 
   * @return Return the houseNumber2
   */
  public String getHouseNum2() {
    return houseNum2;
  }

  /**
   * 
   * @return Return the houseNumberSuffix1
   */
  public String getHouseNumSuffix1() {
    return houseNumSuffix1;
  }

  /**
   * 
   * @return Return the houseNumberSuffix2
   */
  public String getHouseNumSuffix2() {
    return houseNumSuffix2;
  }

  /**
   * 
   * @return Return the locatity
   */
  public String getLocality() {
    return locality;
  }

  /**
   * 
   * @return Return the locationDescriptor
   */
  public String getLocationDescriptor() {
    return locationDescriptor;
  }

  /**
   * 
   * @return Return the lotNumber
   */
  public String getLotNum() {
    return lotNum;
  }

  /**
   * @return Returns the parentCategory.
   */
  public String getParentCategory() {
    return parentCategory;
  }

  /**
   * @return Returns the parentField.
   */
  public String getParentField() {
    return parentField;
  }

  /**
   * 
   * @return Return the postalDeliveryNumberPrefix
   */
  public String getPostalDeliveryNumPrefix() {
    return postalDeliveryNumPrefix;
  }
  
  /**
   * 
   * @return Return the postalDeliveryNumberSuffix
   */
  public String getPostalDeliveryNumSuffix() {
    return postalDeliveryNumSuffix;
  }

  /**
   * 
   * @return Return the postalDeliveryNumberValue
   */
  public String getPostalDeliveryNumValue() {
    return postalDeliveryNumValue;
  }

  /**
   * 
   * @return Return the postalDeliveryType
   */
  public String getPostalDeliveryType() {
    return postalDeliveryType;
  }

  /**
   * 
   * @return Return the postcode
   */
  public String getPostcode() {
    return postcode;
  }

  /**
   * 
   * @return Return the siteDPID
   */
  public String getSiteDPID() {
    return siteDPID;
  }
  
  /**
   * 
   * @return Return the state
   */
  public String getState() {
    return state;
  }
  
  /**
   * 
   * @return Return the List of states
   */
  public List<? extends Object> getStates() {
    return State.STATES;
  }

  /**
   * 
   * @return Return the streetName1
   */
  public String getStreetName1() {
    return streetName1;
  }

  /**
   * 
   * @return Return the streetName2
   */
  public String getStreetName2() {
    return streetName2;
  }

  /**
   * 
   * @return Return the streetSuffix1
   */
  public String getStreetSuffix1() {
    return streetSuffix1;
  }

  /**
   * 
   * @return Return the streetSuffix2
   */
  public String getStreetSuffix2() {
    return streetSuffix2;
  }

  /**
   * 
   * @return Return the List of streetSuffixs
   */
  public List<? extends Object> getStreetSuffixs() {
    return StreetSuffix.STREET_SUFFIXES;
  }

  /**
   * 
   * @return Return the list of postalDeliveryTypes
   */
  public List<? extends Object> getPostalDeliveryTypes() {
    return PostalDeliveryType.POSTAL_DELIVERY_TYPE_CODES;
  }
  
  /**
   * 
   * @return Return the streetType1
   */
  public String getStreetType1() {
    return streetType1;
  }
  
  /**
   * 
   * @return Return the streetType2
   */
  public String getStreetType2() {
    return streetType2;
  }
  
  /**
   * 
   * @return Return list of streetTypes
   */
  public List<? extends Object> getStreetTypes() {
    return StreetType.STREET_TYPES;
  }

  /**
   * 
   * @return Return unstructuredAddress1
   */
  public String getUnstructuredAddress1() {
    return unstructuredAddress1;
  }

  /**
   * 
   * @return Return unstructuredAddress2
   */
  public String getUnstructuredAddress2() {
    return unstructuredAddress2;
  }

  /**
   * 
   * @return Return unstructuredAddress3
   */
  public String getUnstructuredAddress3() {
    return unstructuredAddress3;
  }

  /**
   * 
   * @return Return true if at least one of the locality, postcode or state fields are populated
   */
  public boolean locPostcodeStateExists() {
    return validator.exists(locality) || validator.exists(postcode) || validator.exists(state);
  }

  /**
   * 
   * @param buildingOrPropertyName1 The buildingOrPropertyName1 to set
   */
  public void setBuildingOrPropertyName1(String buildingOrPropertyName1) {
    this.buildingOrPropertyName1 = BasicValidationUtility.trim(buildingOrPropertyName1);
  }
  
  /**
   * 
   * @param buildingOrPropertyName2 The buildingOrPropertyName2 to set
   */
  public void setBuildingOrPropertyName2(String buildingOrPropertyName2) {
    this.buildingOrPropertyName2 = BasicValidationUtility.trim(buildingOrPropertyName2);
  }

  /**
   * @param event The list of event to set
   */
  public void setEvents(List<EventObject> event) {
    events = event;
  }

  /**
   * 
   * @param flatOrUnitNum The flatOrUnitNumber to set
   */
  public void setFlatOrUnitNum(String flatOrUnitNum) {
    this.flatOrUnitNum = BasicValidationUtility.trim(flatOrUnitNum);
  }

  /**
   * 
   * @param flatOrUnitType The flatOrUnitType to set
   */
  public void setFlatOrUnitType(String flatOrUnitType) {
    this.flatOrUnitType = BasicValidationUtility.trim(flatOrUnitType);
  }

  /**
   * 
   * @param floorOrLevelNum The floorOrLevelNumber to set
   */
  public void setFloorOrLevelNum(String floorOrLevelNum) {
    this.floorOrLevelNum = BasicValidationUtility.trim(floorOrLevelNum);
  }

  /**
   * 
   * @param floorOrLevelType The floorOrLevelType to set
   */
  public void setFloorOrLevelType(String floorOrLevelType) {
    this.floorOrLevelType = BasicValidationUtility.trim(floorOrLevelType);
  }

  /**
   * 
   * @param houseNum1 The houseNumber1 to set
   */
  public void setHouseNum1(String houseNum1) {
    this.houseNum1 = BasicValidationUtility.trim(houseNum1);
  }

  /**
   * 
   * @param houseNum2 The houseNumber2 to set
   */
  public void setHouseNum2(String houseNum2) {

    this.houseNum2 = BasicValidationUtility.trim(houseNum2);
  }

  /**
   * 
   * @param houseNumSuffix1 The houseNumberSuffix1 to set
   */
  public void setHouseNumSuffix1(String houseNumSuffix1) {
    this.houseNumSuffix1 = BasicValidationUtility.trim(houseNumSuffix1);
  }

  /**
   * 
   * @param houseNumSuffix2 The houseNumberSuffix2 to set
   */
  public void setHouseNumSuffix2(String houseNumSuffix2) {
    this.houseNumSuffix2 = BasicValidationUtility.trim(houseNumSuffix2);
  }
  
  /**
   * 
   * @param locality The locality to set
   */
  public void setLocality(String locality) {
    this.locality = BasicValidationUtility.trim(locality);
  }

  /**
   * 
   * @param locationDescriptor The locationDescriptor to set
   */
  public void setLocationDescriptor(String locationDescriptor) {
    this.locationDescriptor = BasicValidationUtility.trim(locationDescriptor);
  }

  /**
   * 
   * @param lotNum The lotNumber to set
   */
  public void setLotNum(String lotNum) {
    this.lotNum = BasicValidationUtility.trim(lotNum);
  }

  /**
   * @param parentCategory The parentCategory to set.
   */
  public void setParentCategory(String parentCategory) {
    this.parentCategory = parentCategory;
  }

  /**
   * @param parentField The parentField to set.
   */
  public void setParentField(String parentField) {
    this.parentField = parentField;
  }

  /**
   * 
   * @param postalDeliveryNumPrefix The postalDeliveryNumberPrefix to set
   */
  public void setPostalDeliveryNumPrefix(String postalDeliveryNumPrefix) {
    this.postalDeliveryNumPrefix = BasicValidationUtility.trim(postalDeliveryNumPrefix);
  }

  /**
   * 
   * @param postalDeliveryNumSuffix The postalDeliveryNumberSuffix to set
   */
  public void setPostalDeliveryNumSuffix(String postalDeliveryNumSuffix) {
    this.postalDeliveryNumSuffix = BasicValidationUtility.trim(postalDeliveryNumSuffix);
  }

  /**
   * 
   * @param postalDeliveryNumValue The postalDeliveryNumberValue to set
   */
  public void setPostalDeliveryNumValue(String postalDeliveryNumValue) {
    this.postalDeliveryNumValue = BasicValidationUtility.trim(postalDeliveryNumValue);
  }

  /**
   * 
   * @param postalDeliveryType The postalDeliveryType to set
   */
  public void setPostalDeliveryType(String postalDeliveryType) {
    this.postalDeliveryType = BasicValidationUtility.trim(postalDeliveryType);
  }

  /**
   * 
   * @param postcode The postcode to set
   */
  public void setPostcode(String postcode) {
    this.postcode = BasicValidationUtility.trim(postcode);
  }

  /**
   * 
   * @param siteDPID The siteDPID to set
   */
  public void setSiteDPID(String siteDPID) {
    this.siteDPID = BasicValidationUtility.trim(siteDPID);
  }
  
  /**
   * 
   * @param state The state to set
   */
  public void setState(String state) {
    this.state = BasicValidationUtility.trim(state);
  }

  /**
   * 
   * @param streetName1 The streetName1 to set
   */
  public void setStreetName1(String streetName1) {
    this.streetName1 = BasicValidationUtility.trim(streetName1);
  }

  /**
   * 
   * @param streetName2 The streetName2 to set
   */
  public void setStreetName2(String streetName2) {
    this.streetName2 = BasicValidationUtility.trim(streetName2);
  }

  /**
   * 
   * @param streetSuffix1 The streetSuffix1 to set
   */
  public void setStreetSuffix1(String streetSuffix1) {
    this.streetSuffix1 = BasicValidationUtility.trim(streetSuffix1);
  }

  /**
   * 
   * @param streetSuffix2 The streetSuffix2 to set
   */
  public void setStreetSuffix2(String streetSuffix2) {
    this.streetSuffix2 = BasicValidationUtility.trim(streetSuffix2);
  }

  /**
   * 
   * @param streetType1 The streetType1 to set
   */
  public void setStreetType1(String streetType1) {
    this.streetType1 = BasicValidationUtility.trim(streetType1);
  }

  /**
   * 
   * @param streetType2 The streetType2 to set
   */
  public void setStreetType2(String streetType2) {
    this.streetType2 = BasicValidationUtility.trim(streetType2);
  }

  /**
   * 
   * @param unstructuredAddress1 The unstructuredAddress1 to set
   */
  public void setUnstructuredAddress1(String unstructuredAddress1) {
    this.unstructuredAddress1 = BasicValidationUtility.trim(unstructuredAddress1);
  }

  /**
   * 
   * @param unstructuredAddress2 The unstructuredAddress2 to set
   */
  public void setUnstructuredAddress2(String unstructuredAddress2) {
    this.unstructuredAddress2 = BasicValidationUtility.trim(unstructuredAddress2);
  }

  /**
   * 
   * @param unstructuredAddress3 The unstructuredAddress3 to set
   */
  public void setUnstructuredAddress3(String unstructuredAddress3) {
    this.unstructuredAddress3 = BasicValidationUtility.trim(unstructuredAddress3);
  }

  /**
   * 
   * @return Return true if at least one structuredAddress field exists, false otherwise
   */
  public boolean structuredAddressExists() {
    return validator.exists(flatOrUnitType) || validator.exists(flatOrUnitNum) || validator.exists(floorOrLevelType) || validator.exists(floorOrLevelNum)
        || validator.exists(buildingOrPropertyName1) || validator.exists(buildingOrPropertyName2) || validator.exists(locationDescriptor)
        || validator.exists(houseNum1) || validator.exists(houseNumSuffix1) || validator.exists(houseNum2) || validator.exists(houseNumSuffix2)
        || validator.exists(lotNum) || validator.exists(streetName1) || validator.exists(streetType1) || validator.exists(streetSuffix1)
        || validator.exists(streetName2) || validator.exists(streetType2) || validator.exists(streetSuffix2) || validator.exists(streetSuffix2)
        || validator.exists(postalDeliveryType) || validator.exists(postalDeliveryNumPrefix) || validator.exists(postalDeliveryNumValue)
        || validator.exists(postalDeliveryNumSuffix);
  }
  
  /**
   * 
   * @return Return true if at least one unstructuredAddress field exists, false otherwise
   */
  public boolean unstructuredAddressExists() {
    return validator.exists(unstructuredAddress1) || validator.exists(unstructuredAddress2) || validator.exists(unstructuredAddress3);
  }

  /**
   * 
   * @return Return true if valid, false otherwise
   */
  public boolean validate() {
    boolean result = true;
    try {
      result = validateFlatOrUnitType() && result;
      result = validateFlatOrUnitNum() && result;
      result = validateFloorOrLevelType() && result;
      result = validateFloorOrLevelNum() && result;
      result = validateBuildingOrPropertyName1() && result;
      result = validateBuildingOrPropertyName2() && result;
      result = validateLocationDescriptor() && result;
      result = validateHouseNum1() && result;
      result = validateHouseNumSuffix1() && result;
      result = validateHouseNum2() && result;
      result = validateHouseNumSuffix2() && result;
      result = validateLotNum() && result;
      result = validateStreetName1() && result;
      result = validateStreetType1() && result;
      result = validateStreetSuffix1() && result;
      result = validateStreetName2() && result;
      result = validateStreetType2() && result;
      result = validateStreetSuffix2() && result;
      result = validatePostalDeliveryType() && result;
      result = validatePostalDeliveryNumPrefix() && result;
      result = validatePostalDeliveryNumValue() && result;
      result = validatePostalDeliveryNumSuffix() && result;
      result = validateUnstructuredAddress1() && result;
      result = validateUnstructuredAddress2() && result;
      result = validateUnstructuredAddress3() && result;
      result = validateStructuredUnstructured() && result;
      result = validateLocality() && result;
      result = validateState() && result;
      result = validatePostcode() && result;
      result = validateSiteDPID() && result;
    } catch (UnableToLookupResourceException e) {
      result = false;
      validator.unexpectedException(events, ADDRESS_CODE, e.getMessage());
    }
    return result;
  }

  /**
   * 
   * @return Return true if buildingOrPropertyName1 is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateBuildingOrPropertyName1() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + BUILDING_OR_PROPERTY_NAME1;
    boolean tempresult = true;
    if (getBuildingOrPropertyName1() != null && !getBuildingOrPropertyName1().equals("")) {
      return validator.validatePattern(events, parentCategory, fieldname, getBuildingOrPropertyName1(), "[\\p{L}\\p{N}\\p{Punct}\\s]{1,30}");  //defect 3551 (Justina Tran): added p{Punct} to allow punctuation.
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if buildingOrPropertyName2 is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateBuildingOrPropertyName2() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + BUILDING_OR_PROPERTY_NAME2;
    boolean tempresult = true;
    if (getBuildingOrPropertyName2() != null && !getBuildingOrPropertyName2().equals("")) {
      return validator.validatePattern(events, parentCategory, fieldname, getBuildingOrPropertyName2(), "[\\p{L}\\p{N}\\p{Punct}\\s]{1,30}");  //defect 3551 (Justina Tran): added p{Punct} to allow punctuation.
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if flatOrUnitNumber is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateFlatOrUnitNum() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + FLAT_OR_UNIT_NUM;
    boolean tempresult = true;
    if (getFlatOrUnitType() != null && !getFlatOrUnitType().equals("")) {
      tempresult = validator.validateMandatory(events, parentCategory, fieldname, getFlatOrUnitNum());
      return tempresult && validator.validatePattern(events, parentCategory, fieldname, getFlatOrUnitNum(), "[\\p{L}\\p{N}\\p{P}\\s]{1,7}");
    } else if (getFlatOrUnitNum() != null && !getFlatOrUnitNum().equals("")) {
      return validator.validatePattern(events, parentCategory, fieldname, getFlatOrUnitNum(), "[\\p{L}\\p{N}\\p{P}\\s]{1,7}");
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if flatOrUnitType is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateFlatOrUnitType() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + FLAT_OR_UNIT_TYPE;
    boolean tempresult = true;
    if (getFlatOrUnitNum() != null && !getFlatOrUnitNum().equals("")) {
      tempresult = validator.validateMandatory(events, parentCategory, fieldname, getFlatOrUnitType());
      return tempresult && validator.validateFlatOrUnitType(events, parentCategory, fieldname, getFlatOrUnitType());
    } else if (getFlatOrUnitType() != null && !getFlatOrUnitType().equals("")) {
      return validator.validateFlatOrUnitType(events, parentCategory, fieldname, getFlatOrUnitType());
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if floorOrLevelNumber is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateFloorOrLevelNum() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + FLOOR_OR_LEVEL_NUM;
    boolean tempresult = true;
    if (getFloorOrLevelNum() != null && !getFloorOrLevelNum().equals("")) {
      return validator.validatePattern(events, parentCategory, fieldname, getFloorOrLevelNum(), "[\\p{L}\\p{N}\\p{P}\\s]{1,5}");
    }
    return tempresult;
  }

  /**
   * 
   * @return Return true if floorOrLevelType is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateFloorOrLevelType() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + FLOOR_OR_LEVEL_TYPE;

    boolean tempresult = true;
    if (getFloorOrLevelNum() != null && !getFloorOrLevelNum().equals("")) {
      tempresult = validator.validateMandatory(events, parentCategory, fieldname, getFloorOrLevelType());
      return tempresult && validator.validateFloorOrLevelType(events, parentCategory, fieldname, getFloorOrLevelType());
    } else if (getFloorOrLevelType() != null && !getFloorOrLevelType().equals("")) {
      return validator.validateFloorOrLevelType(events, parentCategory, fieldname, getFloorOrLevelType());
    }
    return tempresult;
  }

  /**
   * 
   * @return Return true if houseNumber1 is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateHouseNum1() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + HOUSE_NUM1;
    boolean tempresult = true;
    if (getHouseNumSuffix1() != null && !getHouseNumSuffix1().equals("")) {
      tempresult = validator.validateMandatory(events, parentCategory, fieldname, getHouseNum1());
      tempresult = tempresult && validator.validateNumeric(events, parentCategory, fieldname, getHouseNum1());
      return tempresult && validator.validateLengthRange(events, parentCategory, fieldname, getHouseNum1(), 0, 5);
    } else if (getHouseNum1() != null && !getHouseNum1().equals("")) {
      tempresult = validator.validateNumeric(events, parentCategory, fieldname, getHouseNum1());
      return tempresult && validator.validateLengthRange(events, parentCategory, fieldname, getHouseNum1(), 0, 5);
    }
    return tempresult;
  }

  /**
   * 
   * @return Return true if houseNumber2 is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateHouseNum2() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + HOUSE_NUM2;
    boolean tempresult = true;
    if (getHouseNumSuffix2() != null && !getHouseNumSuffix2().equals("")) {
      tempresult = validator.validateMandatory(events, parentCategory, fieldname, getHouseNum2());
      tempresult = tempresult && validator.validateNumeric(events, parentCategory, fieldname, getHouseNum2());
      return tempresult && validator.validateLengthRange(events, parentCategory, fieldname, getHouseNum2(), 0, 5);
    } else if (getHouseNum2() != null && !getHouseNum2().equals("")) {
      tempresult = validator.validateNumeric(events, parentCategory, fieldname, getHouseNum2());
      return tempresult && validator.validateLengthRange(events, parentCategory, fieldname, getHouseNum2(), 0, 5);
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if houseNumberSuffix1 is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateHouseNumSuffix1() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + HOUSE_NUM_SUFFIX1;
    boolean tempresult = true;
    if (getHouseNumSuffix1() != null && !getHouseNumSuffix1().equals("")) {
      return validator.validatePattern(events, parentCategory, fieldname, getHouseNumSuffix1(), "[\\p{L}\\p{N}]{1}");
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if houseNumberSuffix2 is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateHouseNumSuffix2() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + HOUSE_NUM_SUFFIX2;
    boolean tempresult = true;
    if (getHouseNumSuffix2() != null && !getHouseNumSuffix2().equals("")) {
      return validator.validatePattern(events, parentCategory, fieldname, getHouseNumSuffix2(), "[\\p{L}\\p{N}]{1}");
    }
    return tempresult;
  }

  /**
   * 
   * @return Return true if locality is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateLocality() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + LOCALITY;
    boolean tempresult = true;

    tempresult = validator.validateMandatory(events, parentCategory, fieldname, getLocality());
    return tempresult && validator.validatePattern(events, parentCategory, fieldname, getLocality(), "[\\p{L}\\p{N}\\p{P}\\s]{1,46}");

  }

  /**
   * 
   * @return Return true if locationDescriptor is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateLocationDescriptor() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + LOCATION_DESCRIPTOR;
    boolean tempresult = true;
    if (getLocationDescriptor() != null && !getLocationDescriptor().equals("")) {
      return validator.validatePattern(events, parentCategory, fieldname, getLocationDescriptor(), "[\\p{L}\\p{N}\\p{P}\\s]{1,30}");
    }
    return tempresult;
  }

  /**
   * 
   * @return Return true if lotNumber is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateLotNum() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + LOT_NUM;
    boolean tempresult = true;
    if (getLotNum() != null && !getLotNum().equals("")) {
      return validator.validatePattern(events, parentCategory, fieldname, getLotNum(), "[\\p{L}\\p{N}\\p{P}\\s]{1,6}");
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if postalDeliveryNumberPrefix is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validatePostalDeliveryNumPrefix() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + POSTAL_DELIVERY_NUM_PREFIX;
    boolean tempresult = true;

    if (getPostalDeliveryNumPrefix() != null && !getPostalDeliveryNumPrefix().equals("")) {
      return validator.validatePattern(events, parentCategory, fieldname, getPostalDeliveryNumPrefix(), "[\\p{Lu}]{1,3}");
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if postalDeliveryNumberSuffix is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validatePostalDeliveryNumSuffix() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + POSTAL_DELIVERY_NUM_SUFFIX;
    boolean tempresult = true;

    if (getPostalDeliveryNumSuffix() != null && !getPostalDeliveryNumSuffix().equals("")) {
      return validator.validatePattern(events, parentCategory, fieldname, getPostalDeliveryNumSuffix(), "[\\p{Lu}]{1,3}");
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if postalDeliveryNumberValue is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validatePostalDeliveryNumValue() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + POSTAL_DELIVERY_NUM_VALUE;
    boolean tempresult = true;

    if (getPostalDeliveryNumValue() != null && !getPostalDeliveryNumValue().equals("")) {
      tempresult = validator.validateNumeric(events, parentCategory, fieldname, getPostalDeliveryNumValue());
      return tempresult && validator.validateLengthRange(events, parentCategory, fieldname, getPostalDeliveryNumValue(), 0, 5);
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if postalDeliveryType is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validatePostalDeliveryType() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + POSTAL_DELIVERY_TYPE;
    boolean tempresult = true;
    if ((getPostalDeliveryNumValue() != null && !getPostalDeliveryNumValue().equals(""))
        || (getPostalDeliveryNumPrefix() != null && !getPostalDeliveryNumPrefix().equals(""))
        || (getPostalDeliveryNumSuffix() != null && !getPostalDeliveryNumSuffix().equals(""))) {
      tempresult = validator.validateMandatory(events, parentCategory, fieldname, getPostalDeliveryType());
      return tempresult && validator.validatePostalDeliveryType(events, parentCategory, fieldname, getPostalDeliveryType());
    } else if (getPostalDeliveryType() != null && !getPostalDeliveryType().equals("")) {
      return validator.validatePostalDeliveryType(events, parentCategory, fieldname, getPostalDeliveryType());
    }
    return tempresult;
  }

  /**
   * 
   * @return Return true if postcode is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validatePostcode() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + POSTCODE;
    boolean tempresult = true;

    tempresult = validator.validateMandatory(events, parentCategory, fieldname, getPostcode());
    return tempresult && validator.validatePattern(events, parentCategory, fieldname, getPostcode(), "[\\p{N}]{4}");

  }

  /**
   * 
   * @return Return true if siteDPID is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateSiteDPID() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + SITE_DPID;
    boolean tempresult = true;

    if (getSiteDPID() != null && !getSiteDPID().equals("")) {
      tempresult = validator.validateNumeric(events, parentCategory, fieldname, getSiteDPID());
      if (!tempresult) return false;
      if (Integer.parseInt(getSiteDPID()) < 10000000) {
        String tempSiteDPID = new Integer(Integer.parseInt(getSiteDPID())).toString();
        return tempresult && validator.validateLength(events, parentCategory, fieldname, tempSiteDPID, 8);
      }
      return tempresult && validator.validateLength(events, parentCategory, fieldname, getSiteDPID(), 8);
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if state is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateState() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + STATE;
    boolean tempresult = true;

    tempresult = validator.validateMandatory(events, parentCategory, fieldname, getState());
    return tempresult && validator.validateState(events, parentCategory, fieldname, getState());

  }

  /**
   * 
   * @return Return true if streetName1 is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateStreetName1() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + STREET_NAME1;
    boolean tempresult = true;
    if ((getStreetType1() != null && !getStreetType1().equals("")) || (getStreetSuffix1() != null && !getStreetSuffix1().equals(""))) {
      tempresult = validator.validateMandatory(events, parentCategory, fieldname, getStreetName1());
      return tempresult && validator.validatePattern(events, parentCategory, fieldname, getStreetName1(), "[\\p{L}\\p{N}\\s\\-']{1,30}");
    } else if (getStreetName1() != null && !getStreetName1().equals("")) {
      return validator.validatePattern(events, parentCategory, fieldname, getStreetName1(), "[\\p{L}\\p{N}\\s\\-']{1,30}");
    }
    return tempresult;
  }

  /**
   * 
   * @return Return true if streetName2 is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateStreetName2() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + STREET_NAME2;
    boolean tempresult = true;
    if ((getStreetType2() != null && !getStreetType2().equals("")) || (getStreetSuffix2() != null && !getStreetSuffix2().equals(""))) {
      tempresult = validator.validateMandatory(events, parentCategory, fieldname, getStreetName2());
      return tempresult && validator.validatePattern(events, parentCategory, fieldname, getStreetName2(), "[\\p{L}\\p{N}\\s\\-']{1,30}");
    } else if (getStreetName2() != null && !getStreetName2().equals("")) {
      return validator.validatePattern(events, parentCategory, fieldname, getStreetName2(), "[\\p{L}\\p{N}\\s\\-']{1,30}");
    }
    return tempresult;
  }

  /**
   * 
   * @return Return true if streetSuffix1 is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateStreetSuffix1() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + STREET_SUFFIX1;
    boolean tempresult = true;
    if (getStreetSuffix1() != null && !getStreetSuffix1().equals("")) {
      return validator.validateStreetSuffix(events, parentCategory, fieldname, getStreetSuffix1());
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if streetSuffix2 is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateStreetSuffix2() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + STREET_SUFFIX2;
    boolean tempresult = true;
    if (getStreetSuffix2() != null && !getStreetSuffix2().equals("")) {
      return validator.validateStreetSuffix(events, parentCategory, fieldname, getStreetSuffix2());
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if streetType1 is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateStreetType1() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + STREET_TYPE1;
    boolean tempresult = true;
    if (getStreetType1() != null && !getStreetType1().equals("")) {
      return validator.validateStreetType(events, parentCategory, fieldname, getStreetType1());
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if streetType2 is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateStreetType2() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + STREET_TYPE2;
    boolean tempresult = true;
    if (getStreetType2() != null && !getStreetType2().equals("")) {
      return validator.validateStreetType(events, parentCategory, fieldname, getStreetType2());
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if unstructuredAddress1 is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateUnstructuredAddress1() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + UNSTRUCTURED_ADDRESS1;
    boolean tempresult = true;

    if (!structuredAddressExists()) {
      tempresult = validator.validateMandatory(events, parentCategory, fieldname, getUnstructuredAddress1());
      return tempresult && validator.validatePattern(events, parentCategory, fieldname, getUnstructuredAddress1(), "[\\p{L}\\p{N}\\p{P}\\s]{1,80}");
    }
    return tempresult;
  }
  
  /**
   * 
   * @return Return true if unstructuredAddress2 is valid
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateUnstructuredAddress2() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + UNSTRUCTURED_ADDRESS2;
    boolean tempresult = true;

    if (getUnstructuredAddress2() != null && !getUnstructuredAddress2().equals("")) {
      return validator.validatePattern(events, parentCategory, fieldname, getUnstructuredAddress2(), "[\\p{L}\\p{N}\\p{P}\\s]{1,80}");
    }
    return tempresult;

  }
  
  /**
   * 
   * @return Return true if unstructuredAddress3
   * @throws UnableToLookupResourceException 
   */
  protected boolean validateUnstructuredAddress3() throws UnableToLookupResourceException {
    String fieldname = parentField + "." + UNSTRUCTURED_ADDRESS3;
    boolean tempresult = true;

    if (getUnstructuredAddress3() != null && !getUnstructuredAddress3().equals("")) {
      return validator.validatePattern(events, parentCategory, fieldname, getUnstructuredAddress3(), "[\\p{L}\\p{N}\\p{P}\\s]{1,80}");
    }
    return tempresult;
  }
  
  protected boolean validateStructuredUnstructured() throws UnableToLookupResourceException 
  {
    String fieldname = parentField + "." + UNSTRUCTURED_ADDRESS1;
    if((structuredAddressExists() && unstructuredAddressExists()))
    {      
      Object[] args = { "Structured or Unstructured Address" };
      validator.addEvent(events, parentCategory, fieldname, args, EventCode.CODE_0202, "invalidChoiceMandatory");
      return false;      
    }
    return true;
  }
}

