/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/CustDetailsRequestBasicValidator.java-arc   1.4   11 Apr 2005 11:11:10   RayanS  $
 * $Revision:   1.4  $
 * $Date:   11 Apr 2005 11:11:10  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * 
 * Date Changed   Changed By     Comments
 * ------------   ----------     -----------
 * 17-Jul-2013    Eddie Graham   Defect 2168. Modify method validateCommentLine1() to make field mandatory if Reason field is set to Data Quality Issue.
 */
package com.capgemini.nemmco.b2bb.validators;

/**
 * The class <code>CustDetailsRequestBasicValidator</code> is the Basic Validator 
 * object of the Transaction.
 *
 * @author  $Author:   RayanS  $
 * @version $Revision:   1.4  $
 */




import com.capgemini.nemmco.b2bb.CustDetailsRequest;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.enums.Reason;
import com.capgemini.nemmco.b2bb.exceptions.TransactionNotSetException;

public class CustDetailsRequestBasicValidator extends Validator
{
  protected CustDetailsRequest transaction;

  /**
   * Constructor. Sets a transaction with the objects initialisation.
   * @param transaction Transaction to use in the validator.
   */
  public CustDetailsRequestBasicValidator(Transaction transaction) {
    super(transaction);
    this.transaction = (CustDetailsRequest) transaction;
  }

  /**
   * Validate the Transaction
   * 
   * @return boolean result of validation
   * validation is called.
   */
  public boolean validate() {
    boolean result = true;

    result &= validateTransactionID() 
      & validateTransactionDate()
      & validateNMI()
      & validateNMIChecksum()
      & validateReason()
      & validateCommentLine1()
      & validateCommentLine2()
      & validateCommentLine3();

    return result;
  }
  
  protected boolean validateNMI() {
    String field = CustDetailsRequest.NMI_FIELD;
    return validator.validateMandatory(events, category, field, transaction.getNMI())
        && validator.validateNMI(events, category, field, transaction.getNMI(), transaction.getNMIChecksum());
  }
  
  protected boolean validateNMIChecksum() {
    return super.validateNMIChecksum();
  }
  
  protected boolean validateReason() 
  {
    String field = CustDetailsRequest.REASON;
    return validator.validateMandatory(events, category, field, transaction.getReason())
       && validator.validateEnumeratedValue(events, category, field, "Reason", Reason.REASONS, transaction.getReason());
  }
  
  protected boolean validateCommentLine1()
  {
    String field = CustDetailsRequest.COMMENT_LINE1;

    boolean result = true;

    if (Reason.OTHER.getCode().equals(transaction.getReason()) ||
        Reason.DATA_QUALITY_ISSUE.getCode().equals(transaction.getReason()))
    {
      result &= validator.validateMandatory(events, category, field, transaction.getCommentLine1());
    }
    result &= validator.validateLengthRange(events, category, field, transaction.getCommentLine1(), 0, 80);

    return result;
  }

  protected boolean validateCommentLine2()
  {
    String field = CustDetailsRequest.COMMENT_LINE2;
    return validator.validateLengthRange(events, category, field, transaction.getCommentLine2(), 0, 80);
  }

  protected boolean validateCommentLine3()
  {
    String field = CustDetailsRequest.COMMENT_LINE3;
    return validator.validateLengthRange(events, category, field, transaction.getCommentLine3(), 0, 80);
  }
}

