/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/MeterDataNotificationBasicValidator.java-arc   1.5   11 Apr 2005 11:11:10   RayanS  $
 * $Revision:   1.5  $
 * $Date:   11 Apr 2005 11:11:10  $
 *
 * Copyright 2005 Capgemini Australia Pty. Ltd.
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 */
package com.capgemini.nemmco.b2bb.validators;

/**
 * The class <code>MeterDataNotificationBasicValidator</code> is the Basic Validator
 * object of the Transaction.
 *
 * @author  $Author:   RayanS  $
 * @version $Revision:   1.5  $
 */
import com.capgemini.nemmco.b2bb.MeterDataNotification;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.exceptions.TransactionNotSetException;

public class MeterDataNotificationBasicValidator extends Validator
{
  protected MeterDataNotification transaction;

  /**
   * Constructor. Sets a transaction with the objects initialisation.
   * @param transaction Transaction to use in the validator.
   */
  public MeterDataNotificationBasicValidator(Transaction transaction) {
    super(transaction);
    this.transaction = (MeterDataNotification) transaction;
  }

  /**
   * Validate the Transaction
   *
   * @return boolean result of validation
   * validation is called.
   */
  public boolean validate() {
    boolean result = true;

    result &= validateTransactionID()
        & validateInitiatingTransactionID()
        & validateTransactionDate()
        & validateCSVData()
        & validateParticipantRole();

    return result;
  }

  protected boolean validateInitiatingTransactionID()
  {
    String field = Transaction.INIT_TRANSACTION_ID;
    return validator.validateTransactionId(events, category, field, transaction.getInitiatingTransactionID());
  }

  protected boolean validateCSVData()
  {
    String csvConsumptionDataField = MeterDataNotification.CSV_CONSUMPTION_DATA;
    String csvIntervalDataField = MeterDataNotification.CSV_INTERVAL_DATA;
    String csvConsumptionData = transaction.getCSVConsumptionData();
    String csvIntervalData = transaction.getCSVIntervalData();

    boolean result = true;

    result &= validator.validateChoice(events, category, new String[] {csvConsumptionDataField, csvIntervalDataField}, new String[] {csvConsumptionData, csvIntervalData}, true);
    return result;
  }

  protected boolean validateParticipantRole()
  {
    String field = MeterDataNotification.PARTICIPANT_ROLE;
    return validator.validateMandatory(events, category, field, transaction.getParticipantRole())
    		&& validator.validateLengthRange(events, category, field, transaction.getParticipantRole(), 1, 4); 
  }
}

