/*
 * MeterFaultAndIssueNotificationBasicValidator.java
 *
 * The Copyright of this work is vested in the Australian Energy Market
 * Operator (AEMO) and the software is issued in confidence for the purpose
 * only for which it is supplied. It must not be reproduced in whole or in
 * part or used for tendering or manufacturing purposes except under an
 * agreement or with the consent in writing of AEMO and then only on the
 * condition that this notice is included in any such reproduction.
 * No information as to the contents or subject matter of this software or
 * any part thereof arising directly or indirectly there from shall be
 * given orally or in writing or communicated in any manner whatsoever to
 * any third party being an individual firm or company or any employee
 * thereof without the prior consent in writing of AEMO.
 *
 * Copyright Australian Energy Market Operator Limited. 2017
 *
 * Modification History
 *
 * Date Changed   Changed By        Description
 * ------------   ----------        -----------
 * 10-Jul-2017    Harpreet Kaur     Created
 * 31-Aug-2017    Harpreet Kaur     7135 Package 7B: Defect#7844 fix
 * 11-Sept-2017    Harpreet Kaur     7135 Package 7B: Defect#7844 fix
 * 28-Sept-2017    Harpreet Kaur     7135 Package 7B: Defect#7482 fix
 * 17-Oct-2017    Hamish McNeish     8159 B2B Errata make NMI checksum optional
 */

package com.capgemini.nemmco.b2bb.validators;

import com.capgemini.nemmco.b2bb.MeterFaultAndIssueNotification;
import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.enums.EventCode;
import com.capgemini.nemmco.b2bb.enums.MeterFaultAndIssueReasons;
import com.capgemini.nemmco.b2bb.enums.MeterFaultAndIssueSupplyOffOptions;
import com.capgemini.nemmco.b2bb.enums.MeterInstallCode;
import com.capgemini.nemmco.b2bb.enums.YesNo;
import com.capgemini.nemmco.b2bb.utils.B2BDateFormatter;

import java.util.Date;

/**
 * The class <code>MeterFaultAndIssueNotificationBasicValidator</code> is the Basic Validator
 * object of the Transaction.
 *
 * @author $Author:   HarpreetK  $
 * @version $Revision:   1.0  $ $Date:    07 Jul 2017 11:11:10  $
 */
public class MeterFaultAndIssueNotificationBasicValidator extends Validator {
   protected MeterFaultAndIssueNotification transaction;
   private String transactionType = "";
   /**
    * Constructor. Sets a transaction with the objects initialisation.
    * @param transaction Transaction to use in the validator.
    */
  public MeterFaultAndIssueNotificationBasicValidator(Transaction transaction) {
          super(transaction);
          this.transaction = (MeterFaultAndIssueNotification) transaction;
          transactionType = transaction.getTransactionType().getCode();
        }

   /**
    * Validate the Transaction
    *
    * @return boolean result of validation
    * validation is called.
    */
   public boolean validate() {
     boolean result = true;

     result &=  validateTransactionID()
         & validateTransactionDate()
         & validateNMI()
         & validateNMIChecksum()
         & validateDate()
         & validateStartDate()
         & validateStartTime()
         & validateEndDate()
         & validateDuration()
         & validateSupplyOn()
         & validateSupplyOff()
         & validateMeterSerialNumber1()
         & validateMeterSerialNumber2()
         & validateMeterSerialNumber3()
         & validateMeterSerialNumber4()
         & validateReasonForNotice()
         & validateNotes();

     return result;
   }


   /**
    * Validate date
    * @return boolean
    */
   protected boolean validateDate() {

      String fieldname = MeterFaultAndIssueNotification.DATE;

        if (transaction.isDateSet()) {

           return validator.validateMandatory(events, category, fieldname, transaction.getDate()) &&
                  validator.validateXMLDate(events, category, fieldname, transaction.getDate());
        } else {
           boolean tempresult = validator.validateMandatory(events, category, fieldname, transaction.getDisplayDate());
           tempresult = tempresult && validator.validateB2BDate(events, category, fieldname, transaction.getDisplayDate());

           return tempresult;
        }

   }


   /**
    * Validate validateStartDate
    * @return Returns true if startDate is valid
    */
   protected boolean validateStartDate() {
      boolean result;
      String fieldname = MeterFaultAndIssueNotification.START_DATE;

      result = true;
      if(validator.exists(transaction.getStartDate()) || validator.exists(transaction.getStartDisplayDate()))
      {
         if (transaction.isStartDateSet()) {
            result = validator.validateXMLDate(events, category, fieldname, transaction.getStartDate());
         } else {
            result =  validator.validateB2BDate(events, category, fieldname, transaction.getStartDisplayDate());
         }
      }
      return result;
   }

   /**
    * Validate validateEndDate
    * @return Returns true if endDate is valid
    */
   protected boolean validateEndDate() {
      String fieldname = MeterFaultAndIssueNotification.END_DATE;

      if (transaction.isEndDateSet()) {
         if (transaction.getEndDate() != null && !transaction.getEndDate().equals("")) {
            try {
               if (transaction.getStartDate() != null && !transaction.getStartDate().equals("")) {
                  B2BDateFormatter df = new B2BDateFormatter(transaction.getEndDate(), B2BDateFormatter.XMLDATE);

                  Date endDate = df.getDate();
                  df = new B2BDateFormatter(transaction.getStartDate(), B2BDateFormatter.XMLDATE);
                  Date startDate = df.getDate();
                  if (startDate.after(endDate)) {
                     Object[] args = { "End Date" };
                     validator.addEvent(events, transactionType, fieldname, args, EventCode.CODE_0202, "invalidEndDateRange");
                     return false;
                  }
               }
            } catch (Exception e) {}
            return validator.validateXMLDate(events, category, fieldname, transaction.getEndDate());
         }
         return true;
      } else {

         if (transaction.getEndDisplayDate() != null && !transaction.getEndDisplayDate().equals("")) {
            try {
               if (transaction.getStartDisplayDate() != null && !transaction.getStartDisplayDate().equals("")) {
                  B2BDateFormatter df =
                     new B2BDateFormatter(transaction.getEndDisplayDate(), B2BDateFormatter.B2BDATE);

                  Date endDate = df.getDate();
                  df = new B2BDateFormatter(transaction.getStartDisplayDate(), B2BDateFormatter.B2BDATE);
                  Date startDate = df.getDate();
                  if (startDate.after(endDate)) {
                     Object[] args = { "End Date" };
                     validator.addEvent(events, transactionType, fieldname, args, EventCode.CODE_0202,"invalidEndDateRange");
                     return false;
                  }
               }
            } catch (Exception e) {}

            return validator.validateB2BDate(events, category, fieldname, transaction.getEndDisplayDate());
         }
         return true;
      }
   }

   /**
    * Validate start time
    * @return boolean
    */
   protected boolean validateStartTime() {
      boolean result;
      String fieldname = MeterFaultAndIssueNotification.START_TIME;

      result = true;
      if (validator.exists(transaction.getStartTime()) || validator.exists(transaction.getStartTime())) {
         if (transaction.getIsStartTimeSet()) {
            result = validator.validateXMLTime(events, category, fieldname, transaction.getStartTime());
         } else {
            if (transaction.getStartDisplayTime() != null && !transaction.getStartDisplayTime().equals(""))
               result = validator.validateB2BTime(events, category, fieldname, transaction.getStartDisplayTime());
         }
      }
      return result;
   }

   /**
    * Validate duration
    * @return boolean
    */
   protected boolean validateDuration() {
      boolean result;
      String fieldname = MeterFaultAndIssueNotification.DURATION;

      result = true;
      if(validator.exists(transaction.getDuration())){
      result = validator.validateLengthRange(events, category, fieldname, transaction.getDuration(), 0, 5) &&
               validator.validateDurationInterval(events, category, fieldname, transaction.getDuration());
      }
      return result;
   }


   /**
    * Validate validateSupplyOn
    * @return boolean
    */
   protected boolean validateSupplyOn() {
      String field = MeterFaultAndIssueNotification.SUPPLY_ON;
      return  validator.validateMandatory(events, category, field, transaction.getSupplyOn())
      		&& validator.validateYesNo(events,category,field,transaction.getSupplyOn());
   }

   /**
    * Validate validateSupplyOff
    * @return boolean
    */
   protected boolean validateSupplyOff() {
      String field = MeterFaultAndIssueNotification.SUPPLY_OFF;
      String value = transaction.getSupplyOff();
    	//Mandatory when SupplyOn is No
      if (YesNo.NO.getCode().equals(transaction.getSupplyOn())) {
      	if (!validator.validateMandatory(events,category,field,value))
      			return false;
      }
      //Optional
      return !validator.exists(value)
      		|| validator.validateEnumeratedValue(events,
      				category,
              field,
              "Supply Off",
              MeterFaultAndIssueSupplyOffOptions.METER_FAULT_AND_ISSUE_SUPPLY_OFF_OPTIONS,
              value);
   }

   /**
    * Validate validateMeterSerialNumber1
    * @return boolean
    */
   protected boolean validateMeterSerialNumber1() {
     String field = MeterFaultAndIssueNotification.METER_SERIAL_NUMBER1;
     return validator.validateLengthRange(events, category, field, transaction.getMeterSerialNumber1(), 0, 12);
   }


   /**
    * Validate validateMeterSerialNumber2
    * @return boolean
    */
   protected boolean validateMeterSerialNumber2() {
     String field = MeterFaultAndIssueNotification.METER_SERIAL_NUMBER2;
     return validator.validateLengthRange(events, category, field, transaction.getMeterSerialNumber2(), 0, 12);
   }


   /**
    * Validate validateMeterSerialNumber3
    * @return boolean
    */
   protected boolean validateMeterSerialNumber3() {
     String field = MeterFaultAndIssueNotification.METER_SERIAL_NUMBER3;
     return validator.validateLengthRange(events, category, field, transaction.getMeterSerialNumber3(), 0, 12);
   }


   /**
    * Validate validateMeterSerialNumber4
    * @return boolean
    */
   protected boolean validateMeterSerialNumber4() {
     String field = MeterFaultAndIssueNotification.METER_SERIAL_NUMBER4;
     return validator.validateLengthRange(events, category, field, transaction.getMeterSerialNumber4(), 0, 12);
   }


   /**
    * Validate validateReasonForNotice
    * @return boolean
    */
   protected boolean validateReasonForNotice() {
      String field = MeterFaultAndIssueNotification.REASON_FOR_NOTICE;
      return  validator.validateMandatory(events, category, field, transaction.getReasonForNotice())
      		&& validator.validateEnumeratedValue(events,
      				category,
              field,
              "Reason For Notice",
              MeterFaultAndIssueReasons.METER_FAULT_AND_ISSUE_REASONS,
              transaction.getReasonForNotice());
   }

   /**
    * Validate validateNotes
    * @return boolean
    */
   protected boolean validateNotes() {
      String fieldname = MeterFaultAndIssueNotification.NOTES;
      String value = transaction.getNotes();
      String conditionalFieldname = MeterFaultAndIssueNotification.REASON_FOR_NOTICE;
      String conditionalFieldValue = transaction.getReasonForNotice();
	  
	  //Mandatory when Reason = "Other"
      return validator.validateMandatoryConditionally1(events, category, fieldname, value, conditionalFieldname, conditionalFieldValue, MeterFaultAndIssueReasons.OTHER.getCode())
	   && validator.validateLengthRange(events, category, fieldname, value, 0, 240);
   }

}
