/**
* $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/RequestData.java-arc   1.0   Sep 15 2005 15:17:48   adij  $
* $Revision:   1.0  $
* $Date:   Sep 15 2005 15:17:48  $
*
* Copyright 2005 Capgemini Australia Pty. Ltd.
* All rights reserved.
* NEMMCO CONFIDENTIAL.
 *
 * Modification History
 *
 * Date Changed   Changed By        Description
 * ------------   ----------        -----------
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb.validators;

import java.util.ArrayList;
import java.util.List;

import com.capgemini.nemmco.b2bb.XMLObject;
import com.capgemini.nemmco.b2bb.ServiceOrderRequest;
import com.capgemini.nemmco.b2bb.enums.XMLObjectType;
import com.capgemini.nemmco.b2bb.enums.TrueFalse;
import com.capgemini.nemmco.b2bb.utils.BasicValidationUtility;
import com.capgemini.nemmco.b2bb.utils.EventObject;

/**
 * Defines RequestData
 *
 * @author $Author:   adij  $
 * @version $Revision:   1.0  $ $Date:   Sep 15 2005 15:17:48  $
 */
public class RequestData extends XMLObject implements ValidatorInterface {

    public static String CUST_CONSULTATION_REQUIRED = "customerConsultationRequired";
    public static String SPECIAL_INSTRUCTIONS1 = "specialInstructions1";
    public static String SPECIAL_INSTRUCTIONS2 = "specialInstructions2";
    public static String SPECIAL_INSTRUCTIONS3 = "specialInstructions3";

    private List<EventObject> events = new ArrayList<EventObject>();
    private String parentField = "";
    private String parentCategory;

    private String customerConsultationRequired="";
    private String specialInstructions1="";
    private String specialInstructions2="";
    private String specialInstructions3="";
    private boolean specialInstructionRequired = false;

    private BasicValidationUtility validator;

    /**
     * Default constructor
     */
    public RequestData() {
        super(XMLObjectType.REQUESTDATA);
        validator = BasicValidationUtility.getBasicInstance();
    }


    /**
     * Constructor
     * @param xmlObjectType The xmlObjectType to set
     */
    public RequestData(XMLObjectType xmlObjectType) {
        super(xmlObjectType);
        validator = BasicValidationUtility.getBasicInstance();
    }


    public boolean fieldsExists() {
        return validator.exists(customerConsultationRequired)
                || validator.exists(specialInstructions1)
                || validator.exists(specialInstructions2)
                || validator.exists(specialInstructions3);
    }


    /**
     * @return Returns the yesNo list
     */
    public List<? extends Object> getYesNo() {
        return TrueFalse.TRUEFALSE_CODES;
    }

    /**
     * @return Returns the specialInstructions.
     */
    public String getSpecialInstructions1() {
        return this.specialInstructions1;
    }
    /**
     * @param specialInstructions1 The specialInstructions to set.
     */
    public void setSpecialInstructions1(String specialInstructions1) {
        this.specialInstructions1 = specialInstructions1;
    }
    /**
     * @return Returns the specialInstructions.
     */
    public String getSpecialInstructions2() {
        return this.specialInstructions2;
    }
    /**
     * @param specialInstructions2 The specialInstructions to set.
     */
    public void setSpecialInstructions2(String specialInstructions2) {
        this.specialInstructions2 = specialInstructions2;
    }
    /**
     * @return Returns the specialInstructions.
     */
    public String getSpecialInstructions3() {
        return this.specialInstructions3;
    }
    /**
     * @param specialInstructions3 The specialInstructions to set.
     */
    public void setSpecialInstructions3(String specialInstructions3) {
        this.specialInstructions3 = specialInstructions3;
    }
    /**
     * @return Returns the customerConsultationRequired.
     */
    public String getCustomerConsultationRequired() {
        return customerConsultationRequired;
    }
    /**
     * @param customerConsultationRequired The customerConsultationRequired to set.
     */
    public void setCustomerConsultationRequired(
            String customerConsultationRequired) {
        this.customerConsultationRequired = customerConsultationRequired;
    }

    /**
     * @see com.capgemini.nemmco.b2bb.validators.ValidatorInterface#getEvents()
     */
    public List<EventObject> getEvents() {
        return events;
    }


    /**
     * @param event
     */
    public void setEvents(List<EventObject> event) {
        events = event;
    }

    /**
     * @return Returns the parentCategory.
     */
    public String getParentCategory() {
        return parentCategory;
    }
    /**
     * @param parentCategory The parentCategory to set.
     */
    public void setParentCategory(String parentCategory) {
        this.parentCategory = parentCategory;
    }

    /**
     * @return Returns the parentField.
     */
    public String getParentField() {
        return parentField;
    }

    /**
     * @param parentField The parentField to set.
     */
    public void setParentField(String parentField) {
        this.parentField = parentField;
    }


  /**
   *
   * @return Return true if valid, false otherwise
   */
    public boolean validate() {
        boolean result = true;
        result = validateCustomerConsultationRequired() && result;
        // always validate SpecialInstructions last as they are dependant on
        // other validations
        result = validateSpecialInstructions1() && result;
        result = validateSpecialInstructions2() && result;
        result = validateSpecialInstructions3() && result;
        return result;
    }


  /**
   *
   * @return Return true if customerConsultationRequired is valid
   */
    protected boolean validateCustomerConsultationRequired() {
        String fieldname = ServiceOrderRequest.CUST_CONSULTATION_REQUIRED;
        boolean tmpresult = true;
        if (validator.exists(getSpecialInstructions1())) {
            tmpresult = validator.validateMandatory(events,parentCategory,fieldname,getCustomerConsultationRequired());
        } else if (!validator.exists(getCustomerConsultationRequired())) {
            return tmpresult;
        }
        tmpresult = tmpresult && validator.validateTrueFalse(events,parentCategory,fieldname,getCustomerConsultationRequired());
        // if Customer Consultation is TRUE then special instruction is mandatory
        if (tmpresult && getCustomerConsultationRequired().equals(TrueFalse.TRUE.getCode())){
            specialInstructionRequired = true;
        }
        return tmpresult;
    }

  /**
   *
   * @return Return true if specialInstructions1 is valid
   */
    protected boolean validateSpecialInstructions1() {
        String fieldname = ServiceOrderRequest.SPECIAL_INSTRUCTIONS1;
        boolean tmpresult = true;
        if (specialInstructionRequired){
            tmpresult = tmpresult && validator.validateMandatory(events,parentCategory,fieldname,getSpecialInstructions1());
        }
        return tmpresult && validator.validateLengthRange(
                events,
                parentCategory,
                fieldname,
                getSpecialInstructions1(),
                0,80);
    }

  /**
   *
   * @return Return true if specialInstructions2 is valid
   */
    protected boolean validateSpecialInstructions2() {
        String fieldname = ServiceOrderRequest.SPECIAL_INSTRUCTIONS2;
        return validator.validateLengthRange(events,
                parentCategory,
                fieldname,
                getSpecialInstructions2(),
                0,80);
    }

  /**
   *
   * @return Return true if specialInstructions3 is valid
   */
    protected boolean validateSpecialInstructions3() {
        String fieldname = ServiceOrderRequest.SPECIAL_INSTRUCTIONS3;
        return validator.validateLengthRange(events,
                parentCategory,
                fieldname,
                getSpecialInstructions3(),
                0,80);
    }
}

