/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/Telephone.java-arc   1.12   Mar 21 2005 08:55:18   adij  $
 * $Revision:   1.12  $
 * $Date:   Mar 21 2005 08:55:18  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 *
 * Modification History
 *
 * Date Changed   Changed By        Description
 * ------------   ----------        -----------
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb.validators;

import java.util.ArrayList;
import java.util.List;

import com.capgemini.nemmco.b2bb.XMLObject;
import com.capgemini.nemmco.b2bb.enums.PhoneServiceComment;
import com.capgemini.nemmco.b2bb.enums.PhoneServiceType;
import com.capgemini.nemmco.b2bb.enums.XMLObjectType;
import com.capgemini.nemmco.b2bb.utils.BasicValidationUtility;
import com.capgemini.nemmco.b2bb.utils.EventObject;

/**
 * Defines Telephone
 *
 * @author $Author:   adij  $
 * @version $Revision:   1.12  $ $Date:   Mar 21 2005 08:55:18  $
 */
public class Telephone extends XMLObject implements ValidatorInterface {
  public static final String NUMBER = "number";
  public static final String PREFIX = "prefix";
  public static final String SERVICE_COMMENT = "serviceComment";
  public static final String SERVICE_TYPE = "serviceType";

  private List<EventObject> events = new ArrayList<EventObject>();
  
  private String number = "";
  private String parentCategory;
  private String parentField;
  private String prefix = "";
  private String serviceComment = "";
  private String serviceType = "";
  
  private BasicValidationUtility validator;

  public Telephone() {
    super(XMLObjectType.TELEPHONE);
    validator = BasicValidationUtility.getBasicInstance();
  }
  
  public boolean fieldsExists() {
    return validator.exists(number)
        || validator.exists(prefix)
        || validator.exists(serviceComment)
        || validator.exists(serviceType);
  }

  /**
   * @see com.capgemini.nemmco.b2bb.validators.ValidatorInterface#getEvents()
   */
  public List<EventObject> getEvents() {
    return events;
  }
  
  /**
   * @return Returns the number.
   */
  public String getNumber() {
    return number;
  }

  /**
   * @return Returns the parentCategory.
   */
  public String getParentCategory() {
    return parentCategory;
  }
  
  /**
   * @return Returns the parentField.
   */
  public String getParentField() {
    return parentField;
  }
  
  /**
   * @return
   */
  public List<? extends Object> getPhoneServiceComments()
  {
    return PhoneServiceComment.PHONE_SERVICE_COMMENTS;
  }

  /**
   * @return
   */
  public List<? extends Object> getPhoneServiceTypes()
  {
    return PhoneServiceType.PHONE_SERVICE_TYPES;
  }

  /**
   * @return Returns the prefix.
   */
  public String getPrefix() {
    return prefix;
  }

  /**
   * @return Returns the serviceComment.
   */
  public String getServiceComment() {
    return serviceComment;
  }

  /**
   * @return Returns the serviceType.
   */
  public String getServiceType() {
    return serviceType;
  }

  /**
   * @param event 
   */
  public void setEvents(List<EventObject> event) {
    events = event;
  }
  
  /**
   * @param number The number to set.
   */
  public void setNumber(String number) {
    this.number = number != null?number.trim():null;
  }
  
  /**
   * @param parentCategory The parentCategory to set.
   */
  public void setParentCategory(String parentCategory) {
    this.parentCategory = parentCategory;
  }
  
  /**
   * @param parentField The parentField to set.
   */
  public void setParentField(String parentField) {
    this.parentField = parentField;
  }
  
  /**
   * @param prefix The prefix to set.
   */
  public void setPrefix(String prefix) {
    this.prefix = prefix != null?prefix.trim():null;
  }
  
  /**
   * @param serviceComment The serviceComment to set.
   */
  public void setServiceComment(String serviceComment) {
    this.serviceComment = serviceComment != null?serviceComment.trim():null;
  }
  
  /**
   * @param serviceType The serviceType to set.
   */
  public void setServiceType(String serviceType) {
    this.serviceType = serviceType != null?serviceType.trim():null;
  }

  /**
   * @return
   */
  public boolean validate() {
    return validateNumber()
        & validatePrefix()
        & validateServiceComment()
        & validateServiceType();
  }

  protected boolean validateNumber() {
    String fieldname = parentField + "." + NUMBER;
    boolean tmpresult = validator.validateMandatory(events, parentCategory, fieldname, getNumber());
    return tmpresult && validator.validateLengthRange(events, parentCategory, fieldname, getNumber(), 0, 15);
  }

  protected boolean validatePrefix() {
    String fieldname = parentField + "." + PREFIX;
    boolean tmpresult = validator.validateMandatory(events, parentCategory, fieldname, getPrefix());
    return tmpresult && validator.validateLengthRange(events, parentCategory, fieldname, getPrefix(), 0, 4);
  }

  protected boolean validateServiceComment() {
    String fieldname = parentField + "." + SERVICE_COMMENT;
    
    if(getServiceComment()!=null && !getServiceComment().equals(""))
    {
      return validator.validatePhoneServiceComment(events, parentCategory, fieldname, getServiceComment());  
    }
    return true;
  }

  protected boolean validateServiceType() {
    String fieldname = parentField + "." + SERVICE_TYPE;
    boolean tmpresult = validator.validateMandatory(events, parentCategory, fieldname, getServiceType());
    return tmpresult && validator.validatePhoneServiceType(events, parentCategory, fieldname, getServiceType());
  }
}

