/*
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/Validator.java-arc   1.10   11 Apr 2005 11:11:10   RayanS  $
 * $Revision:   1.10  $
 * $Date:   11 Apr 2005 11:11:10  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 *
 * Modification History
 *
 * Date Changed   Changed By        Comments
 * ------------   ----------        -----------
 * 27-Jun-2017    Harpreet Kaur     7135 Package 7B:Add new method validateMandatoryNMIChecksum
 * 11-Sept-2017    Harpreet Kaur     Defect fix 7482 as part of POC SMP.
 */

package com.capgemini.nemmco.b2bb.validators;

import com.capgemini.nemmco.b2bb.Transaction;

/**
 * The class <code>Validator</code> is the parent Validator object for all
 * transaction validators.
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.10  $ $Date:   11 Apr 2005 11:11:10  $
 */
public abstract class Validator extends XMLObjectValidator {
  private final Transaction transaction;

  /**
   * Constructor
   * @param transaction Transaction for this validator
   */
  public Validator(Transaction transaction) {
    super(transaction);
    this.transaction = transaction;
  }

  protected boolean validateTransactionID() {
    String field = Transaction.TRANSACTION_ID;
    return validator.validateMandatory(events, category, field, transaction.getTransactionID())
        && validator.validateTransactionId(events, category, field, transaction.getTransactionID())
        && validator.validateId(events, category, field, transaction.getTransactionID());
  }

  protected boolean validateTransactionDate() {
    String field = Transaction.TRANSACTION_DATE;
    if (transaction.isTransactionDateSet()) {
      return validator.validateMandatory(events, category, field, transaction.getTransactionDate())
          & validator.validateXMLDateTime(events, category, field, transaction.getTransactionDate());
    } else {
      return validator.validateMandatory(events, category, field, transaction.getTransactionDisplayDate())
          & validator.validateB2BDateTime(events, category, field, transaction.getTransactionDisplayDate());
    }
  }
  
  protected boolean validateNMI() {
    String field = Transaction.NMI_FIELD;
    return validator.validateMandatory(events, category, field, transaction.getNMI())
        && validator.validateNMI(events, category, field, transaction.getNMI(), transaction.getNMIChecksum());
  }
  
  protected boolean validateNMIChecksum() {
    String fieldname = Transaction.NMI_CHECKSUM;
    // if both fields dont exist return true
    if (!validator.exists(transaction.getNMI()) && !validator.exists(transaction.getNMIChecksum())) return true;
    // if NMI checksum doest exist return true
    if (!validator.exists(transaction.getNMIChecksum())) return true;
    // Checksum can only exist if a nmi exists
    if (!validator.validateDependancy(events,category,fieldname,Transaction.NMI_FIELD,transaction.getNMI(),transaction.getNMIChecksum())) return false;
    return validator.validateNMIChecksum(events, category, fieldname, transaction.getNMI(), transaction.getNMIChecksum());
  }

   protected boolean validateMandatoryNMIChecksum() {
     String fieldname = Transaction.NMI_CHECKSUM;
     // if NMI checksum doest exist return false as its mandatory field
     if (!validator.exists(transaction.getNMIChecksum()))
        return validator.validateMandatory(events, category, fieldname, transaction.getNMIChecksum());
      // if both fields dont exist return true
      if (!validator.exists(transaction.getNMI()) && !validator.exists(transaction.getNMIChecksum())) return true;
     // Checksum can only exist if a nmi exists
     if (!validator.validateDependancy(events,category,fieldname,Transaction.NMI_FIELD,transaction.getNMI(),transaction.getNMIChecksum())) return false;
     return validator.validateNMIChecksum(events, category, fieldname, transaction.getNMI(), transaction.getNMIChecksum());
   }

}

