/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/ValidatorFactory.java-arc   1.2   11 Apr 2005 11:11:14   RayanS  $
 * $Revision:   1.2  $
 * $Date:   11 Apr 2005 11:11:14  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * Date Changed   Changed By        Comments
 * ------------   ----------        -----------
 * 14-Aug-2012    Bob Jones         7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb.validators;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import com.capgemini.nemmco.b2bb.Transaction;
import com.capgemini.nemmco.b2bb.exceptions.UnableToInstantiateClassException;

/**
 * The class <code>ValidatorFactory</code> is used to create the Validation Objects.
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.2  $ $Date:   11 Apr 2005 11:11:14  $
 */
public class ValidatorFactory {

  /**
   * Returns a Basic validator object for a specified transaction
   * @param transaction
   * @return BasicValidator class
   * @throws UnableToInstantiateClassException if the class is not available
   */
  public static Validator getValidator(Transaction transaction) throws UnableToInstantiateClassException {
    Object[] arguments = new Object[] {
      transaction};
    return getValidatorClass((transaction.getTransactionType()).getBasicValidatorClass(), arguments);
  }

  /**
   * Returns a Extended validator object for a specified transaction
   * @param transaction
   * @return Extended Validator class
   * @throws UnableToInstantiateClassException if the class is not available
   */
  public static Validator getExtendedValidator(Transaction transaction) throws UnableToInstantiateClassException {
    Object[] arguments = new Object[] {
      transaction};
    return getValidatorClass((transaction.getTransactionType()).getExtendedValidatorClass(), arguments);
  }

  /**
   * implements reflection to instantiate a class for the above factory methods
   * @param className to instantiate
   * @param arguments to use in the constructor
   * @return the Validator class instantiated
   * @throws UnableToInstantiateClassException if the class is not available
   */
  static private Validator getValidatorClass(String className, Object[] arguments) throws UnableToInstantiateClassException {
    try {
      Class<?> tmpClass = Class.forName(className);
      Class<?>[] typeArray = new Class<?>[] {
        Transaction.class};
      Constructor<?> validatorConstructor = tmpClass.getConstructor(typeArray);
      return (Validator) validatorConstructor.newInstance(arguments);
    } catch (IllegalAccessException ex) {
      throw new UnableToInstantiateClassException(ex.getMessage());
    } catch (InstantiationException ex) {
      throw new UnableToInstantiateClassException(ex.getMessage());
    } catch (ClassNotFoundException ex) {
      throw new UnableToInstantiateClassException(ex.getMessage());
    } catch (NoSuchMethodException ex) {
      throw new UnableToInstantiateClassException(ex.getMessage());
    } catch (InvocationTargetException ex) {
      throw new UnableToInstantiateClassException(ex.getMessage());
    }
  }
}

