/**
 * $Header:   //cardsrc1/pvcs/B2B_BROWSER/archives/src/com/capgemini/nemmco/b2bb/validators/XMLObjectValidator.java-arc   1.2   11 Apr 2005 11:11:14   RayanS  $
 * $Revision:   1.2  $
 * $Date:   11 Apr 2005 11:11:14  $
 * 
 * Copyright 2005 Capgemini Australia Pty. Ltd. 
 * All rights reserved.
 * NEMMCO CONFIDENTIAL.
 * Date Changed   Changed By     Comments
 * ------------   ----------     -----------
 * 14-Aug-2012    Bob Jones      7984 Java upgrade from 6 to 7.
 */

package com.capgemini.nemmco.b2bb.validators;

import java.util.ArrayList;
import java.util.List;

import com.capgemini.nemmco.b2bb.Header;
import com.capgemini.nemmco.b2bb.XMLObject;
import com.capgemini.nemmco.b2bb.enums.XMLObjectType;
import com.capgemini.nemmco.b2bb.exceptions.XMLObjectNotSetException;
import com.capgemini.nemmco.b2bb.utils.BasicValidationUtility;
import com.capgemini.nemmco.b2bb.utils.EventObject;

/**
 * The class <code>Validator</code> is the parent Validator object for all
 * transaction validators.
 *
 * @author $Author:   RayanS  $
 * @version $Revision:   1.2  $ $Date:   11 Apr 2005 11:11:14  $
 */
public abstract class XMLObjectValidator implements ValidatorInterface {
  protected final BasicValidationUtility validator;
  protected List<EventObject> events = new ArrayList<EventObject>();
  protected final XMLObject xmlObject;
  protected final String category;
  protected Header header = null;

  /**
   * Constructor
   * @param xmlObject Transaction for this validator
   */
  public XMLObjectValidator(XMLObject xmlObject) {
    this.xmlObject = xmlObject;
    this.category = retrieveCategory(xmlObject.getXMLObjectType());
    this.validator = BasicValidationUtility.getBasicInstance();
  }

  /**
   * 
   * @param err An Event Object to add to the list of errors.
   */
  protected void addEvents(EventObject err) {
    events.add(err);
  }
  
  /**
   * 
   * @param The category to retrieve from
   */
  private String retrieveCategory(XMLObjectType xmlObjectType) {
    String transactionClass = xmlObjectType.getCategory();
    return transactionClass;
  }

  /**
   * 
   * @return a list of events which were encountered while validating
   * the Transaction. A Blank list is returned if no events are found. null
   * is returned if validate has not yet been called.
   */
  public List<EventObject> getEvents() {
    return events;
  }
  
  /**
   * Sets a header object for this transaction
   * 
   * @param header to set
   */
  public void setHeader(Header header) {
    this.header=header;
  }
  
  /**
   * 
   * @return the header object if one is set. Null otherwise.
   */
  public Header getHeader() {
    return header;
  }
  
  
  /**
   * Called to validate the XMLObject.
   * 
   */
  public abstract boolean validate() throws XMLObjectNotSetException;

}

